module mo_extfrc
  !---------------------------------------------------------------
  ! 	... insitu forcing module
  !---------------------------------------------------------------

  use shr_kind_mod, only : r8 => shr_kind_r8
  use ppgrid,       only : pcols, begchunk, endchunk, pver, pverp
  use chem_mods,    only : gas_pcnst, extcnt
  use spmd_utils,   only : masterproc,iam
  use abortutils,   only : endrun
  use cam_history,  only : addfld, outfld, phys_decomp, add_default
  use cam_logfile,  only : iulog
  use tracer_data,  only : trfld,trfile

  use h_setkin_par, only : NACT !Guangxing Lin
  implicit none

  type :: forcing
     integer           :: frc_ndx
     real(r8)              :: mw
     character(len=265) :: filename
     real(r8), pointer     :: times(:)
     real(r8), pointer     :: levi(:)
     character(len=8)  :: species
     character(len=8)  :: units
     integer                   :: nsectors
     character(len=32),pointer :: sectors(:)
     type(trfld), pointer      :: fields(:)
     type(trfile)              :: file
  end type forcing

  private
  public  :: extfrc_inti
  public  :: extfrc_set
  public  :: extfrc_timestep_init

  save

  integer, parameter :: time_span = 1

  character(len=256) ::   filename
  integer, parameter :: gas_pcnst_um = NACT   !Guangxing Lin

  logical :: has_extfrc(gas_pcnst)
  !added by Guangxing Lin
  logical :: has_extfrc_um(gas_pcnst_um)
  type(forcing), allocatable  :: forcings(:)
  type(forcing), allocatable  :: forcings_um(:)!Guangxing Lin 
  integer :: extfrc_cnt = 0
  integer :: extfrc_cnt_um = 0 !Guangxing Lin

contains

  subroutine extfrc_inti( extfrc_specifier, extfrc_type, extfrc_cycle_yr, extfrc_fixed_ymd, extfrc_fixed_tod)

    !-----------------------------------------------------------------------
    ! 	... initialize the surface forcings
    !-----------------------------------------------------------------------
    use cam_pio_utils, only : cam_pio_openfile
    use pio, only : pio_inq_dimid, pio_inquire, pio_inq_varndims, pio_closefile, &
         pio_inq_varname, pio_nowrite, file_desc_t
    use mo_tracname,   only : solsym
    use mo_chem_utls,  only : get_extfrc_ndx, get_spc_ndx
    use chem_mods,     only : frc_from_dataset
    use tracer_data,   only : trcdata_init
    use phys_control,  only : phys_getopts
    use physics_buffer, only : physics_buffer_desc

    use um_aerosol_intr, only: um_get_spc_ndx ! added by Guangxing Lin
    use h_setkin_lchem, only: lchemvar !Guangxing Lin
    implicit none

    !-----------------------------------------------------------------------
    ! 	... dummy arguments
    !-----------------------------------------------------------------------
    character(len=*), dimension(:), intent(in) :: extfrc_specifier
    character(len=*), intent(in) :: extfrc_type
    integer  , intent(in)        :: extfrc_cycle_yr
    integer  , intent(in)        :: extfrc_fixed_ymd
    integer  , intent(in)        :: extfrc_fixed_tod

    !-----------------------------------------------------------------------
    ! 	... local variables
    !-----------------------------------------------------------------------
    integer :: astat
    integer :: j, l, m, n, i,mm                          ! Indices
    character(len=16)  :: species
    character(len=16)  :: spc_name
    character(len=256) :: locfn
    character(len=256) :: spc_fnames(gas_pcnst)
    ! added by Guangxing Lin
    character(len=256) ::    extfrc_type_um
    integer          :: extfrc_cycle_yr_um
    character(len=256) :: spc_fnames_um(gas_pcnst_um)
    character(len=256) :: um_extfrc_specifier(gas_pcnst_um)


    integer ::  vid, ndims, nvars, isec, ierr
    type(file_desc_t) :: ncid
    character(len=32)  :: varname

    character(len=1), parameter :: filelist = ''
    character(len=1), parameter :: datapath = ''
    logical         , parameter :: rmv_file = .false.
    logical  :: history_aerosol      ! Output the MAM aerosol tendencies

    !-----------------------------------------------------------------------
 
    call phys_getopts( history_aerosol_out        = history_aerosol   )

    do i = 1, gas_pcnst
       has_extfrc(i) = .false.
       spc_fnames(i) = ''
    enddo

     has_extfrc_um(:) = .false.  !added by Guangxing Lin
     spc_fnames_um(:) = ''
    ! added by Guangxing Lin
    um_extfrc_specifier = ' '
  !  um_extfrc_specifier(1)='CO     ->/glade/p/cesmdata/cseg/inputdata/atm/cam/chem/trop_mozart/emis/extfrc.CO.1.9x2.5_c101206.nc' 
   ! um_extfrc_specifier(2)= 'NO     ->/glade/p/cesmdata/cseg/inputdata/atm/cam/chem/trop_mozart/emis/extfrc.NO.1.9x2.5_c101206.nc' 
   um_extfrc_specifier(1) = 'NO2    ->/glade/p/cesmdata/cseg/inputdata/atm/cam/chem/1850-2000_emis/IPCC_emissions_aircraft_NO2_1850-2000_1.9x2.5_c090729.nc'
   um_extfrc_specifier(2) ='SO2    ->/glade/p/cesmdata/cseg/inputdata/atm/cam/chem/trop_mozart_aero/emis/ar5_mam3_so2_elev_2000_c120315.nc'
   um_extfrc_specifier(3) ='bc_a1  ->/glade/p/cesmdata/cseg/inputdata/atm/cam/chem/trop_mozart_aero/emis/ar5_mam3_bc_elev_2000_c120315.nc'
     um_extfrc_specifier(4)='pom_a1 ->/glade/p/cesmdata/cseg/inputdata/atm/cam/chem/trop_mozart_aero/emis/ar5_mam3_oc_elev_2000_c120315.nc' 
      um_extfrc_specifier(5)='so4_a1 ->/glade/p/cesmdata/cseg/inputdata/atm/cam/chem/trop_mozart_aero/emis/ar5_mam3_so4_a1_elev_2000_c120315.nc'
      um_extfrc_specifier(6)= 'so4_a2 ->/glade/p/cesmdata/cseg/inputdata/atm/cam/chem/trop_mozart_aero/emis/ar5_mam3_so4_a2_elev_2000_c120315.nc'

    extfrc_type_um='CYCLICAL'
    extfrc_cycle_yr_um=2000
    !-----------------------------------------------------------------------
    ! 	... species has insitu forcing ?
    !-----------------------------------------------------------------------

    !write(iulog,*) 'Species with insitu forcings'

    !write(iulog,*) 'gxlin: gas_pcnst', gas_pcnst
    count_emis: do n=1,gas_pcnst

       if ( len_trim(extfrc_specifier(n) ) == 0 ) then
          exit count_emis
       endif

       i = scan(extfrc_specifier(n),'->')
       spc_name = trim(adjustl(extfrc_specifier(n)(:i-1)))
       filename = trim(adjustl(extfrc_specifier(n)(i+2:)))

       m = get_extfrc_ndx( spc_name )

       if ( m < 1 ) then
          call endrun('extfrc_inti: '//trim(spc_name)// ' does not have an external source')
       endif

       if ( .not. frc_from_dataset(m) ) then
          call endrun('extfrc_inti: '//trim(spc_name)//' cannot have external forcing from additional dataset')
       endif

       mm = get_spc_ndx(spc_name)
       spc_fnames(mm) = filename

       has_extfrc(mm) = .true.
       !write(iulog,*) '   ',  spc_name ,' : filename = ',trim(spc_fnames(mm)),' spc ndx = ',mm

    enddo count_emis

    extfrc_cnt = count( has_extfrc(:) )

    if( extfrc_cnt < 1 ) then
       if (masterproc) write(iulog,*) 'There are no species with insitu forcings'
       return
    end if

    if (masterproc) write(iulog,*) ' '

    !-----------------------------------------------------------------------
    ! 	... allocate forcings type array
    !-----------------------------------------------------------------------
    allocate( forcings(extfrc_cnt), stat=astat )
    if( astat/= 0 ) then
       write(iulog,*) 'extfrc_inti: failed to allocate forcings array; error = ',astat
       call endrun
    end if
    
    !added by Guangxing Lin
    count_emis_um: do n=1,gas_pcnst_um

       if ( len_trim(um_extfrc_specifier(n) ) == 0 ) then
          exit count_emis_um
       endif

       i = scan(um_extfrc_specifier(n),'->')
       spc_name = trim(adjustl(um_extfrc_specifier(n)(:i-1)))
       filename = trim(adjustl(um_extfrc_specifier(n)(i+2:)))
         select case( spc_name )
    !      case( 'SO2' )
     !      m = um_get_spc_ndx('fSO2')
      !   case( 'CO' )
      !    m = um_get_spc_ndx('CO')
      !    case( 'NO' )
      !     m = um_get_spc_ndx('NO')
          case( 'NO2' )
           m = um_get_spc_ndx('NO2')
         case( 'SO2' )
           m = um_get_spc_ndx('fSO2')
        case( 'bc_a1'  )
           m = um_get_spc_ndx('bBC')
         case( 'pom_a1' )
           m = um_get_spc_ndx('bOC')
         case( 'so4_a1' ) !1 is accumulation in MAM3 -> m3 in IMPACT
           m = um_get_spc_ndx('SO4m3')
         case( 'so4_a2' )!2 is Aitken in MAM3 -> m2 in IMPACT, m1 is Nucleation mode in IMPACT
           m = um_get_spc_ndx('SO4m2')        
         end select
          
       


       if ( m < 1 ) then
          call endrun('um_extfrc_inti: '//trim(spc_name)// ' does not have an external source')
       endif

      ! if ( .not. frc_from_dataset(m) ) then
      !    call endrun('extfrc_inti: '//trim(spc_name)//' cannot have external forcing from additional dataset')
      ! endif

       spc_fnames_um(m) = filename

       has_extfrc_um(m) = .true.
       !write(iulog,*) '   ',  spc_name ,' : filename = ',trim(spc_fnames(mm)),' spc ndx = ',mm

    enddo count_emis_um

    extfrc_cnt_um = count( has_extfrc_um(:) )

    if( extfrc_cnt_um < 1 ) then
       if (masterproc) write(iulog,*) 'There are no species with insitu forcings_um'
       return
    end if

    if (masterproc) write(iulog,*) ' '

    !-----------------------------------------------------------------------
    ! 	... allocate forcings type array
    !-----------------------------------------------------------------------
    !added by Guangxing Lin
    allocate( forcings_um(extfrc_cnt_um), stat=astat )
    if( astat/= 0 ) then
       write(iulog,*) 'extfrc_inti_um: failed to allocate forcings array; error = ',astat
       call endrun
    end if
    !-----------------------------------------------------------------------
    ! 	... setup the forcing type array
    !-----------------------------------------------------------------------
    n = 0
    species_loop : do m = 1,gas_pcnst
       has_forcing : if( has_extfrc(m) ) then
          spc_name = trim( solsym(m) )
          n        = n + 1
          !-----------------------------------------------------------------------
          ! 	... default settings
          !-----------------------------------------------------------------------
          forcings(n)%frc_ndx          = get_extfrc_ndx( spc_name )
          forcings(n)%species          = spc_name
          forcings(n)%filename         = spc_fnames(m)
          call addfld( trim(spc_name)//'_XFRC',  'molec/cm3/s', pver, 'A', &
                       'external forcing for '//trim(spc_name),   phys_decomp )
          call addfld( trim(spc_name)//'_CLXF',  'molec/cm2/s', 1, 'A', &
                       'vertically intergrated external forcing for '//trim(spc_name),   phys_decomp )
          if ( history_aerosol ) then 
             call add_default( trim(spc_name)//'_XFRC', 1, ' ' )
             call add_default( trim(spc_name)//'_CLXF', 1, ' ' )
          endif
       end if has_forcing
    end do species_loop
    !added by Guangxing Lin
    n = 0
    species_loop_um : do m = 1,gas_pcnst_um
       has_forcing_um : if( has_extfrc_um(m) ) then
          spc_name = trim(lchemvar(m)) 
          n        = n + 1
          !-----------------------------------------------------------------------
          ! 	... default settings
          !-----------------------------------------------------------------------
          forcings_um(n)%frc_ndx          = m 
          forcings_um(n)%species          = spc_name
          forcings_um(n)%filename         = spc_fnames_um(m)
          call addfld( trim(spc_name)//'_XFRC',  'molec/cm3/s', pver, 'A', &
                       'UM_external forcing for '//trim(spc_name),   phys_decomp )
          call addfld( trim(spc_name)//'_CLXF',  'molec/cm2/s', 1, 'A', &
                       'UM_vertically intergrated external forcing for '//trim(spc_name),   phys_decomp )
          if ( history_aerosol ) then 
             call add_default( trim(spc_name)//'_XFRC_UM', 1, ' ' )
             call add_default( trim(spc_name)//'_CLXF_UM', 1, ' ' )
          endif
       end if has_forcing_um
    end do species_loop_um

    if (masterproc) then
       !-----------------------------------------------------------------------
       ! 	... diagnostics
       !-----------------------------------------------------------------------
       write(iulog,*) ' '
       write(iulog,*) 'extfrc_inti: diagnostics'
       write(iulog,*) ' '
       write(iulog,*) 'extfrc timing specs'
       write(iulog,*) 'type = ',extfrc_type
       if( extfrc_type == 'FIXED' ) then
          write(iulog,*) ' fixed date = ', extfrc_fixed_ymd
          write(iulog,*) ' fixed time = ', extfrc_fixed_tod
       else if( extfrc_type == 'CYCLICAL' ) then
          write(iulog,*) ' cycle year = ',extfrc_cycle_yr
       end if
       write(iulog,*) ' '
       write(iulog,*) 'there are ',extfrc_cnt,' species with external forcing files'
       do m = 1,extfrc_cnt
          write(iulog,*) ' '
          write(iulog,*) 'forcing type ',m
          write(iulog,*) 'species = ',trim(forcings(m)%species)
          write(iulog,*) 'frc ndx = ',forcings(m)%frc_ndx
          write(iulog,*) 'filename= ',trim(forcings(m)%filename)
       end do
       write(iulog,*) ' '
    endif

    !added by Guangxing Lin
    if (masterproc) then
       !-----------------------------------------------------------------------
       ! 	... diagnostics
       !-----------------------------------------------------------------------
       write(iulog,*) ' '
       write(iulog,*) 'UM extfrc_inti: diagnostics'
       write(iulog,*) ' '
       write(iulog,*) 'UM extfrc timing specs'
       write(iulog,*) 'type = ',extfrc_type_um
       if( extfrc_type == 'FIXED' ) then
          write(iulog,*) ' fixed date = ', extfrc_fixed_ymd
          write(iulog,*) ' fixed time = ', extfrc_fixed_tod
       else if( extfrc_type == 'CYCLICAL' ) then
          write(iulog,*) ' cycle year = ',extfrc_cycle_yr_um
       end if
       write(iulog,*) ' '
       write(iulog,*) 'there are ',extfrc_cnt_um,' species with external forcing files'
       do m = 1,extfrc_cnt_um
          write(iulog,*) ' '
          write(iulog,*) 'forcing type ',m
          write(iulog,*) 'species = ',trim(forcings_um(m)%species)
          write(iulog,*) 'frc ndx = ',forcings_um(m)%frc_ndx
          write(iulog,*) 'filename= ',trim(forcings_um(m)%filename)
       end do
       write(iulog,*) ' '
     end if
    !-----------------------------------------------------------------------
    ! read emis files to determine number of sectors
    !-----------------------------------------------------------------------
    frcing_loop: do m = 1, extfrc_cnt

       forcings(m)%nsectors = 0

       call cam_pio_openfile ( ncid, trim(forcings(m)%filename), PIO_NOWRITE)
       ierr = pio_inquire (ncid, nVariables=nvars)

       do vid = 1,nvars

          ierr = pio_inq_varndims (ncid, vid, ndims)

          if( ndims < 4 ) then
             cycle
          elseif( ndims > 4 ) then
             ierr = pio_inq_varname (ncid, vid, varname)
             write(iulog,*) 'extfrc_inti: Skipping variable ', trim(varname),', ndims = ',ndims, &
                  ' , species=',trim(forcings(m)%species)
             cycle
          end if

          forcings(m)%nsectors = forcings(m)%nsectors+1

       enddo

       allocate( forcings(m)%sectors(forcings(m)%nsectors), stat=astat )
       if( astat/= 0 ) then
         write(iulog,*) 'extfrc_inti: failed to allocate forcings(m)%sectors array; error = ',astat
         call endrun
       end if

       isec = 1
       do vid = 1,nvars

          ierr = pio_inq_varndims (ncid, vid, ndims)
          if( ndims == 4 ) then
             ierr = pio_inq_varname(ncid, vid, forcings(m)%sectors(isec))
             isec = isec+1
          endif

       enddo

       call pio_closefile (ncid)
  ! only in cesm1.2
       
       allocate(forcings(m)%file%in_pbuf(size(forcings(m)%sectors)))
       forcings(m)%file%in_pbuf(:) = .false.

       call trcdata_init( forcings(m)%sectors, &
                          forcings(m)%filename, filelist, datapath, &
                          forcings(m)%fields,  &
                          forcings(m)%file, &
                          rmv_file, extfrc_cycle_yr, extfrc_fixed_ymd, extfrc_fixed_tod, extfrc_type)

    enddo frcing_loop

    !added by Guangxing 
    !-----------------------------------------------------------------------
    ! read emis files to determine number of sectors
    !-----------------------------------------------------------------------
    frcing_loop_um: do m = 1, extfrc_cnt_um

       forcings_um(m)%nsectors = 0

       call cam_pio_openfile ( ncid, trim(forcings_um(m)%filename), PIO_NOWRITE)
       ierr = pio_inquire (ncid, nVariables=nvars)

       do vid = 1,nvars

          ierr = pio_inq_varndims (ncid, vid, ndims)

          if( ndims < 4 ) then
             cycle
          elseif( ndims > 4 ) then
             ierr = pio_inq_varname (ncid, vid, varname)
             write(iulog,*) 'UM_extfrc_inti: Skipping variable ', trim(varname),', ndims = ',ndims, &
                  ' , species=',trim(forcings_um(m)%species)
             cycle
          end if

          forcings_um(m)%nsectors = forcings_um(m)%nsectors+1

       enddo

       allocate( forcings_um(m)%sectors(forcings_um(m)%nsectors), stat=astat )
       if( astat/= 0 ) then
         write(iulog,*) 'UM_extfrc_inti: failed to allocate forcings(m)%sectors array; error = ',astat
         call endrun
       end if

       isec = 1
       do vid = 1,nvars

          ierr = pio_inq_varndims (ncid, vid, ndims)
          if( ndims == 4 ) then
             ierr = pio_inq_varname(ncid, vid, forcings_um(m)%sectors(isec))
             isec = isec+1
          endif

       enddo

       call pio_closefile (ncid)

  ! only in cesm1.2
       
       allocate(forcings_um(m)%file%in_pbuf(size(forcings_um(m)%sectors)))
       forcings_um(m)%file%in_pbuf(:) = .false.
       call trcdata_init( forcings_um(m)%sectors, &
                          forcings_um(m)%filename, filelist, datapath, &
                          forcings_um(m)%fields,  &
                          forcings_um(m)%file, &
                          rmv_file, extfrc_cycle_yr_um, extfrc_fixed_ymd, extfrc_fixed_tod, extfrc_type_um)

    enddo frcing_loop_um

  end subroutine extfrc_inti

  subroutine extfrc_timestep_init( pbuf2d, state )
    !-----------------------------------------------------------------------
    !       ... check serial case for time span
    !-----------------------------------------------------------------------

    use physics_types,only : physics_state
    use ppgrid,       only : begchunk, endchunk
    use tracer_data,  only : advance_trcdata
    use physics_buffer, only : physics_buffer_desc

    implicit none

    type(physics_state), intent(in):: state(begchunk:endchunk)                 
    type(physics_buffer_desc), pointer :: pbuf2d(:,:)

    !-----------------------------------------------------------------------
    !       ... local variables
    !-----------------------------------------------------------------------
    integer :: m

    do m = 1,extfrc_cnt
       call advance_trcdata( forcings(m)%fields, forcings(m)%file, state, pbuf2d  )
    end do
    !added by Guangxing Lin
    do m = 1,extfrc_cnt_um
       call advance_trcdata( forcings_um(m)%fields, forcings_um(m)%file, state, pbuf2d  )
    end do

  end subroutine extfrc_timestep_init

  subroutine extfrc_set( lchnk, zint, frcing, ncol )
    !--------------------------------------------------------
    !	... form the external forcing
    !--------------------------------------------------------
    use um_aerosol_intr, only: um_get_spc_ndx, um_emis_ver, um_emis_ver_has ! added by Cheng
    implicit none

    !--------------------------------------------------------
    !	... dummy arguments
    !--------------------------------------------------------
    integer,  intent(in)    :: ncol                  ! columns in chunk
    integer,  intent(in)    :: lchnk                 ! chunk index
    real(r8), intent(in)    :: zint(ncol, pverp)                  ! interface geopot above surface (km)
    real(r8), intent(inout) :: frcing(ncol,pver,extcnt)   ! insitu forcings (molec/cm^3/s)

    !--------------------------------------------------------
    !	... local variables
    !--------------------------------------------------------
    integer  ::  i, m, n
    character(len=16) :: xfcname
    real(r8) :: frcing_col(1:ncol)
    integer  :: k, isec, ispec !ispec is added by Cheng
    real(r8),parameter :: km_to_cm = 1.e5_r8
    real(r8) :: frcing_um(ncol,pver,gas_pcnst_um)   ! Guangxing Lin 

#if (defined MODAL_AERO)
    real(r8) :: fr_to_ait, fr_to_acc, fr_to_pom, fr_to_cor
    real(r8) :: demis_ait, demis_acc, demis_pom, demis_cor
           ! emitted mass-mean diameter (m) for aitken, accum, coarse modes
           ! for log-normal, demis = dgn_emis * exp(1.5*(lnsigmag*2))
    real(r8) :: x_mton_ait, x_mton_acc, x_mton_pom, x_mton_cor
           ! [(number emissions)/(mass emissions)] for aitken, accum, coarse modes
           ! x_mton_xxx = 1/(emitted particle mean mass)  (#/kg)
    real(r8) :: om_to_oc
#endif

    if( extfrc_cnt < 1 .or. extcnt < 1 ) then
       return
    end if

    !--------------------------------------------------------
    !	... set non-zero forcings
    !--------------------------------------------------------
    src_loop : do m = 1,extfrc_cnt

!!$#ifdef DIAGS
!!$          write(iulog,*) ' '
!!$          write(iulog,*) 'src frcing'
!!$          write(iulog,'(1p5g15.7)') frc_data
!!$          write(iulog,*) 'model frcing'
!!$          write(iulog,'(1p5g15.7)') frc_model(:,1)
!!$          data_dz(:)  = forcings(m)%levi(2:nlev+1) - forcings(m)%levi(1:nlev)
!!$          model_dz(:) = model_z(2:pverp) - model_z(1:pver)
!!$          data_sum  = dot_product( frc_data,data_dz )
!!$          model_sum = dot_product( frc_model(:,1),model_dz )
!!$          write(iulog,*) 'data_sum, model_sum = ',data_sum, model_sum
!!$          write(iulog,*) '======================================================'
!!$          if ( abs( (data_sum-model_sum)/max(1.e-20,(data_sum+model_sum)) ) > 0.3 ) then
!!$             call endrun('EXTFRC_SET: data_sum and model_sum disagree')
!!$          endif
!!$#endif

       n = forcings(m)%frc_ndx

       frcing(:ncol,:,n) = 0._r8
       do isec = 1,forcings(m)%nsectors
          if (forcings(m)%file%alt_data) then
             frcing(:ncol,:,n) = frcing(:ncol,:,n) + forcings(m)%fields(isec)%data(:ncol,pver:1:-1,lchnk)
          else
             frcing(:ncol,:,n) = frcing(:ncol,:,n) + forcings(m)%fields(isec)%data(:ncol,:,lchnk)
          endif
       enddo

   

#if (defined MODAL_AERO)
   !! this is done here because so4_a1 and so4_a2 have the same emis files
   !   fr_to_ait = 0.01             ! fraction of SO4 mass emitted to aitken mode
   !   fr_to_acc = 1.0 - fr_to_ait  ! fraction of SO4 mass emitted to accu. mode
   !   demis_ait = 0.018e-6    ! m
   !   demis_acc = 0.11e-6     ! m
       om_to_oc  = 1.4_r8

   !   x_mton_ait = 6.0 * specmw_so4_amode /      &     ! kmol -> #
   !               (pi*specdens_so4_amode*(demis_ait**3))
   !   x_mton_acc = 6.0 * specmw_so4_amode /      &     ! kmol -> #
   !               (pi*specdens_so4_amode*(demis_acc**3))

       select case( forcings(m)%species )
   !   case( 'so4_a1' )
   !      frcing(:ncol,:,n) = frcing(:ncol,:,n) * fr_to_acc * 1._r8/2.5_r8 ! 0._r8
   !   case( 'so4_a2' )
   !      frcing(:ncol,:,n) = frcing(:ncol,:,n) * fr_to_ait * 1._r8/2.5_r8 ! 0._r8
   !   case( 'num_a1' )
   !      frcing(:ncol,:,n) = frcing(:ncol,:,n) * fr_to_acc * x_mton_acc * 1._r8/2.5_r8 ! 0._r8
   !   case( 'num_a2' )
   !      frcing(:ncol,:,n) = frcing(:ncol,:,n) * fr_to_ait * x_mton_ait * 1._r8/2.5_r8 ! 0._r8
#if ( defined MODAL_AERO_7MODE )
       case( 'pom_a3' )
          frcing(:ncol,:,n) = frcing(:ncol,:,n) * om_to_oc
#elif ( defined MODAL_AERO_3MODE )
       case( 'pom_a1' )
          frcing(:ncol,:,n) = frcing(:ncol,:,n) * om_to_oc
#endif
       end select
#endif

       xfcname = trim(forcings(m)%species)//'_XFRC'
       call outfld( xfcname, frcing(:ncol,:,n), ncol, lchnk )

       frcing_col(:ncol) = 0._r8
       do k = 1,pver
          frcing_col(:ncol) = frcing_col(:ncol) + frcing(:ncol,k,n)*(zint(:ncol,k)-zint(:ncol,k+1))*km_to_cm
       enddo
       xfcname = trim(forcings(m)%species)//'_CLXF'
       call outfld( xfcname, frcing_col(:ncol), ncol, lchnk )

    end do src_loop

    src_loop_um : do m = 1, extfrc_cnt_um 

!!$#ifdef DIAGS
!!$          write(iulog,*) ' '
!!$          write(iulog,*) 'src frcing'
!!$          write(iulog,'(1p5g15.7)') frc_data
!!$          write(iulog,*) 'model frcing'
!!$          write(iulog,'(1p5g15.7)') frc_model(:,1)
!!$          data_dz(:)  = forcings(m)%levi(2:nlev+1) - forcings(m)%levi(1:nlev)
!!$          model_dz(:) = model_z(2:pverp) - model_z(1:pver)
!!$          data_sum  = dot_product( frc_data,data_dz )
!!$          model_sum = dot_product( frc_model(:,1),model_dz )
!!$          write(iulog,*) 'data_sum, model_sum = ',data_sum, model_sum
!!$          write(iulog,*) '======================================================'
!!$          if ( abs( (data_sum-model_sum)/max(1.e-20,(data_sum+model_sum)) ) > 0.3 ) then
!!$             call endrun('EXTFRC_SET: data_sum and model_sum disagree')
!!$          endif
!!$#endif

       n = forcings_um(m)%frc_ndx

       frcing_um(:ncol,:,n) = 0._r8
       do isec = 1,forcings_um(m)%nsectors
          if (forcings_um(m)%file%alt_data) then
             frcing_um(:ncol,:,n) = frcing_um(:ncol,:,n) + forcings_um(m)%fields(isec)%data(:ncol,pver:1:-1,lchnk)
          else
             frcing_um(:ncol,:,n) = frcing_um(:ncol,:,n) + forcings_um(m)%fields(isec)%data(:ncol,:,lchnk)
          endif
       enddo

   !-------------
   ! added by Cheng
   !--------------
!       if (masterproc) then
!   	  	write(*,'(A,3I3,A,A,A)'),'mo_extfrc:', lchnk,m,n,' ',forcings(m)%species,forcings(m)%fields(1)%units
!       endif
!   0:mo_extfrc: 17  1  1 SO2     molecules/cm3/s
!   0:mo_extfrc: 17  2  2 so4_a1  molecules/cm3/s
!   0:mo_extfrc: 17  3  4 pom_a1  molecules/cm3/s
!   0:mo_extfrc: 17  4  5 bc_a1   molecules/cm3/s
!   0:mo_extfrc: 17  5  6 num_a1  (particles/cm3/s) * 6.022e26
!   0:mo_extfrc: 17  6  3 so4_a2  molecules/cm3/s
!   0:mo_extfrc: 17  7  7 num_a2  (particles/cm3/s) * 6.022e26
       
    !-------------------------------------
    !following is added by Cheng to store srf flux in um_aerosol_intr,oc has been converted to OM above
    !-------------------------------------
       ispec = -1 !unit is #/cm3/s
       select case( trim(forcings_um(m)%species) )  ! 
        case( 'NO2'  )
           ispec = um_get_spc_ndx('NO2')
           um_emis_ver(lchnk,:ncol,:,ispec) = frcing_um(:ncol,pver:1:-1,n) 
         case( 'fSO2' )
           ispec = um_get_spc_ndx('fSO2')
           um_emis_ver(lchnk,:ncol,:,ispec) = frcing_um(:ncol,pver:1:-1,n) ! from top down to bottom up
        case( 'bBC'  )
           ispec = um_get_spc_ndx('bBC')
           um_emis_ver(lchnk,:ncol,:,ispec) = frcing_um(:ncol,pver:1:-1,n) 
         case( 'bOC' )
           ispec = um_get_spc_ndx('bOC')
           um_emis_ver(lchnk,:ncol,:,ispec) = frcing_um(:ncol,pver:1:-1,n)
         case( 'SO4m3' ) !1 is accumulation in MAM3 -> m3 in IMPACT
           ispec = um_get_spc_ndx('SO4m3')
           um_emis_ver(lchnk,:ncol,:,ispec) = frcing_um(:ncol,pver:1:-1,n)
         case( 'SO4m2' )!2 is Aitken in MAM3 -> m2 in IMPACT, m1 is Nucleation mode in IMPACT
           ispec = um_get_spc_ndx('SO4m2')        
           um_emis_ver(lchnk,:ncol,:,ispec) = frcing_um(:ncol,pver:1:-1,n)
       end select
         
           	
        if (ispec .ge. 1) then ! has emission
          um_emis_ver_has(ispec) = .true. 
        end if
   !---------end output to um_aerosol arrays ---------------------------------
         
           	
   


    end do src_loop_um

  end subroutine extfrc_set


end module mo_extfrc
