  module um_aerosol_number

  !-------------------------------------------------------------------------------------------------------
  ! Purpose: calculate aerosol number.
  !
  ! 
  ! Author: Cheng Zhou, 2013-03-26,Zhouc@umich.edu
  !-------------------------------------------------------------------------------------------------------
  use shr_kind_mod,  only: r8=>shr_kind_r8
  use ppgrid,        only: pver, pcols
  use abortutils,    only: endrun
  use cam_logfile,   only: iulog
  use spmd_utils,    only: masterproc,iam
!----  	

  implicit none
  private
  save
  
  integer  :: idx_aBC, idx_aPOM, idx_cBC, idx_cPOM
  real(r8),parameter :: re_airsoot = 0.03*1.0d-6, rhoairsoot=2.300*1.0e3 ,  rhoapom =9.0*1.0e2 
  real(r8) :: pm_aBC, pm_aPOM ! particle mass
  	
  public :: um_aerosol_number_init  ! called by physpkg.F90
  public :: um_aircraft_soot_number ! called by microp_aero.F90
  
  contains
  
  subroutine um_aerosol_number_init ! all chunks
  use cam_history,     only: addfld, add_default, phys_decomp
  use um_aerosol_intr, only: um_get_spc_ndx_in_q
  	
  call addfld      ('um_asoot_num', '#/cm3', pver, 'I', 'aircraft soot number (aBC +aPOM)', phys_decomp) !
  call add_default ('um_asoot_num', 1, 'A')
  call add_default ('um_asoot_num', 3, 'I')    
  
  !! calculate particle mass 
  pm_aBC  = 3.141592653_r8 * 4.0_r8 / 3.0_r8 * re_airsoot ** 3 * rhoairsoot 
  pm_aPOM = 3.141592653_r8 * 4.0_r8 / 3.0_r8 * re_airsoot ** 3 * rhoapom
  
  !! get index in q
  idx_aBC  = um_get_spc_ndx_in_q ('aBC')
  idx_aPOM = um_get_spc_ndx_in_q ('aPOM')
  idx_cBC  = um_get_spc_ndx_in_q ('cBC')
  idx_cPOM = um_get_spc_ndx_in_q ('cPOM')  
  !print*, 'idx_aBC,idx_aPOM',idx_aBC,idx_aPOM 
  
  end subroutine um_aerosol_number_init
   
  subroutine um_aircraft_soot_number (state, asoot_num) !one chunk
!----------------------------------------------------------------------- 
!
! Purpose: calculate the IN number from aircraft soot
!
!-----------------------------------------------------------------------
   use cam_history,     only: outfld
   use ppgrid,          only: begchunk, endchunk
   use physics_types,   only: physics_state
   implicit none
   
  ! Arguments:
   type(physics_state), intent(in) :: state          ! Physics state variables
   real(r8), dimension(pcols,pver) :: asoot_num
   	
   	
   ! Local Variables
   integer m,k
   integer lchnk,ncol
   real(r8), dimension(pcols,pver) :: rho


!  Begin
   		
   	lchnk = state%lchnk
    ncol  = state%ncol
    
    rho(:ncol,:) = state%pmid(:ncol,:) / 287.0_r8 / state%t(:ncol,:)  ! p = rho R T
   
!   asoot_num(:ncol,:) = rho(:ncol,:) * (state%q(:ncol, :, idx_aBC ) / pm_aBC + state%q(:ncol, :, idx_aPOM) / pm_aPOM) *1e-6_r8 !!! kg/kg to kg/m3 then to #/m3 then to #/cm3        	
    asoot_num(:ncol,:) = rho(:ncol,:) * (state%q(:ncol, :, idx_cBC ) / pm_aBC + state%q(:ncol, :, idx_cPOM) / pm_aPOM) *1e-6_r8 !!! kg/kg to kg/m3 then to #/m3 then to #/cm3        	
  
	  call outfld('um_asoot_num' , asoot_num(:ncol,:) , ncol, lchnk)
	  	       	   
  end subroutine um_aircraft_soot_number
  
  !============================================================================ !
  !                                                                             !
  !============================================================================ !
       
  end module  um_aerosol_number
