c=============================================================================
c
c $Id: chemread.F,v 1.1 2003/05/16 22:22:25 akinorii Exp $
c
c CODE DEVELOPER
c   Original code from Sandy Sillman, U of M.
c   IMPACT quadchem modifications:  Akinori Ito.
c   Modified again by Sandy Sillman 2007 version
c
c FILE
c   chemread.F
c
c ORIGINAL FILE
c   rbcheminit.f   (full information there)
c
c ARGUMENTS
c   none.  
c     lrin : File ID of chem_infile_name - replaced by c_rin
c
c
c=============================================================================
c ROUTINES:  
c    chemread:  reads input for mechanism: species, reactions, etc.
c    namechem:  function to return chemical species index from name.

c  cheminit.f   April, 2007
c 
c  ---------------------------------------------------------
c  Program history:
c    12/06 Initial program by Sandy Sillman based on boxchemv7.f 
c  ---------------------------------------------------------
c
          subroutine chemread

c This reads the chemistry input file (REACTION.DAT)         
c   which includes:
c    species names, steady-state indices, and 'lumping' for I/O
c    aqueous equilibria
c    reaction rates
c    reaction list and stoichiometries
c    order  of solution ('cascade') 

c Note:  photolysis rate input is in HVREAD.
c        box model input parameters are in BREDIN.
c
c Inputs:  None.
c
c Outputs:  All chemistry parameters listed above  
c
c
c Called by:  Do_Quad_Initn (as part of chemical setup)
c
c calls to:   (Optional call to  cheminit)
c
c
c ---------------------------------------------
c History:
c  12/06 Written by Sandy Sillman from boxchemv7.f
c
c -------------------------------------------------------------------
c
      use h_rbchemmech
c      use h_rbchemlocal, only: avogadrl

      IMPLICIT NONE

c#     include "rbchemmech.h"
c #     INCLUDE "rbchemvars.h" 
c#     include "rbchemlocal.h"
c-----below are from rbchemlocal.h
       real*8 :: stoicx                         ! stoichiometry sum
      integer :: ic,ic1,ic2,ic3,icc,ics,ics2,icc1,icc2,icc3 !Chem index
      integer :: is,iss,iscs,isc2            !Chem local
      integer :: ich,icq,icx,icx1,icx2,icy1,icy2,icp,icp1,icp2 !Chem
      integer :: icr1,icr2,icr,isr1,isr2,icpair  !chem index - pair and
      integer :: ionsum                                  ! aquasolve ion
      integer :: ica1,ica2,icb1,icb2,nra1,nra2,nrb1,nrb2 !Aquasolve chem
      integer :: neq                                     !Aqueous
      integer :: nc,nc1,nc2,ncc,ncf,nn,nne               !Chem species
      integer :: nr,nr1,nr2,nrh,nrq,nrqq,nrx,np          ! Reaction
      integer :: icat1,icat2,icatp,icatp2  ! indices used for species
      integer :: kk, kw                            ! Vectorization
      integer :: i, j, k, ii, ij, iii, n           ! General counters
       integer :: nsol           ! Number of pair groups being solved
      integer :: nsolv          ! Number of species being solved for
c      integer :: nsdim          ! Maximum number of pair groups being
c      integer :: ncdim          ! Maximum Number of species being
      integer :: ncsolv(c_cdim) ! Species number (ic) for species in
      integer :: nssolv(c_cdim) ! Pair group number (is) for spec. in

c
c LOCAL VARIABLES  
c   DUMMY VARIABLES FOR READ
      character*8 :: tdum(5)       ! dummy input character variable
      character*4 :: titl          ! dummy vbl to identify READ
      integer :: ndum(5)           ! dummy input integer :: variable
      real :: xdum(5)              ! dummy input real variable

c   OTHER LOCAL VARIABLES

c  caspair(ic,2)  Array to identify linked pair in cascade solver;
c                    2nd species is linked in pair chain to 1st
c                    used to establish pair chain array (nppair)

      integer :: caspair(c_cdim, 2    )  ! Array to ID paired species
      integer :: ncas              ! Index for species number in cascade
      integer :: ncasp             ! Integer for spec. number in caspair
c
c  lcastest: Flag set to TRUE when species is read in cascade.
      logical :: lcastest(c_cdim)    ! Flag to check species in cascade
c
      integer :: nlump         ! Counter for number of lumped species
      integer :: jj            ! General counter
      integer :: nnr           ! Added reaction counter (nr in chemlocal)

      integer :: namechem      ! function to return chem species index

c -------------------------------------------------------------------
c
      kk=1             ! Set vector variable for non-vectorized case

c GENERAL FORMATS
c 11    format(///////,a1)
11    format(a4)
c 11    format(a8)
12    format(i6)
13    format(1pe10.3)

c PRELIMINARY ZERO FOR READ
         do    nr=1,c_rdim
c         do    nr=1,548
         c_nnpro(nr) = 0
          do    i=1,20
           if (i.le.2) c_reactant(nr,i) = 0
           if(i.le.6) c_treac(i,nr) = '        '
           if(i.le.2.and.nr.le.61) c_treach(i,nr) = '        '
           if(i.le.2.and.nr.le.61)  c_henry(nr,i) = 0      
           if(i.le.3.and.nr.le.61) c_treacq(i,nr) = '        '
           if(i.le.3.and.nr.le.61) c_aqueous(nr,i) = 0     
           c_product(nr,i) = 0
           c_stoich(nr,i) = 0.
          enddo
          do ic=1,c_cdim
            c_prodarr(nr,ic) = 0.
          enddo
         enddo    

         do  i=1,c_cdim
          do  ii=1,c_cdim
           c_cascade(i,ii) = 0
          enddo    
          do ii=1,3
            c_lump(i,ii)=0
          enddo
          do ii=1,2
           caspair(i,ii) = 0
          enddo
c PAIR POINTERS to linked species in a pair chain:
c  (c_nppair(i,1) points to directly linked pair above in chain     )
c  (c_nppair(i,2) points to primary species for pair chain          )
c  (c_nppair(i,3) number of pair group subspecies for this species  )
c  (c_nppair(i,4+) chem. index (ic) for pair subspeice for this sp. )
c
c initially point at self;  
c   counter and pointers to secondary species zero (see nequil)
c
          c_nppair(i,1) = i
          c_nppair(i,2) = i
          do ii=3,23
            c_nppair(i,ii) = 0
          enddo
         enddo    

c ZERO SPECIAL SPECIES INDICES
         c_nh2o  =0
         c_nco2  =0
         c_nhplus   =0
         c_nohmin  =0
         c_noh   =0
         c_nho2  =0
         c_nh2o2 =0
         c_no3   =0
         c_nno2  =0
         c_nno   =0
         c_nno3  =0
         c_nn2o5 =0
         c_nhno3 =0
         c_nh2   =0
         c_nco   =0
         c_nch4  =0
c         c_no2  =0


c READ LOOP:  Read char*4 until you find 'START'. 
c  Then BREAK and begin first read cycle.
c 
       do 9005 i=1,1111
        read(c_rin,11) titl
        if(titl.eq.'STAR') go to 9006
9005   continue
9006   continue
c       write(c_out,11) titl


c READ PRELIMINARY INDICES:  VECTOR KKW, KMAX; NUMITER AND CONVERGE
       read(c_rin,12) c_kkw
       read(c_rin,12) c_kmax
       read(c_rin,12) c_numitr
         if(c_kkw.eq.5) write(c_out,12) c_kkw, c_kmax,c_numitr
       read(c_rin,13) c_converge
         if(c_kkw.eq.5) write(c_out,13) c_converge

c ------------------------
c READ LOOP: NAMES AND CATEGORIES FOR TRANSPORTED (INPUT/OUTPUT) SPECIES
c ------------------------
       ic=0

       do 9010 i=1,1111
        read(c_rin,11) titl
        if(titl.eq.'STAR') go to 9011
9010   continue
9011   continue
        if(c_kkw.eq.5) write(c_out,11) titl

c OPTION: READ IN GROUPS OF 1 OR 5:  ii=1,1 vs ii=1,5 THROUGHOUT
c   (main species read here and lumped species read below).
c   (also can change in summary write)

       do 30 i=1,c_cdim
       read(c_rin,29) (tdum(ii),ndum(ii),ii=1,5)
        if(c_kkw.eq.5) write(c_out,29) (tdum(ii),ndum(ii),ii=1,5)
c 29     format(5(7x,a4,i3,2x))
29     format(5(3x,a8,i3,2x))

c WHILE NAME NE 'END', ENTER DUMMY READ INTO TCHEM ARRAY.   
c AT 'END' BREAK AND LEAVE LOOP
        do 35 ii=1,5
         if(tdum(ii).eq.'     END'.or.tdum(ii).eq.'    END ') go to 31
         ic=ic+1
            if(ic.eq.c_cdim) then
              write(c_out,901) ic
              write(6,901) ic

901           format(/,'WARNING: NCHEM MAY EXCEED DIMENSION'
     *        ,' IN COMMON BLOCK.  NCHEM=',i4)
            endif
         c_tchem(ic)=tdum(ii)
         c_icat(ic)=ndum(ii)

c INSERT:  ASSIGN LSTS=T IF ICAT<0.  (Note, moved below after write)
c        c_lsts(ic) = .FALSE.
c        if(c_icat(ic).lt.0) c_lsts(ic) = .TRUE.
c        if(c_icat(ic).lt.0) c_icat(ic) = 0-c_icat(ic)
c          if(c_kkw.eq.5) write(c_out,*) 
c    *          ic, c_tchem(ic),c_icat(ic), c_lsts(ic)

35      continue
30     continue
31     continue
c
c READ INSERT:  RECORD NUMBER OF INPUT   SPECIES (NCHEM1)

       c_nchem1=ic

       if(c_kkw.gt.0) write(c_out,49) c_nchem1
49     format(' TOTAL NUMBER OF CHEMICALS READ =', i5)

c SUMMARY WRITE:  INPUT/OUTPUT SPECIES
       write(c_out,1301)
1301    format(/,'TRANSPORTED (INPUT) SPECIES:  ',
     *     '  (negative=steady state)'   ,/)

       ncf = int(0.2*(float(c_nchem1)-0.01))+1
       do      nc=1,ncf
         nc1 = 5*(nc-1) + 1
         nc2 = nc1 + 4
         if(nc1.gt.c_nchem1)nc1=c_nchem1
         if(nc2.gt.c_nchem1)nc2=c_nchem1
         write(c_out, 1302) (nn, c_tchem(nn),c_icat(nn) , nn=nc1,nc2 )
c 1302      format(5(i3,4x,a4,i3,2x))
1302      format(5(i3,   a8,i3,2x))
c 1302      format(1(i3,   a8,i3,2x))
       enddo

c SET CATEGORY AND STEADY STATE INDEX (after WRITE)
c    ASSIGN LSTS=T IF ICAT<0, MAKE CATEGORY POSITIVE.
       do ic=1,c_nchem1
         c_lsts(ic) = .FALSE.
         if(c_icat(ic).lt.0) c_lsts(ic) = .TRUE.
         if(c_icat(ic).lt.0) c_icat(ic) = 0-c_icat(ic)
c          if(c_kkw.eq.5) write(c_out,*) 
c    *            ic, c_tchem(ic),c_icat(ic), c_lsts(ic)
       enddo               !do ic=1,c_nchem1

c RECORD INTERIM NUMBER OF TOTAL SPECIES (INPUT + INTERNAL) (NCHEM2)
c  (This will be increased as other species are added)

       c_nchem2=c_nchem1

c ------------------------
c END OF CHEMICAL NAME LOOP.  TOTAL NUMBER OF CHEMICALS=NCHEM2
c  (to be moved below, after HENRY'S LAW and AQUEOUS EQULIBRIA)
c ------------------------

c -------------------------------------------------------
c READ LUMPED SPECIES
c -------------------------------------------------------
c LUMP(IC, IC1, IC2): SPECIES FROM IC1 TO IC2 ARE LUMPED
c INTO ACCUMULATOR (IC), AND SHARE LSTS, ICAT WITH LUMPED SUM.
c -------------------------------------------------------
c
c  11/22/06
c NEW READ: List of individual lumped species read here.

c NOTE: Individual lumped species are automatically added 
c       to the species list.
c The name of the lump group must be already included
c      in the list of 'primary' (i/o) species above.

       nlump=0

       do 9020 i=1,1111
        read(c_rin,11) titl
        if(titl.eq.'STAR') go to 9021
9020   continue
9021   continue
           if(c_kkw.eq.5) write(c_out,11) titl

      do 210 i=1, 200
c       read(c_rin,209)(tdum(ii),ii=1,3)
        read(c_rin,209)(tdum(ii),ii=1,1)
           if(c_kkw.eq.5) write(c_out,209)(tdum(ii),ii=1,1)
c 209     format(3(4x,a4,3x))
209     format(3(   a8,3x))
        if(tdum(1).eq.'     END'.or.tdum(1).eq.'    END ') 
     *                                              go to 211

c ADVANCE COUNTER, SET SPECIES INDICES AT ZERO.

        nlump = nlump + 1

        c_lump(nlump,1) = 0
        c_lump(nlump,2) = 0
        c_lump(nlump,3) = 0
c
c
c ENTER LUMP SPECIES NUMBER INTO LUMP ARRAY.
c IF LUMPED SPECIES  NAME IS UNKNOWN, ERROR AND EXIT LOOP.
         ic=namechem(tdum( 1))
         if(ic.eq.0) then
          write(c_out,219) (tdum(j),j=1,1),ii
          write(   6,219) (tdum(j),j=1,1),ii
219       format(/,'MAJOR ERROR: UNKNOWN SPECIES IN LUMPED SPECIES READ'
     *      ,/, 'SPECIES LIST = ',1(a8,4x),'UNKNOWN #',i3)
          c_lump(nlump,1) = 0
          c_lump(nlump,2) = 0
          c_lump(nlump,3) = 0
          nlump=nlump-1
          go to 210
         endif

        c_lump(nlump, 1) = ic

        c_llump(ic) = .TRUE.
c READ LIST OF SPECIES TO BE INCLUDED IN LUMP, ADD TO SPECIES LIST.

        do 40 jj=1,111
          read(c_rin,39) (tdum(ii),ii=1,5)
          if(c_kkw.eq.5) write(c_out,39) (tdum(ii),ii=1,5)
c 39        format(5(7x,a4,5x))
39       format(5(3x,a8,5x))

          do 45 ii=1,5
           if(tdum(ii).eq.'       x'.or.tdum(ii).eq.'        '
     *      .or.tdum(ii).eq.'      x '
     *                                               ) go to 41

c  ENTER SPECIES INTO SPECIES LIST AND INTO LUMP ARRAY.
c     (lump2 = 1st lumped species, lump3 = last lumped species)

           ic=namechem(tdum(ii))
           if(ic.eq.0) then

             c_nchem2 = c_nchem2+1
             if(c_nchem2.eq.c_cdim) then
               write(c_out,901) c_nchem2
c---------------Guangxing Lin ---11-19-2010----------------
c              write(6,*)'gxlin-test1'
c---------------Guangxing Lin ---11-19-2010----------------

               write(6,901) c_nchem2
             endif
             c_tchem(c_nchem2)=tdum(ii)

c  ADD TO LUMP ARRAY:
             if(c_lump(nlump,2).eq.0) c_lump(nlump, 2) = c_nchem2
             c_lump(nlump, 3) = c_nchem2

           else     !  if(ic.eq.0)

c  TEST IF SPECIES IS ALREADY INCLUDED IN SPECIES LIST -
c    IF LUMPED SPECIES ALREADY INCLUDED, ERROR AND EXIT LUMP LOOP

            write(c_out,218) (tdum(j),j=1,1)
            write(   6,218) (tdum(j),j=1,1)
218       format(/,'MAJOR ERROR: PREVIOUSLY SET SPECIES IN LUMPED LIST '
     *      ,/, 'SPECIES = ',1(a8   )               )
            c_lump(nlump,1) = 0
            c_lump(nlump,2) = 0
            c_lump(nlump,3) = 0
            nlump=nlump-1
            go to 210

         endif    !  if(ic.eq.0)

45      continue
40     continue
41     continue


c ENTER CATEGORY AND STEADY STATE FOR LUMPED SPECIES.
         do 225 ic=c_lump(nlump,2),c_lump(nlump,3)
          c_icat(ic) = c_icat(c_lump(nlump,1))
          c_lsts(ic) = c_lsts(c_lump(nlump,1))
225      continue

c SUMMARY WRITE: LUMPED SPECIES  SUMMARY
       if(nlump.eq.1) write(c_out, 1310)
1310   format(/, 'LUMPED SPECIES: NAME AND SPECIES CONTENTS')
       write(c_out,1311) c_tchem(c_lump(nlump,1))
1311    format(/,   a8)

       ncf = int(0.2*(float(c_lump(nlump,3)-c_lump(nlump,2)+1)-0.01))+1
       do      nc=1, ncf
         nc1 = 5*(nc-1) + c_lump(nlump,2)
         nc2 = nc1 + 4
         if(nc1.gt.c_lump(nlump,3)) nc1=c_lump(nlump,3)
         if(nc2.gt.c_lump(nlump,3)) nc2=c_lump(nlump,3)
         write(c_out, 1302) (nn, c_tchem(nn),c_icat(nn) , nn=nc1,nc2 )
       enddo

210    continue
211    continue
       write(c_out,999) nlump
999     format(/,'nlump= ', i4)
       write(c_out,999) i 
c -------------------------------------------------------
c END LUMPED SPECIES READ
c -------------------------------------------------------

c -------------------------------------------------------
c END LUMPED SPECIES READ
c -------------------------------------------------------
c
c ------------------------------------
c READ LOOP:  HENRY'S LAW COEFFICIENTS
c ------------------------------------
        nrh=0

       do 9030 i=1,1111
        read(c_rin,11) titl
        if(titl.eq.'STAR') go to 9031
9030   continue
9031   continue
         if(c_kkw.eq.5) write(c_out,11) titl

c LOOP FOR HENRY'S LAW READS: CONTINUE UNTIL 'END'.
      do  50 i=1,c_cdim

c ADVANCE COUNTER.  
       nrh = nrh+1
c --------------------------------
c READ NAMES OF HENRY'S LAW SPECIES.
c --------------------------------
            if(nrh.eq.161.or.nrh.eq.121.or.nrh.eq.c_cdim) then
              write(c_out,903) ic
              write(6,903) ic
            endif
903         format(/,'WARNING: NHENRY MAY EXCEED DIMENSION'
     *      ,' IN COMMON BLOCK.  NHENRY=',i4)

        read(c_rin, 59) tdum(1),tdum(2)
            if(c_kkw.eq.5) write(c_out, 59) tdum(1),tdum(2)
c 59      format(3(4x,a4,3x))
59      format(3(   a8,3x))

c
c ENTER NAMES INTO HENRY'S LAW MATRIX
c 'END' BREAKS THE LOOP TO END THE HENRY'S LAW READ.
         do  65 iii=1,2
          if(tdum(iii) .eq.'     END'.or.tdum(iii).eq.'    END ') 
     *                                                  go to  51

             c_treach(iii,nrh)=tdum(iii)

          ic = namechem(tdum(iii))
c            write(c_out,*) nrh,iii, tdum(iii),ic

c ADD AQUEOUS HENRY'S LAW SPECIES INTO SPECIES LIST (if not already there)
          if(ic.eq.0.and.iii.eq.2) then
           ic     = c_nchem2+1
           c_nchem2 = ic
           if(c_nchem2.eq.c_cdim) then
              write(c_out,901) c_nchem2
c---------------Guangxing Lin ---11-19-2010----------------
c              write(6,*)'gxlin-test2'
c---------------Guangxing Lin ---11-19-2010----------------
              write(6,901) c_nchem2
            endif
           c_tchem(c_nchem2)=tdum(iii)
          endif              !if(ic.eq.0.and.iii.eq.2) then

c ENTER SPECIES INTO HENRY LAW ARRAY
          if(ic.gt.0) then
                  c_henry(nrh,iii)=ic
          else
c TEST FOR POSSIBLE ERROR IN CHEMICAL NAMES.
c  THE GAS-PHASE SPECIES SHOULD ALREADY BE IN THE SPECIES LIST.
            if(                        tdum(iii).ne.'        '.and.
     *         tdum(iii).ne.'       x'.and.tdum(iii).ne.'       X'.and.
     *         tdum(iii).ne.'      x '.and.tdum(iii).ne.'      X '.and.
     *         tdum(iii).ne.'    MORE')  then
                 write(c_out, 69) tdum(iii), nrh,iii,(tdum(j),j=1,2)
                 write(   6, 69) tdum(iii), nrh,iii,(tdum(j),j=1,2)
            endif
69         format(/,'WARNING:  UNRECOGNIZED CHEMICAL NAME:',a8,/,
     *      ' NRH=',i4,'  NAME # ',i4,
     *      '  HENRY:  ',a8,'=',a8)
          endif
65       continue

c DEFAULT ACCOMODATION COEFFICIENT AND MOLECULAR WEIGHT
       c_accom(nrh) = 0.05
       c_molwt(nrh) = 30.

c  READ RATE PARAMETERS
c      write(c_out,*) nrh

c CURRENT OPTION: ACCOMODATION COEFFICIENT AND MOLECULAR WT READ.
c FUTURE MODIFICATION: ADD ALTERNATIVE RATE PARAMETER FORMATS IF NEEDED.
c  
c  READ LINES WITH THE FOLLOWING  FORMAT:  
c    CO2 =     CO2L
c   1    0 3.400E-02     2400. 5.000E-02       44.

       read(c_rin, 33)   c_nrkh(nrh),  c_rkh(1,nrh),c_rkh(2,nrh), 
     *    c_accom(nrh), c_molwt(nrh)
          if(c_kkw.eq.5) write(c_out, 33) 
     *       c_nrkh(nrh), c_rkh(1,nrh),c_rkh(2,nrh),c_accom(nrh)
     *         , c_molwt(nrh)

33     format(5x,i5, 1pe10.3, 0pf10.0, 1pe10.3, 0pf10.0)

50    continue
51    continue
c --------------------------------------------------------
c END HENRY'S LAW LOOP.
c --------------------------------------------------------
       c_nreach = nrh-1

       if(c_kkw.gt.0) write(c_out,57) c_nreach
57     format(' TOTAL NUMBER OF HENRYS-LAW COEFFICIENTS READ =', i5)
c
c
c ------------------------------------
c READ LOOP:  AQUEUS EQUILIBRIUM CONSTANTS WITH H+
c ------------------------------------
        nrq=0

       do 9040 i=1,1111
        read(c_rin,11) titl
        if(titl.eq.'STAR') go to 9041
9040   continue
9041   continue
         if(c_kkw.eq.5) write(c_out,11) titl

c LOOP FOR EQUILIBRIUM READS: CONTINUE UNTIL 'END'.
      do  70 i=1,c_cdim

c ADVANCE COUNTER.  
       nrq = nrq+1

            if(nrq.eq. 161.or.nrq.eq.121.or.nrq.eq. c_cdim)
     *      write(c_out,904) nrq
904         format(/,'WARNING: NAQUEOUS MAY EXCEED DIMENSION'
     *      ,' IN COMMON BLOCK.  NAQUEOUS=',i4)

c --------------------------------
c READ NAMES OF H+ EQUILIBRIUM SPECIES:  A = B + C* (*C=H+or OH-)
c    NOTE, 1st EQUILIBRIUM MUST BE 0<=>H+ + OH-
c --------------------------------

c         write(c_out,*) nrq
        read(c_rin, 59) tdum(1),tdum(2), tdum(3)
            if(c_kkw.eq.5) write(c_out, 59) tdum(1),tdum(2), tdum(3)

c SWITCH 2nd AND 3rd SPECIES if 2nd SPECIES = H+ or OH-
        if (nrq.gt.1) then
          if (tdum(2).eq.c_treacq(2,1).or.tdum(2).eq.c_treacq(3,1)
     *        .or.tdum(2).eq.'      H+'.or.tdum(2).eq.'     OH-'      
     *        .or.tdum(2).eq.'     H+ '.or.tdum(2).eq.'    OH- '      
     *                                                     ) then
            if(                        tdum( 3 ).ne.'        '.and.
     *         tdum( 3 ).ne.'       x'.and.tdum( 3 ).ne.'       X'.and.
     *         tdum( 3 ).ne.'      x '.and.tdum( 3 ).ne.'      X '.and.
     *         tdum( 3 ).ne.'    MORE')  then

              tdum(4) = tdum(2)
              tdum(2) = tdum(3)
              tdum(3) = tdum(4)
              if(c_kkw.gt.0) write(c_out,88) 
     *                nrq, tdum(1),tdum(2),tdum(3)
88            format(/,'AQUEOUS ORDER SWITCH: NUMBER:',i3,'  REACTION:',
     *               a8,'=',a8,'&',a8)

            endif     !if (tdum(3).ne.'        ')
          endif     !if (tdum(2).eq. ...H+)
        endif                !if (nrq.gt.1) then

c
c ENTER NAMES INTO AQUEOUS EQUILIBRIUM MATRIX
c 'END' BREAKS THE LOOP TO END THE EQUILIBRIUM READ.
         do  85 iii=1,3

          if(tdum(iii) .eq.'     END'.or.tdum(iii).eq.'    END ') 
     *                                                   go to  71

          c_treacq(iii,nrq)=tdum(iii)

          ic = namechem(tdum(iii))
c            write(c_out,*) nrq,iii, tdum(iii),ic

c ENTER NEW CATION/ANION INTO SPECIES LIST IF NOT ALREADY INCLUDED.
c   (NOTE, FIRST EQUILIBRIUM SHOULD BE 0 <=> H+ + OH-;
c    SUBSEQUENT SHOULD BE A <=> B + H+ or OH-)

         if(ic.eq.0.and.(iii.eq.2.or.(iii.eq.3.and.nrq.eq.1))) then
           ic     = c_nchem2+1
           c_nchem2 = ic
           if(c_nchem2.eq.c_cdim) then
              write(c_out,901) c_nchem2
              write(6,901) c_nchem2
            endif
           c_tchem(c_nchem2)=tdum(iii)

         endif         !if(ic.eq.0.and.(iii.eq.2....))


c ENTER SPECIES INTO AQUEOUS EQUILIBRIUM ARRAY
          if(ic.gt.0) then
                c_aqueous(nrq,iii)=ic
          else 
c TEST FOR POSSIBLE ERROR IN CHEMICAL NAMES
            if(                        tdum(iii).ne.'        '.and.
     *         tdum(iii).ne.'       x'.and.tdum(iii).ne.'       X'.and.
     *         tdum(iii).ne.'      x '.and.tdum(iii).ne.'      X '.and.
     *         tdum(iii).ne.'    MORE')  then
                 write(c_out, 89) tdum(iii), nrq,iii,(tdum(j),j=1,3)
                 write(   6, 89) tdum(iii), nrq,iii,(tdum(j),j=1,3)
            endif
89          format(/,'WARNING:  UNRECOGNIZED CHEMICAL NAME:',a8,/,
     *      ' NRQ=',i4,'  NAME # ',i4,
     *      ' AQUEOUS:  ',a8,'=',a8,' + ',a8)
          endif
85       continue

c  END NAMEREAD LOOP.  READ RATE PARAMETERS

       read(c_rin, 33)   c_nrkq(nrq), c_rkq(1,nrq),c_rkq(2,nrq)
          if(c_kkw.eq.5) write(c_out, 33) 
     *              c_nrkq(nrq), c_rkq(1,nrq),c_rkq(2,nrq)

70    continue
71    continue

       c_nreacq = nrq-1

       if(c_kkw.gt.0) write(c_out,79) c_nreacq
79     format(' TOTAL NUMBER OF AQUEOUS EQUILIBRIUM CONSTANTS =', i5)

c
c ------------------------------------
c READ LOOP:  AQUEUS SPECIAL EQUILIBRIUM CONSTANTS WITHOUT H+/OH- 
c ------------------------------------
c (note:  These are special.  A <->B+C; the combined species A are set 
c  in equilibria to be automatically solved when B is called. 
c  Species C MUST have much higher concentrations then either A or B,
c  or else be zero.)

        nrqq=0

       do 9045 i=1,1111
        read(c_rin,11) titl
        if(titl.eq.'STAR') go to 9046
9045   continue
9046   continue
         if(c_kkw.eq.5) write(c_out,11) titl

c LOOP FOR EQUILIBRIUM READS: CONTINUE UNTIL 'END'.
      do  170 i=1,c_cdim

c ADVANCE COUNTER.  
       nrqq = nrqq+1

            if(nrqq.eq.121.or.nrqq.eq. 91.or.nrqq.eq.c_cdim)
     *      write(c_out,905) nrqq
905         format(/,'WARNING: NAQUEOUSQ MAY EXCEED DIMENSION'
     *      ,' IN COMMON BLOCK.  NAQUEOUSQ=',i4)

c --------------------------------
c READ NAMES OF SPECIAL EQUILIBRIUM SPECIES:  A = B + C* (*C=common spec or 0)
c --------------------------------

c         write(c_out,*) nrqq
        read(c_rin, 59) tdum(1),tdum(2), tdum(3)
            if(c_kkw.eq.5) write(c_out, 59) tdum(1),tdum(2), tdum(3)
c
c ENTER NAMES INTO AQUEOUS SPECIAL EQUILIBRIUM MATRIX
c 'END' BREAKS THE LOOP TO END THE EQUILIBRIUM READ.
         do  86 iii=1,3

          if(tdum(iii) .eq.'     END'.or.tdum(iii).eq.'    END ') 
     *                                                  go to  171

          c_treaqq(iii,nrqq)=tdum(iii)

          ic = namechem(tdum(iii))
c            write(c_out,*) nrqq,iii, tdum(iii),ic
          if(ic.gt.0) then
                c_aqspec(nrqq,iii)=ic
          else 
c TEST FOR POSSIBLE ERROR IN CHEMICAL NAMES
            if(                        tdum(iii).ne.'        '.and.
     *         tdum(iii).ne.'       x'.and.tdum(iii).ne.'       X'.and.
     *         tdum(iii).ne.'      x '.and.tdum(iii).ne.'      X '.and.
     *         tdum(iii).ne.'    MORE')  then
                 write(c_out, 87) tdum(iii), nrqq,iii,(tdum(j),j=1,3)
                 write(   6, 87) tdum(iii), nrqq,iii,(tdum(j),j=1,3)
            endif
87          format(/,'WARNING:  UNRECOGNIZED CHEMICAL NAME:',a8,/,
     *      ' NRQQ=',i4,'  NAME # ',i4,
     *      ' AQUEOUSQ:  ',a8,'=',a8,' + ',a8)
          endif
86       continue

c  END NAMEREAD LOOP.  READ RATE PARAMETERS
c   NOTE FUTURE DO OPTION: Make rate=rate of return reaction
c     with nn corresponding to nn for regular reactions (   nrk)

       read(c_rin, 33)   c_nrkqq(nrqq),  c_rkqq(1,nrqq),c_rkqq(2,nrqq)
          if(c_kkw.eq.5) write(c_out, 33) 
     *          c_nrkqq(nrqq), c_rkqq(1,nrqq),c_rkqq(2,nrqq)

170    continue
171    continue

       c_nreaqq = nrqq-1

       if(c_kkw.gt.0) write(c_out,99) c_nreaqq
99     format(' TOTAL NUMBER OF AQUEOUSQ EQUILIBRIUM CONSTANTS =', i5)


c --------------------------------------
c READ LOOP:  REACTION RATES; REACTANTS, PRODUCTS AND STOICHIOMETRIES
c --------------------------------------
       nr = 0

       do 9050 i=1,1111
        read(c_rin,11) titl
        if(titl.eq.'STAR') go to 9051
9050   continue
9051   continue
         if(c_kkw.eq.5) write(c_out,11) titl

c LOOP FOR REACTION READS: CONTINUE UNTIL 'END'.
c      do 100 i=1,548
       do 100 i=1,c_rdim

c ADVANCE REACTION COUNTER
       nr = nr+1
c      write(c_out,*) nr

            if(nr.eq.1711.or.nr.eq.1911.or.nr.eq.3211)
     *      write(c_out,906) nr
906         format(/,'WARNING: NREACT   MAY EXCEED DIMENSION'
     *      ,' IN COMMON BLOCK.  NREACT  =',i4)

c --------------------------------------------------------
c READ NAMES OF REACTANTS, PRODUCTS AND STOICHIOMETRIES.
c --------------------------------------------------------
       do 110 ii=1,10
c         write(c_out,*) ii
        read(c_rin,109) tdum(1),tdum(2),xdum(3),tdum(3),
     *   xdum(4),tdum(4),xdum(5),tdum(5)
          if(c_kkw.eq.5) write(c_out,109) tdum(1),tdum(2),xdum(3),
     *     tdum(3),xdum(4),tdum(4),xdum(5),tdum(5)
c 109     format(4x,a4,7x,a4,4x,2(f6.3,6x,a4,3x),f6.3,6x,a4)
109     format(   a8,3x,a8,4x,2(f6.3,2x,a8,3x),f6.3,2x,a8)
c
c ENTER NAMES INTO REACTANT OR PRODUCT MATRIX. (also TREAC)
c 'NEXT' BREAKS THE LOOP TO ADVANCE TO NEXT REACTION
c 'END' BREAKS THE LOOP TO END THE REACTION READ.
         do 120 iii=1,5
          if(ii.eq.1.and.tdum(iii).ne.'    NEXT'.and.
     *                   tdum(iii).ne.'     END'. and.
     *                   tdum(iii).ne.'   NEXT '. and.
     *                   tdum(iii).ne.'    END '
     *                                                     ) then
             c_treac(iii,nr)=tdum(iii) 
          endif

          if(tdum(iii).eq.'    NEXT'.or.tdum(iii).eq.'   NEXT '
     *                                                   ) go to 111
          if(tdum(iii) .eq.'     END'.or.tdum(iii).eq.'    END ') 
     *                                                     go to 101

          ic = namechem(tdum(iii))
c            write(c_out,*) nr, tdum(iii),c_tchem(1),ic
          if(ic.gt.0) then
            if(iii.le.2) then
               c_reactant(nr,iii)=ic
c              if(c_kkw.eq.5) write(c_out,*) nr,iii,c_reactant(nr,iii)
            else
              do 125 j=1,20
               if (c_product(nr,j).eq.0) go to 126
125           continue
126           continue
              c_product(nr,j) = ic
              c_stoich(nr,j) = xdum(iii)
              c_nnpro(nr) = j
              c_prodarr(nr,ic) = xdum(iii)
c             if(c_kkw.eq.5) write(c_out,*) nr,j,c_nnpro(nr),
c    *          c_product(nr,j)

c TEST WRITE PRODARR 
c  (=stoichiometry for production of species ic from reaction nr)
c
            if(c_kkw.gt.0) write(c_out,127) nr, c_rk(1,nr), tdum
            if(c_kkw.gt.0) write(c_out,128)
     *         j, ic, c_stoich(nr,j), c_prodarr(nr,ic)
127         format(' ID PRODUCTS:  NR, RK1=', i5,1pe10.3,/,
     *      '  TREACT= ',a8,'+',a8,'->',a8,'+',a8,'+',a8)
128         format(' J IC STOIC PRODARR = ', 2i5,2f10.3)

            endif
          else 
c 
c FLAG FOR HV REACTION:  Identified by 'hv' as Reactiant #2.  
c   Set REACTANT(2) =-1, 
c    and default hv rate index 4 (rate proportional to jNO2)
c
            if(iii.eq.2.and.
     *          (tdum(iii).eq.'      hv'.or.tdum(iii).eq.'     hv ')
     *                                         ) then 
               c_reactant(nr,iii)=-1
               if(c_nrk(nr).eq.0) c_nrk(nr) =  4 

c  TEST WRITE HV
c           write(c_out,138) nr, c_rk(1,nr), tdum
c 138         format(' ID HV RATES:  NR, RK1=', i5,1pe10.3,/,
c    *      '  TREACT= ',a8,'+',a8,'->',a8,'+',a8,'+',a8)

            endif       ! FLAG FOR HV
c
c TEST FOR POSSIBLE ERROR IN CHEMICAL NAMES
            if(tdum(iii).ne.'      hv'.and.tdum(iii).ne.'        '.and.
     *         tdum(iii).ne.'       x'.and.tdum(iii).ne.'       X'.and.
     *         tdum(iii).ne.'     hv '.and.tdum(iii).ne.'      x '.and.
     *         tdum(iii).ne.'    MORE')  then
                 write(c_out,139) tdum(iii), nr,iii,tdum
                 write(   6,139) tdum(iii), nr,iii,tdum
            endif
139         format(/,'WARNING:  UNRECOGNIZED CHEMICAL NAME:',a8,/,
     *      ' NR=',i4,'  NAME # ',i4,
     *      '  TREACT= ',a8,'+',a8,'->',a8,'+',a8,'+',a8)
          endif
120      continue

110    continue
111    continue
c -------------
c END READ REACTION NAMES
c -------------

c --------------------------------------------------------
c  READ RATE PARAMETERS (moved after NAMES)
c --------------------------------------------------------

c      read(c_rin, 33) c_rk(1,nr),c_rk(2,nr), c_nrk(nr)
       read(c_rin,131) ii, c_nrk(nr)
          if(c_kkw.eq.5) write(c_out,131) ii, c_nrk(nr)
131    format(2i5)

c READ OPTIONS AND FLAG FOR READ
       if(c_nrk(nr).eq.0.or.c_nrk(nr).eq.-1.or.c_nrk(nr).eq.-20
     &     .or.c_nrk(nr).eq.-23) then
          read(c_rin,132) c_rk(1,nr), c_rk(2,nr)
            if(c_kkw.eq.5) write(c_out,132) c_rk(1,nr), c_rk(2,nr)
       endif
132     format(10x,1pe10.3,0pf8.0,0pf6.1)

       if( c_nrk(nr).eq.-2.or.c_nrk(nr).eq.-5 .or.c_nrk(nr).eq.-6) then
          read(c_rin,132) c_rk(1,nr), c_rk(2,nr), c_rk(3,nr)
            if(c_kkw.eq.5) write(c_out,132) 
     *                 c_rk(1,nr), c_rk(2,nr), c_rk(3,nr)
       endif

       if(c_nrk(nr).eq.-3) then
          read(c_rin,133) c_rk(1,nr), c_rk(2,nr), c_rk(3,nr)
     *                    ,c_rk(4,nr), c_rk(5,nr)
            if(c_kkw.eq.5) write(c_out,133) 
     *                 c_rk(1,nr), c_rk(2,nr), c_rk(3,nr)
     *                    ,c_rk(4,nr), c_rk(5,nr)
       endif
133     format(10x,0pf6.2,1pe10.3,0pf6.1,1pe10.3,0pf6.1)

       if(c_nrk(nr).eq.-4) then
          read(c_rin,134) c_rk(1,nr), c_rk(2,nr), c_rk(3,nr)
     *                    ,c_rk(4,nr), c_rk(5,nr),c_rk(6,nr), c_rk(7,nr)
            if(c_kkw.eq.5) write(c_out,134) 
     *                 c_rk(1,nr), c_rk(2,nr), c_rk(3,nr)
     *                    ,c_rk(4,nr), c_rk(5,nr),c_rk(6,nr), c_rk(7,nr)
       endif
134     format(10x,1pe10.3,0pf8.0,0pf6.2,1pe10.3,0pf6.1,1pe10.3,0pf6.1)

       if(c_nrk(nr).eq.-7) then
          read(c_rin,135) c_rk(1,nr), c_rk(2,nr), c_rk(3,nr)
     *                    ,c_rk(4,nr), c_rk(5,nr),c_rk(6,nr)
            if(c_kkw.eq.5) write(c_out,135) 
     *                 c_rk(1,nr), c_rk(2,nr), c_rk(3,nr)
     *                    ,c_rk(4,nr), c_rk(5,nr),c_rk(6,nr)
       endif
135     format(10x,1pe10.3,0pf8.0,4(1pe10.3))

       if(c_nrk(nr).eq.-8) then
          read(c_rin,136) c_rk(1,nr), c_rk(2,nr), c_rk(3,nr)
     *                    ,c_rk(4,nr), c_rk(5,nr)
            if(c_kkw.eq.5) write(c_out,136) 
     *                 c_rk(1,nr), c_rk(2,nr), c_rk(3,nr)
     *                    ,c_rk(4,nr), c_rk(5,nr)
       endif
136     format(10x,3(1pe10.3,0pf8.0) )         ! format used for others

       if(c_nrk(nr).eq.-9.or.c_nrk(nr).eq.-10                 ) then
          read(c_rin,136) c_rk(1,nr), c_rk(2,nr), c_rk(3,nr)
     *                    ,c_rk(4,nr), c_rk(5,nr),c_rk(6,nr)
            if(c_kkw.eq.5) write(c_out,136) 
     *                 c_rk(1,nr), c_rk(2,nr), c_rk(3,nr)
     *                    ,c_rk(4,nr), c_rk(5,nr),c_rk(6,nr)
       endif

       if(c_nrk(nr).eq.-11) then
          read(c_rin,137) c_rk(1,nr), c_rk(2,nr), c_rk(3,nr)
     *                    ,c_rk(4,nr), c_rk(5,nr)
            if(c_kkw.eq.5) write(c_out,137) 
     *                 c_rk(1,nr), c_rk(2,nr), c_rk(3,nr)
     *                    ,c_rk(4,nr), c_rk(5,nr)
       endif
137     format(10x,1pe10.3,0pf8.0,1pe10.3,0pf8.3,0pf8.0)

       if(c_nrk(nr).eq.-12) then
          read(c_rin,136) c_rk(1,nr), c_rk(2,nr), c_rk(3,nr)
     *                    ,c_rk(4,nr)
            if(c_kkw.eq.5) write(c_out,136) 
     *                 c_rk(1,nr), c_rk(2,nr), c_rk(3,nr)
     *                    ,c_rk(4,nr), c_rk(5,nr),c_rk(6,nr)
       endif

       if(c_nrk(nr).eq.-14) then
          read(c_rin,138) c_rk(1,nr), c_rk(2,nr), c_rk(3,nr)
     *                    ,c_rk(4,nr), c_rk(5,nr)
            if(c_kkw.eq.5) write(c_out,138)
     *                 c_rk(1,nr), c_rk(2,nr), c_rk(3,nr)
     *                    ,c_rk(4,nr), c_rk(5,nr)
       endif
138     format(10x,1pe10.3,0pf8.0,1pe10.3,0pf8.0,0pf8.3)
c 138     format(10x,3(1pe10.3))

c 138     format(10x,3(1pe10.3))

c READ FOR hv REACTIONS
       if(c_nrk(nr).gt.0.or.c_nrk(nr).le.-30  ) then
          read(c_rin,132) c_rk(1,nr)
            if(c_kkw.eq.5) write(c_out,132) c_rk(1,nr)
       endif

c ********************************************
c ** Begin of modifications by Luis Olcese, UM
c ** Changed to include secondary organic aerosols
       if(c_nrk(nr).eq.-21) then
        read(c_rin,142) c_rk(1,nr),c_rk(2,nr),c_rk(3,nr),c_rk(4,nr)
        if(c_kkw.eq.5) write(c_out,142) c_rk(1,nr),c_rk(2,nr),
     &   c_rk(3,nr),c_rk(4,nr)
       endif

       if(c_nrk(nr).eq.-22) then
        read(c_rin,142) c_rk(1,nr),c_rk(2,nr),c_rk(3,nr),c_rk(4,nr)
        if(c_kkw.eq.5) write(c_out,142) c_rk(1,nr),c_rk(2,nr),
     &   c_rk(3,nr),c_rk(4,nr)
       endif

142    format(10x,1pe10.3,1x,1pe10.3,1x,1pe10.3,1x,1pe10.3)

c ** End of modifications by Luis Olcese, UM
c ********************************************

100   continue
101   continue
c --------------------------------------------------------
c END REACTION LOOP.
c --------------------------------------------------------
        c_nreac = nr -1
c
c -------------------------------------------------------
c READ SPECIAL FAMILY SPECIES (CURRENTLY HX AND NX, IN ORDER)
c -------------------------------------------------------
c DELETED - REPLACED WITH SPECIAL SPECIES INDICES 
c           AND AUTOMATIC ID FROM SPECIES NAMES
c
c -------------------------------------------------------
c READ CASCADE SPECIES LIST.  (INCLUDES CASCADE PAIRS)
c -------------------------------------------------------
c   FUTURE DO:  
c     CHANGE CASCADE READ to be c_cascade(i,1), n; 
c      then if n>1, read c_cascade(i,ii),i=2,n
c      (maybe add flag for DIFFICULT CONVERGENCE)
c
c
c  THE CASCADE: 
c   IT CONSISTS OF A SEQUENCE OF SPECIES, GIVING THE ORDER OF SOLUTION.
c   THE CASCADE INCLUDES THREE TYPES OF SOLUTIONS
c
c    (1) A SINGLE SPECIES ON A LINE, to be solved by itself.
c    (2) A PAIR OF INTERACTING SPECIES, or A SERIES OF LINKED PAIRS
c         These are solved simultaneously by an algorithm for linked pairs
c          and can include:  A<->B1, A<->B2, B1<->C1, etc.
c    (3) A GROUP OF THREE OR INTERACTING MORE SPECIES, 
c          to be solved by the MULTISOLVE algorithm.  
c
c     SINGLE SPECIES are entered on a single line:  (SPEC1  x   x)
c
c     CASCADE PAIRS are entered either as a PAIR on a single line
c                                                  (SPEC1 SPEC2 x)
c        This tells the cascade to solve for the species now.  
c        or with a PAIR declaration                (pair SPEC1 SPEC2)
c        This tells the cascade to solve SPEC2 elsewhere, 
c           whenever SPEC1 is called
c       (which can be a single species, part of a pair, or multisolve.)
c
c     MULTISOLVE SPECIES are entered as groups of three, 
c       continuing until a blank is entered (   x).
c         
c  Notes on CASCADE PAIRS:  
c   A primary species may be paired with many secondary species.
c   The secondary species may then be paired with its own subspecies.
c   Typically there are rapid reactions between directly paired species.
c   It is OK for the paired species to have no interactions
c      (e.g. species that interact only in aqueous phase)

c   The chain of paired  species is built
c     with parent primary species first, followed by its (multiple)
c      secondary species each with its  own  subspecies
c
c   The pair chain uses  pointers, which are set below:
c  c_nppair(ic,1) = ics:  pointer from species ic  
c       to its directly linked primary pair species ics (or self)
c  c_nppair(ic,2) = ics:  pointer from species ic 
c    to ultimate primary pair species ics (or self)
c  c_nppair(ics,3) = number of chemical pairs associated w/ species ics.
c  c_nppair(ics,np),np>=4:  identifies nth pair species or subspecies 
c    for species ics, for total number given by nppair(ics,3).
c
c    (old format: pointers as with nequil:
c      npair(ics) = # of chem pairs  associated with the species
c      nppair(ic) = ics:  pointer to primary  pair species (or self)
c     ncpair(ics,np) = ic:  Identifies nth  pair species orsubspec.
c    )
c
c BEGIN CASCADE READ
c
       do 9070 i=1,1111
        read(c_rin,11) titl
        if(titl.eq.'STAR') go to 9071
9070   continue
9071   continue

       ncas = 1
       ncasp = 0
       nc = 0

       do  i=1,c_cdim                  ! CASCADE READ LOOP 
        read(c_rin,309)(tdum(ii),ii=1,3)
c 309     format(3(4x,a4,2x))
309     format(3(   a8,2x))

c LOOP TO READ THREE SPECIES AND CONVERT TO SPECIES NUMBERS
c   NOTE:  This read loop repeats to read full group 
c          until it reads a break mark (x)

        do ii=1,3           ! LOOP TO READ GROUP OF THREE SPECIES (320)
c-------Guangxing Lin ---03032013------- 
           ic=0
c-------Guangxing Lin ---03032013------- 
c IF NAME IS 'END', BREAK THE CASCADE READ AND EXIT
c     (ALSO ADJUST COUNTER)

         if(tdum(1).eq.'     END'.or.tdum(1).eq.'    END ') then
           if(nc .eq.0) ncas = ncas - 1
           go to 311
         endif          !if(tdum(1).eq.'     END'....

c IF NAME IS  x, BREAK, PROCESS, AND GO TO THE NEXT CASCADE  GROUP.

          if(tdum(ii).eq.'       x'.or.tdum(ii).eq.'        '
     *      .or. tdum(ii).eq.'     pre'.or.tdum(ii).eq.'    post'
     *                                               ) go to 310

c IDENTIFY SPECIES NAME
         ic=namechem(tdum(ii)) 

c IF FIRST NAME IS 'pair' , SET FLAG TO ESTABLISH CASCADE PAIRS
         if (ii.eq.1.and.
     *        (tdum(ii).eq.'    pair'
     *         .or.tdum(ii).eq.'    PAIR'
     *         .or.tdum(ii).eq.'   pair '
     *         .or.tdum(ii).eq.'   PAIR '
     *                                 ) ) ic=-1

c WARNING FOR NO-NAME IN CASCADE. (no-name is regarded as break)
         if(ic.eq.0) then
          write(c_out,319) (tdum(j),j=1,3),ii
          write(   6,319) (tdum(j),j=1,3),ii
319       format(/,'MAJOR ERROR: UNKNOWN SPECIES IN CASCADE SPEC  READ.'
     *      ,/, 'SPECIES LIST = ',3(a8,4x),'UNKNOWN #',i3)
          go to 310
         endif

c ENTER SPECIES IN CASCADE LIST (AND ADVANCE COUNTER)
c  (NOTE: LATER ADJUSTMENT FOR CASCADE PAIR FLAG)
        nc = nc + 1
        c_cascade(ncas ,nc ) = ic 


        enddo              !do ii=1,3  LOOP TO READ GROUP OF THREE SPECIES (320)

310    continue            ! AFTER READ OF COMPLETE CASCADE GROUP.

c PROCESS CASCADE SPECIES GROUP  AFTER READ OF COMPLETE GROUP.  

c ESTABLISH CASCADE PAIRS BASED ON READ GROUP OF THREE SPECIES.

c (1) IF CASCADE LIST CONSISTS OF JUST TWO SPECIES,
c     THEN IDENTIFY SECOND SPECIES AS PAIRED WITH THE FIRST
c     AND REMOVE THE SECOND SPECIES  FROM THE CASCADE ARRAY
c    (paired species will automatically be solved when primary is called)

      if (c_cascade(ncas,1).gt.0.and.
     *     c_cascade(ncas,2).gt.0.and.c_cascade(ncas,3).eq.0) then

         ncasp = ncasp + 1
         caspair(ncasp ,1) = c_cascade(ncas,1)
         caspair(ncasp ,2) = c_cascade(ncas,2)
         c_cascade(ncas,2) = 0

      endif    !if (c_cascade(ncas,2).gt.0....

c (2) IF CASCADE LIST HAS 'PAIR' FLAG (first species is -1)
c     THEN IDENTIFY 2nd AND 3rd SPECIES AS PAIR
c     AND REMOVE BOTH FROM CASCADE ARRAY.
c      (2nd species should be in cascade or in cascade pair elsewhere)
c    (with 1st species set to zero, this cascade number will be re-done)

      if(c_cascade(ncas,1).eq.-1
     *    .and.c_cascade(ncas,2).gt.0.and.c_cascade(ncas,3).gt.0) then

         ncasp = ncasp + 1
         caspair(ncasp ,1) = c_cascade(ncas,2)
         caspair(ncasp ,2) = c_cascade(ncas,3)

         c_cascade(ncas,1) = 0
         c_cascade(ncas,2) = 0
         c_cascade(ncas,3) = 0
         
          nc = 0                  ! Reset counter nc for next species  7-4-2010
         ic = 0                   ! Flag to move to next cascade group

c   ADD ERROR WRITE if species is zero

      endif     !if(c_cascade(ncas,1).eq.-1) then

c ADVANCE THE SPECIES COUNTER ncas AND RESET COUNTER nc  FOR THE NEXT SPECIES
c   Do not advance or reset if multi-species list is continuing (ic>0)
c   Reset nc but do not advance ncas if this is 'pair' line
c   Do not advance if the species is missing.
c
c Corrected for multi-species group solution 7-4-2010  (impact version 3-2-13)

              if(c_cascade(ncas,1).gt.0.and.ic.eq.0) then
                ncas = ncas + 1
                nc = 0
              endif

c    older version
c              if(c_cascade(ncas,1).gt.0) ncas = ncas + 1
c              nc = 0

       enddo         !do  i=1,c_cdim  CASCADE READ LOOP 
311    continue

c TEMPORARY TEST:  LIST CASCADE PAIRS AND LIST CASCADE
c       if(c_kkw.eq.5) then
       if(c_kkw.ne.0) then
         write(c_out,314)
314      format(/,' LIST OF CASCADE PAIRS AND CASCADE')
         do i=1,ncasp
           write(c_out, 315) i, c_tchem(caspair(i,1)),
     *                      c_tchem(caspair(i,2))
315        format(i4,   a8,   a8,   a8)
         enddo
         do i=1,  ncas
           do ii=1,3
             ic = c_cascade(i,ii)
             if(ic.gt.0) then
               write(c_out,315) i, c_tchem(ic)
             endif
           enddo
         enddo
       endif           !if(c_kkw.eq.5) then

croy
c       write(42,*) 'in chemread'
c       do i = 1, c_cdim
c       write(42,316) (c_cascade(i,ii), ii=1, c_cdim)
c       enddo
c316    format('in chemread',/,10i8)
croy
c -------------------------------------------------------
c END CASCADE READ
c -------------------------------------------------------
c
c --------------------------
c IDENTIFY SPECIAL SPECIES NUMBERS AND CATEGORIES
c --------------------------
c  This replaces 'family'.  

c LOOP TO IDENTIFY SPECIAL SPECIES
      do ic=1, c_nchem2

         if(c_tchem(ic).eq.'     H2O'.or.c_tchem(ic).eq.'    H2O '
     *                                                 )    then
            c_nh2o =ic
            c_icat(ic) = 0
         endif
         if(c_tchem(ic).eq.'     CO2'.or.c_tchem(ic).eq.'    CO2 '
     *                                                 )      then
            c_nco2 =ic
            c_icat(ic) =20
         endif
         if(c_tchem(ic).eq.'      H+'.or.c_tchem(ic).eq.'     H+ '
     *                                                 )      then
            c_nhplus  =ic
            c_icat(ic) =17
         endif
         if(c_tchem(ic).eq.'     OH-'.or.c_tchem(ic).eq.'    OH- '
     *                                                 )      then
            c_nohmin =ic
            c_icat(ic) =18
         endif
         if(c_tchem(ic).eq.'      OH'.or.c_tchem(ic).eq.'     OH '
     *                                                 )      then
            c_noh  =ic
            c_icat(ic) = 9
         endif
         if(c_tchem(ic).eq.'     HO2'.or.c_tchem(ic).eq.'    HO2 '
     *                                                 )      then
            c_nho2 =ic
            c_icat(ic) =10
         endif
         if(c_tchem(ic).eq.'    H2O2'.or.c_tchem(ic).eq.'   H2O2 '
     *                                                 )      then 
            c_nh2o2=ic
            c_icat(ic) =19
         endif
         if(c_tchem(ic).eq.'      O3'.or.c_tchem(ic).eq.'     O3 '
     *                                                 )      then
            c_no3  =ic
            c_icat(ic) =11
         endif
         if(c_tchem(ic).eq.'     NO2'.or.c_tchem(ic).eq.'    NO2 '
     *                                                 )      then
            c_nno2 =ic
            c_icat(ic) =12
         endif
         if(c_tchem(ic).eq.'      NO'.or.c_tchem(ic).eq.'     NO '
     *                                                 )      then
            c_nno  =ic
            c_icat(ic) =13
         endif
         if(c_tchem(ic).eq.'     NO3'.or.c_tchem(ic).eq.'    NO3 '
     *                                                 )      then
            c_nno3 =ic
            c_icat(ic) =14
         endif
         if(c_tchem(ic).eq.'    N2O5'.or.c_tchem(ic).eq.'   N2O5 '
     *                                                 )      then 
            c_nn2o5=ic
            c_icat(ic) =15
         endif
         if(c_tchem(ic).eq.'    HNO3'.or.c_tchem(ic).eq.'   HNO3 '
     *                                                 )      then
            c_nhno3=ic
            c_icat(ic) =16
         endif
         if(c_tchem(ic).eq.'      H2'.or.c_tchem(ic).eq.'     H2 '
     *                                                 )      then
            c_nh2  =ic
            c_icat(ic) =1
         endif
         if(c_tchem(ic).eq.'      CO'.or.c_tchem(ic).eq.'     CO '
     *                                                 )      then
            c_nco  =ic
            c_icat(ic) =1
         endif
         if(c_tchem(ic).eq.'     CH4'.or.c_tchem(ic).eq.'    CH4 '
     *                                                 )      then
            c_nch4 =ic
            c_icat(ic) =1
         endif
c-------------Guangxing Lin -10-21-2011- add for aqueous SOA reactions----
c         if(c_tchem(ic).eq.'      O2'.or.c_tchem(ic).eq.'     O2 '
c     *                                                 )      then
c            c_no2 =ic
c            c_icat(ic) =1
c         endif
c-------------Guangxing Lin -10-21-2011- add for aqueous SOA reactions----


      enddo             !do ic=1, c_nchem2
c END  LOOP TO IDENTIFY SPECIAL SPECIES

c WARN IF THESE SPECIES ARE MISSING

      if(c_nh2o .eq.0) write(c_out, 1381)
1381  format(/,'WARNING. THE FOLLOWING SPECIES MAY BE MISSING',
     *         ' OR MISNAMED:  H2O')
      if(c_nco2 .eq.0) write(c_out, 1382)
1382  format(/,'WARNING. THE FOLLOWING SPECIES MAY BE MISSING',
     *         ' OR MISNAMED:  CO2')
      if(c_nhplus  .eq.0) write(c_out, 1383)
1383  format(/,'WARNING. THE FOLLOWING SPECIES MAY BE MISSING',
     *         ' OR MISNAMED:   H+')
      if(c_nohmin .eq.0) write(c_out, 1384)
1384  format(/,'WARNING. THE FOLLOWING SPECIES MAY BE MISSING',
     *         ' OR MISNAMED:  OH-')
      if(c_noh  .eq.0) write(c_out, 1385)
1385  format(/,'WARNING. THE FOLLOWING SPECIES MAY BE MISSING',
     *         ' OR MISNAMED:   OH')
      if(c_nho2 .eq.0) write(c_out, 1386)
1386  format(/,'WARNING. THE FOLLOWING SPECIES MAY BE MISSING',
     *         ' OR MISNAMED:  HO2')
      if(c_nh2o2.eq.0) write(c_out, 1387)
1387  format(/,'WARNING. THE FOLLOWING SPECIES MAY BE MISSING',
     *         ' OR MISNAMED: H2O2')
      if(c_no3  .eq.0) write(c_out, 1388)
1388  format(/,'WARNING. THE FOLLOWING SPECIES MAY BE MISSING',
     *         ' OR MISNAMED:   O3')
      if(c_nno2 .eq.0) write(c_out, 1389)
1389  format(/,'WARNING. THE FOLLOWING SPECIES MAY BE MISSING',
     *         ' OR MISNAMED:  NO2')
      if(c_nno  .eq.0) write(c_out, 1390)
1390  format(/,'WARNING. THE FOLLOWING SPECIES MAY BE MISSING',
     *         ' OR MISNAMED:   NO')
      if(c_nno3 .eq.0) write(c_out, 1391)
1391  format(/,'WARNING. THE FOLLOWING SPECIES MAY BE MISSING',
     *         ' OR MISNAMED:  NO3')
      if(c_nn2o5.eq.0) write(c_out, 1392)
1392  format(/,'WARNING. THE FOLLOWING SPECIES MAY BE MISSING',
     *         ' OR MISNAMED: N2O5')
      if(c_nhno3.eq.0) write(c_out, 1393)
1393  format(/,'WARNING. THE FOLLOWING SPECIES MAY BE MISSING',
     *         ' OR MISNAMED: HNO3')
c-------------Guangxing Lin -10-21-2011- add for aqueous SOA reactions----
c      if(c_no2.eq.0) write(c_out, 1394)
c1394  format(/,'WARNING. THE FOLLOWING SPECIES MAY BE MISSING',
c     *         ' OR MISNAMED:   O2')
c-------------Guangxing Lin -10-21-2011- add for aqueous SOA reactions----

C ----------------------
c ASSIGN AQUEOUS SPECIES TO ASSOCIATED GAS-MASTER SPECIES
c IN SEQUENCE:  GAS->AQUEOUS->AQUEOUS ION->AQUEOUS DOUBLE ION
c IDENTIFIED FROM HENRY'S LAW AND AQUEOUS EQUILIBRIUM SEQUENCE
C ----------------------
c
c GAS-AQUEOUS LINK INCLUDES:
c  c_npequil = pointer to gas-master species for aqueous link
c  c_nequil = number of aqueous species associated with gas-master.  
c  c_ncequil = ic for each aqueous species assoc. with gas-master.
c   (up to 3 species in sequence: aqueous, ion1, ion2)
c 
c   (FUTURE DO ALTERNATIVE:  combine.
c      c_npequil(ic,1) = pointer from aqueous to gas-master speciees
c      c_npequil(ic,2) = number of aqueous spec. assoc with gas-master
c      c_npequil(ic,n),n=3,4,5:  ic for each aq species assoc w/gas
c   (
c  nrequil = nrh, nrq for reaction number associated with 
c            corresponding aqueous species in ncequil 
c  
c ------------------------------------------------------------

c PRELIMINARY ZERO  
         do 610 ic=1,c_nchem2
          c_nequil(ic) = 0
          c_npequil(ic) = ic
          c_ion(ic) = 0
          do 620 i=1,6
           c_ncequil(ic,i) = 0
620        continue
610       continue

c SET ION INDEX FOR H+, OH-
c H+  
       if(c_aqueous(1,2).gt.0) then
         c_ion(c_aqueous(1,2))= 1
       endif
c OH-
       if(c_aqueous(1,3).gt.0) then
         c_ion(c_aqueous(1,3))=-1
       endif

c LOOP FOR HENRY'S LAW COEFFICIENTS

       do 650 nrh=1,c_nreach
        if(c_henry(nrh,1).eq.0) go to 651

c FOR EACH HENRY'S LAW COEFFICIENT, ASSOCIATE AQUEOUS WITH GAS SPECIES.
        ic = c_henry(nrh,1)
        ic1 = c_henry(nrh,2)
        c_npequil(ic1) = ic
        c_icat(ic1)=c_icat(ic)
        c_lsts(ic1)=c_lsts(ic)
        c_nequil(ic) = c_nequil(ic) + 1
        nne=c_nequil(ic)
        c_ncequil(ic,nne) = ic1
        c_nrequil(ic,nne) = nrh

c CHANGE PRODUCT ARRAY TO GAS-MASTER
c                     (add to avoid replacing already-set array)
       do nr=1,c_nreac
         c_prodarr(nr,ic) = c_prodarr(nr,ic) + c_prodarr(nr,ic1)
         c_prodarr(nr,ic1) = 0.

c TEST WRITE PRODARR
c        if(c_kkw.ne.0.and.c_prodarr(nr,ic).ne.0) 
c    *    write(c_out,656) c_tchem(ic), c_tchem(ic1),
c    *      nr, ic,ic1, c_prodarr(nr,ic)
c 656      format('TEST PRODARR ADJUSTMENT: NR IC IC1 PRODARR= ',
c    *       /,2(a8,2x),3i5, f10.3)
       enddo

c GO THROUGH AQUEOUS EQUILIBRIA.  ADD EQUILIBRIUM SPECIES
c TO GAS-MASTER SPECIES LIST.  
c NOTE EQUILIBRIA MUST COME IN SEQUENCE: 
c             GAS->AQUEOUS->FIRST ION->DOUBLE ION.
c ALSO SET ION INDEX.

        do 660 nrq=1,c_nreacq
        if(c_aqueous(nrq,1).eq.0.and.c_aqueous(nrq,2).eq.0) go to 661

c  H+ SPECIAL here: for H+, OH-  skip equilibrium index assignment
        if(c_aqueous(nrq,1).eq.0) go to 660


        if(c_npequil(c_aqueous(nrq,1)).eq.ic) then
          ic1 = c_aqueous (nrq,2)
          c_npequil(ic1) = ic
          c_icat(ic1)=c_icat(ic)
          c_lsts(ic1)=c_lsts(ic)
          c_nequil(ic) = c_nequil(ic) + 1
          nne=c_nequil(ic)
          c_ncequil(ic,nne) = ic1
          c_nrequil(ic,nne) = nrq

          c_ion(ic1) = c_ion(c_aqueous(nrq,1))
          if(c_aqueous(nrq,3).gt.0) then
            c_ion(ic1) = c_ion(ic1) - c_ion(c_aqueous(nrq,3))
            if(c_kkw.gt.0)  write(c_out,659) (c_treacq(j,nrq),j=1,3),
     *       c_ion(ic1), c_ion(c_aqueous(nrq,3))
659          format(a8,'=',a8,'+',a8,2x, 'ION2=',i3, '  ION3=',i3)
          endif

c CHANGE PRODUCT ARRAY TO GAS-MASTER.  
c                (add to avoid replacing already-set array)
          do nr=1,c_nreac
            c_prodarr(nr,ic) = c_prodarr(nr,ic) + c_prodarr(nr,ic1)
            c_prodarr(nr,ic1) = 0.
c TEST WRITE PRODARR
c        if(c_kkw.ne.0.and.c_prodarr(nr,ic).ne.0) 
c    *    write(c_out,656) c_tchem(ic), c_tchem(ic1),
c    *      nr, ic,ic1, c_prodarr(nr,ic)

          enddo

        endif

660     continue
661     continue

650    continue
651    continue

c      if(c_kkw.gt.0) write(c_out,669) (c_npequil(i),i=1,c_nchem2)
c 669    format(/'TEST NPEQUIL:',/,(20i5))

c SUMMARY WRITE: HENRY'S LAW AND AQUEOUS EQUILIBRIUM SPECIES (ncf)
       write(c_out,1321)
1321    format(/,'HENRYs LAW AND LINKED AQUEOUS EQUILIBRIUM SPECIES',/)

        write(c_out,1322) c_tchem(c_aqueous(1,2)), 
     *                c_tchem(c_aqueous(1,3))
        do  nrh=1,c_nreach
          ic = c_henry(nrh,1)
          if(c_nequil(ic).gt.0)  then
            write(c_out, 1322) c_tchem(ic),
     *        ( c_tchem(c_ncequil(ic,i))
     *                                      ,i=1,c_nequil(ic) )
c 1322            format(5(4x,a4,        2x))
1322            format(5(   a8,        2x))
          endif                    !if(c_nequil(ic).gt.0)  then
        enddo                   !do  nrh=1,c_nreach

c -----------------------------------------
c  END ASSIGN-AQUEOUS
c -----------------------------------------
c
c
c -----------------------------------------
c  CONVERT SPECIAL EQUILIBRIUM SPECIES TO BACK-FORTH REACTIONS 
c -----------------------------------------
c  Special equilibrium forward reaction 1e8 sec-1, 
c   backward reaction based on equilibrium coefficient.
c
c  (Special equilibrium reactions NOT entered into cascade pair.)

c LOOP FOR SPECIAL AQUEOUS EQUILIBRIUM SPECIES

       nr = c_nreac 

       do     nrqq=1,c_nreaqq
        if(c_aqspec(nrqq,1).ne.0.and.c_aqspec(nrqq,2).ne.0) then
          icc1 = c_aqspec(nrqq,1)
          icc2 = c_aqspec(nrqq,2)
          icc3 = c_aqspec(nrqq,3)

          ic1 = c_npequil(icc1)
          ic2 = c_npequil(icc2)
          ic3 = c_npequil(icc3)

c ENTER INTO CASCADE PAIR - CUT.
c Note:  1st SPECIAL EQUIL species is secondary paired species
c        2nd species is primary paired species  (A <=> B + C)
c        3rd species is totally independent
c If this is included, also change 'ncasp =' above.

c         caspair(nrqq,  1 ) = icc2 
c         caspair(nrqq,  2 ) = icc1 

c ENTER FORWARD REACTION            
          nr = nr+1
          if(nr.eq.1711.or.nr.eq.1911.or.nr.eq.3211)
     *      write(c_out,906) nr
          c_treac(1,nr) = c_tchem(icc1)
          c_treac(3,nr) = c_tchem(icc2)
          c_reactant(nr,1) = icc1
          c_product(nr,1) = icc2
          c_stoich(nr,1) = 1.
          c_nnpro(nr) = 1
          c_prodarr(nr,ic2 ) = 1.
          if(icc3.gt.0) then
           c_treac(4,nr) = c_tchem(icc3)
           c_product(nr,2) = icc3
           c_stoich(nr,2) = 1.
           c_nnpro(nr) = 2
           c_prodarr(nr,ic3 ) = 1.
          endif
c FORWARD RATE CONSTANT:  1e8 s-1.
c OPTION - maybe  this causes cncn-> 0.  alt 1e2 s-1
c         c_rk(1,nr) = 1.0D+08
          c_rk(1,nr) = 1.0D+02

c ENTER BACKWARD REACTION            
          nr = nr+1
          if(nr.eq.711.or.nr.eq.911.or.nr.eq.1211)
     *      write(c_out,906) nr
          c_treac(1,nr) = c_tchem(icc2)
          c_treac(3,nr) = c_tchem(icc1)
          c_reactant(nr,1) = icc2
          c_product(nr,1) = icc1
          c_stoich(nr,1) = 1.
          c_nnpro(nr) = 1
          c_prodarr(nr,ic1 ) = 1.
          if(icc3.gt.0) then
           c_treac(2,nr) = c_tchem(icc3)
           c_reactant(nr,2) = icc3
          endif
c BACKWARD RATE CONSTANT
c  Equil  constant in  moles/liter A exp(-B * (1/temp - 1/298) )
c  A<->B+C  KA = BC  K in moles/liter 
c  k1a = k2BC,  k1/k2=K,  k2=k1/K,  k2 as A2 exp(-B2/temp) also moles/liter
c  k1=1e8 s-1.  A2=k1/(Aexp(+B/298),  B2=-B
          c_rk(1,nr) = 1.0D+08/
     *                  ( c_rkqq(1,nrqq)*exp(c_rkqq(2,nrqq)/298.) )
          c_rk(2,nr) = 0. - c_rkqq(2, nrqq )

        endif
       enddo
c END LOOP FOR SPECIAL EQUILIBRIUM SPECIES

       c_nreac = nr 
c -----------------------------------------
c  END CONVERT SPECIAL EQUILIBRIUM SPECIES TO BACK-FORTH REACTIONS 
c -----------------------------------------
c
c -----------------------------------------
c  ADD NO3-N2O5 (hard-wired) INTO CASCADE PAIR LIST
c -----------------------------------------
       ncasp = ncasp + 1
       caspair(ncasp,  1 ) = c_nno3
       caspair(ncasp,  2 ) = c_nn2o5
c
c -----------------------------------------
c GENERATE POINTERS FOR CASCADE PAIRS
c -----------------------------------------

c  Notes:
c   A primary species may be paired with many secondary pair species.
c   The secondary pair species may then have its own pair subspecies.
c   Typically there are rapid reactions between directly linked pairs.
c   It is OK for the paired species to have no interactions
c      (e.g. species that interact only in aqueous phase)
c 
c   The chain of paired  species is built
c     with parent primary species first, 
c       followed by its secondary pair  species,
c          each with its  own pair subspecies.   
c
c   Pointers all point from and to gas-master species
c
c   The pair chain pointers are: 
c  c_nppair(ic,1) = ics:  pointer from species ic  
c       to its directly linked primary pair species ics (or self)
c  c_nppair(ic,2) = ics:  pointer from species ic 
c    to ultimate primary pair species ics (or self)
c  c_nppair(ics,3) = number of chemical pairs associated w/ species ics.
c  c_nppair(ics,np),np>=4:  identifies nth pair species or subspecies 
c    for species ics, for total number given by nppair(ics,3).
c
c    (old format: pointers as with nequil:
c      npair(ics) = # of chem pairs  associated with the species
c      nppair(ic) = ics:  pointer to primary  pair species (or self)
c     ncpair(ics,np) = ic:  Identifies nth  pair species orsubspec.
c    )
c
c
c      GENERATE PAIR POINTERS
c    - if c_nppair(2nd spec) does not equal self, error
c    - make 2nd species and its subspecies point to primary of 1st spec.
c    - if 2nd has subspecies, make point to 1st species
c    - if 1st species points elsewhere, make 2nd and its subspecies 
c       also point there.
c

c LOOP THROUGH CASCADE PAIRS
      do i=1,ncasp
        icc1 = caspair(i,1)
        icc2 = caspair(i,2)
        ic1 = 0
        ic2 = 0
        if(icc1.gt.0) then
          ic1 = c_npequil(icc1)
        endif
        if(icc2.gt.0) then 
          ic2 = c_npequil(icc2)
        endif
        ic3 = 0
        if(ic2.gt.0) then
         ic3=c_nppair(ic2,1)
        endif

c ERROR CHECK:  IF 2ND SPECIES IS ALREADY PAIRED, MAJOR ERROR
        if(ic3.ne.ic2.or.icc1.eq.0.or.icc2.eq.0.or.
     *                       ic1.eq.0.or.ic2.eq.0)  then
          write(c_out,667) i, icc1, icc2, ic1, ic2, c_nppair(ic2,1) 
          write( 6  ,667) i, icc1, icc2, ic1, ic2, c_nppair(ic2,1) 
667       format(/,' MAJOR ERROR IN CASCADE PAIR SETUP:',/,
     *     '   EITHER ZERO OR SECOND CASCADE PAIR ALREADY PAIRED.',
     *    /,' PAIR NUMBER, icc1, icc2, ic1, ic2, c_nppair(ic2,1)=', 6i5)
          if(icc1.gt.0) write (c_out,309) c_tchem(icc1)
          if(icc2.gt.0) write (c_out,309) c_tchem(icc2)
          if(ic1.gt.0) write (c_out,309) c_tchem(ic1)
          if(ic2.gt.0) write (c_out,309) c_tchem(ic2)
          if(icc1.gt.0) write (c_out,309) c_tchem(icc1)
          if(icc1.gt.0) write ( 6  ,309) c_tchem(icc1)
          if(icc2.gt.0) write ( 6  ,309) c_tchem(icc2)
          if(ic1.gt.0) write ( 6  ,309) c_tchem(ic1)
          if(ic2.gt.0) write ( 6  ,309) c_tchem(ic2)
          if(icc1.gt.0) write ( 6  ,309) c_tchem(icc1)
        else

c Assign all species and subspecies to the primary species 
c               of the first of the pair.
c  Note:  npair(ic2,3) = number of subspecies;  
c         npair(ic2,4) = 1st subsp, etc.
c  Also assign direct pair species - just for this secondary species.

          c_nppair(ic2,1) = ic1

          ic3=c_nppair(ic1,2)
          do ii = 1, (c_nppair(ic2,3)+1)
            ic  = ic2
            if(ii.gt.1) ic  = c_nppair(ic2,(ii+2))
            c_nppair(ic ,2) = ic3
            c_nppair(ic3,3) = c_nppair(ic3,3) + 1
            np = c_nppair(ic3,3) + 3
            c_nppair(ic3,np) = ic 

c Assign CATEGORY and STEADY STATE INDEX to secondary species 
c  also assign to all its subspecies, including aqueous equil. species.

c           do neq=1, (c_nequil(ic )+1)
c             icc = ic
c             if(neq.gt.1) icc = c_ncequil(ic, (neq-1) )
c             c_nppair(ic ,1) = ic3
c             c_icat(icc) = c_icat(ic3)
c             c_lsts(icc) = c_lsts(ic3)
c           enddo

          enddo
c End Assign loop

        endif
c End error check if

      enddo
c END LOOP THROUGH CASCADE PAIRS
c
c -----------------------------------------
c END GENERATE POINTERS FOR CASCADE PAIRS
c -----------------------------------------

c -------------------------------------------------------
c RUN CASCADE - IDENTIFY MISSING SPECIES.  
c   ALSO IDENTIFY MAX. NUMBER OF SPECIES SOLVED TOGETHER (NCDIM)
c   AND MAXIMUM MULTISOLVE GROUP (NSDIM)
c TEST CATEGORY LIST.  VARIABLES W/O CATEGORIES ARE ASSIGNED 'xx'
c  AND A WARNING ISSUED
c -------------------------------------------------------
c
c  Set LCASTEST = false initially
c   Set LCASTEST = true for H+, OH-, CO2, H2O  
c                   - these should never be in cascade
      do ic=1,c_nchem2
        lcastest(ic) = .FALSE.
        if(ic.eq.c_aqueous(1,2).or.ic.eq.c_aqueous(1,3)) 
     *        lcastest(ic) = .TRUE.
        if(c_tchem(c_npequil(ic )).eq.'     CO2') lcastest(ic) = .TRUE.
        if(c_tchem(c_npequil(ic )).eq.'     H2O') lcastest(ic) = .TRUE.
      enddo            !do ic=1,c_nchem2

c   Set LCASTEST = true for LUMPED SPECIES - these are never included.
      do     i=1,nlump
         icc=c_lump(i,1)
         lcastest(icc)=.TRUE.
      enddo

c SET COUNTERS (NSOL, NSOLV) AND MAXIMA (NCDIM, NSDIM)
      ncdim = 1
      nsdim = 1

c TEST WRITE CASCADE
c     if(c_kkw.gt.0) write(c_out,347)
c 347   format(/,'CASCADE TEST: LIST OF SPECIES IN ORDER OF SOLUTION')

c  RUN  CASCADE.  IDENTIFY INCLUDED SPECIES.  
c   COUNT SPECIES IN CASCADE GROUP.   ALSO ASSIGN NMULTI
c
      do     i=1,c_nchem2
        if(c_cascade(i,1).eq.0) go to 332   
        nsolv = 0
        nsol = 0
c--------Guangxing Lin---03032013---debug only---
c        do     ii=1, 3
c--------Guangxing Lin---03032013---debug only---
        do     ii=1,c_nchem2
          ics =c_cascade(i,ii)
c--------Guangxing Lin---03032013---debug only---
          if (ics.lt.0) then
           write(6,*) "gxlin--ics:,i, ii ", i, ii
          end if 
c--------Guangxing Lin---03032013---debug only---

           
          if(ics.eq.0) go to 331
          ics =c_nppair(c_npequil(ics ) , 2)
          if(ii.eq.1)  ic1=ics
          nsol = nsol + 1
          nsolv = nsolv + 1 + c_nppair(ics,3)

          do n=1,(c_nppair(ics,3)+1)
            icc = ics
            if(n.gt.1) then
              icc=c_nppair(ics,n+2)
            endif

c TEST WRITE CASCADE
               if(c_kkw.ge.1.and.icc.gt.0) write(c_out,348)n, 
     *                                              c_tchem(icc)
348          format(i4,4x,a8)

            do  neq=1,(c_nequil(icc)+1)
              ic=icc
              if(neq.gt.1) ic = c_ncequil(icc,(neq-1))
              lcastest(ic) = .TRUE.
              c_npmulti(ic,1) = ic1
            enddo                  !do  neq=1,(c_nequil(icc)+1)
          enddo                  !do n=1,(c_nppair(ic,3)+1)
        enddo                !do     ii=1,c_nchem2
331     continue
        if(ncdim.lt.nsolv) ncdim=nsolv
        if(nsdim.lt.nsol)  nsdim=nsol
      enddo                !do     i=1,c_nchem2
332   continue

c  INCLUDE FAMILY SPECIES, SAME AS FOR CASCADE.
c
      do     i=1,9
        nsolv = 0
        nsol = 0
        do ii=1,3
         if(i.eq.1.and.ii.eq.1) ics=c_noh
         if(i.eq.1.and.ii.eq.2) ics=c_nho2
         if(i.eq.1.and.ii.eq.3) ics=c_nh2o2
         if(i.eq.2.and.ii.eq.1) ics=c_no3
         if(i.eq.2.and.ii.eq.2) ics=c_nno2
         if(i.eq.2.and.ii.eq.3) ics=c_nno
         if(i.eq.3.and.ii.eq.1) ics=c_nno3
         if(i.eq.3.and.ii.eq.2) ics=c_nn2o5
         if(i.eq.3.and.ii.eq.3) ics=c_nhno3

         if(ics.gt.0) then
          ics =c_nppair(c_npequil(ics ) , 2)
          nsol = nsol + 1
          nsolv = nsolv + 1 + c_nppair(ics,3)

          do n=1,(c_nppair(ics,3)+1)
            icc = ics
            if(n.gt.1) then
              icc=c_nppair(ics,n+2)
            endif
            do  neq=1,(c_nequil(icc)+1)
              ic=icc
              if(neq.gt.1) ic = c_ncequil(icc,(neq-1))
              lcastest(ic) = .TRUE.
            enddo                  !do  neq=1,(c_nequil(icc)+1)
          enddo                  !do n=1,(c_nppair(ic,3)+1)
         endif                  !if(ics.gt.0) then

        enddo         !do ii=1,3
        if(ncdim.lt.nsolv) ncdim=nsolv
        if(nsdim.lt.nsol)  nsdim=nsol
      enddo                !do     i=1,3
c
c TEST WRITE
      if(c_kkw.gt.0) write(c_out,336) ncdim, nsdim
336   format(/,' MAXIMUM CASCADE SPECIES, PAIR GROUPS = ', 2i5)

       if(c_kkw.eq.1.or.c_kkw.eq.5) write(c_out,342)
342    format(/'TEST IC CHEM ICAT ISTS  NEQUIL PRIMARY  NPAIR PRIMARY')

c CATEGORY TEST LOOP AMONG SPECIES.
c   IDENTIFY SPECIES MISSING FROM CASCADE AND MISSING CATEGORIES. 

       do ic=1,c_nchem2
c  Control for diagnostic write
         if(c_kkw.eq.1.or.c_kkw.eq.5) then
           write(c_out,343) ic, c_tchem(ic), c_icat(ic), c_lsts(ic),
     *       c_nequil(ic),c_npequil(ic),  c_nppair(ic,3)
     *       , c_nppair(ic,2)  ,c_nppair(ic,1)
343         format(i4,2x,a8,2x,i5,l4,2x,8i5)
           if(c_npequil(ic).gt.0) then
             write(c_out,309) c_tchem(c_npequil(ic))
           endif
           if(c_nppair(ic,2).gt.0) then
             write(c_out,309) c_tchem(c_nppair(ic,2))
           endif
           if(c_nppair(ic,1).gt.0) then
             write(c_out,309) c_tchem(c_nppair(ic,1))
           endif
           if(c_nequil(ic).gt.0) then
             write(c_out,344) (c_ncequil(ic,i),i=1,c_nequil(ic))
344          format(' c_ncequil(aqueous subspecies) = ',10i5)
             do i=1, c_nequil(ic)
               if(c_ncequil(ic,i).gt.0) then
                 write(c_out,309) c_tchem(c_ncequil(ic,i))
               endif
             enddo
           endif
           if(c_nppair(ic,3).gt.0) then
             write(c_out,345) ( c_nppair(ic,i),i=4,(3+c_nppair(ic,3)) )
345          format(' ncpair (chem. paired subspecies)  = ',10i5)
             do i=4, (3+c_nppair(ic,3))
               if(c_nppair(ic,i).gt.0) then
                 write(c_out,309) c_tchem(c_nppair(ic,i))
               endif
             enddo
           endif
         endif
c End control for diagnostic write.

         if(c_icat(ic).le.0) then
          write(c_out,341) ic,c_tchem(ic), c_icat(ic)
 341      format(/,' WARNING:  MISSING SPECIES CATEGORY',/,
     *   'SPECIES =',i4,2x,a8,'  CATEGORY=',i4)
c         c_icat(ic) = 23
         endif

         if(.not.lcastest(ic))
     *    write(c_out,352) ic,c_tchem(ic), c_npequil(ic)
     *  ,c_tchem(c_npequil(ic))
 352      format(/,' WARNING:  SPECIES OMITTED FROM CASCADE',/,
     *    'SPECIES =',i4,4x,a8,'  NPEQUIL =',i4,2x, a8)

       enddo     !do ic=1,c_nchem2
c END OF CATEGORY TEST LOOP
c
c -------------------------------------------------------
c CASCADE TEST:  RUN THROUGH CASCADE TO MAKE SURE THAT
c THERE ARE NO SPECIES SOURCES AFTER THE SPECIES HAS BEEN 
c SOLVED FOR. 
c
c NOTE:  IN REVISED VERSION, PRODUCTION FURTHER DOWN THE CHAIN
c  IS COUNTED IN THE NEXT ITERATION.
c  (This is retained as a warning for sources of nonconvergence only)
c
c  OLD NOTE:  THE 'PRE' LABEL IN THE CASCADE RUNS REACTIONS ASSOCIATED
c  WITH A SPECIES USING PRIOR VALUES. THIS OBTAINS REACTION PRODUCTS
c  EVEN BEFORE THE SPECIES HAS BEEN SOLVED FOR (SLOW SPEC. ONLY.)
c  BUT THERE MUST BE NO FURTHER PRODUCTION AFTER SOLVE. 

       do 350 i=1,c_nchem2
        lcastest(i) = .FALSE.
350    continue

c RUN CASCADE.  SET INDEX=1 AS EACH SPECIES IS SOLVED FOR.
c ( DO WHILE NCASCADE(1).NE.0) 

c LOOP:  RUN CASCADE
      do     i=1,c_cdim
       if(c_cascade(i,1).eq.0) go to 401

c LOOP:  TEST REACTIONS CALLED BY THE CASCADE SPECIES
       do     ii=1,c_cdim
        icc=c_cascade(i,ii)
        if(icc.gt.0) then
         icc=c_npequil(      (icc))
         ic=icc
c
c NOTE PROBLEM HERE (write 419): TEST OF REACTANTS AND PRODUCTS
c   BELONGS IN QUADINIT

c FIRST TEST REACTANTS AND PRODUCTS FROM CASCADE REACTIONS.
c  EXCEPT FOR 'POST' FLAG, PRODUCTS SHOULD BE ALL DOWN-CASCADE.
c  CASCADE MODIFIED TO REFLECT ALL CHEMICAL PAIRS 
c                                ->POINT TO PRIMARY SPEC.
          if(c_cascade(i,2).ne.-2) then
            nnr=c_nnrchem(ic)
            if(nnr.gt.0) then
              do 420 n=1,nnr
               nr= c_nrchem(ic,n)
               if(nr.gt.0) then
                 icr1=c_reactant(nr,1)
                 if(icr1.gt.0) then
                   icr1=c_nppair( c_npequil(icr1), 2 )
                 endif
                 icr2=c_reactant(nr,2)
                 if(icr2.gt.0) then
                   icr2=c_nppair( c_npequil(icr2), 2 )
                 endif
                 if(c_nnpro(nr).gt.0) then
                   do 425 nn=1,c_nnpro(nr)
                     icp=c_product(nr,nn)
                     if(icp.gt.0) then
                       icp=c_nppair( c_npequil(icp), 2 )
                     endif
                     if(icp.eq.icr1) then
                       icr1=0
                       icp=0
                     endif
                     if(icp.eq.icr2) then
                       icr2=0
                       icp=0
                     endif
c PRODUCT TEST       
                      if(icp.gt.0) then
                         if(lcastest(icp)) then
                           write(c_out,419) 
     *                        c_tchem(icp),nr,c_tchem(ic)
     *                            ,(c_treac(jj,nr),jj=1,5)
                           write(   6,419) 
     *                        c_tchem(icp),nr,c_tchem(ic)
     *                            ,(c_treac(jj,nr),jj=1,5)
                        endif
                      endif
425                continue
                 endif
c REACTANT TEST
                 if(icr1.gt.0) then
                   if(lcastest(icr1)) then
                      write(c_out,419) 
     *                  c_tchem(icr1),nr,c_tchem(ic)
     *                    ,(c_treac(jj,nr),jj=1,5)
                      write(   6,419) 
     *                  c_tchem(icr1),nr,c_tchem(ic)
     *                    ,(c_treac(jj,nr),jj=1,5)
                   endif
                 endif
                 if(icr2.gt.0) then
                   if(lcastest(icr2))then
                     write(c_out,419) 
     *                  c_tchem(icr2),nr,c_tchem(ic)
     *                    ,(c_treac(jj,nr),jj=1,5)
                     write(   6,419) 
     *                  c_tchem(icr2),nr,c_tchem(ic)
     *                    ,(c_treac(jj,nr),jj=1,5)
                   endif
                 endif

               endif
420           continue
            endif
          endif

c LAST, ADD CASCADE REACTIONS TO REACTION LIST, UNLESS 'pre' FLAG
          if(c_cascade(i,2).ne.-1) then
             if(lcastest(ic)) then 
               write(c_out,418) c_tchem(ic),i, (c_cascade(i,jj),jj=1,3)
               write(   6,418) c_tchem(ic),i, (c_cascade(i,jj),jj=1,3)
             endif
             lcastest(ic) = .TRUE.
          endif

        endif
       enddo    
c END LOOP:  TEST REACTIONS CALLED BY THE CASCADE SPECIES
  
      enddo    
c END LOOP:  RUN CASCADE

401   continue

419   format(/,'WARNING:  CASCADE OUT OF SEQUENCE FOR SPECIES NAME=  ',
     *  a8,/,'PRODUCED AT REACTION #',i5,'  CALLED FOR SPECIES= ',
     *  a8,/,' REACTION: ',a8,'+',a8,'=>',a8,'+',a8,'+',a8)

418   format(/,'WARNING:  SPECIES CALLED TWICE IN CASCADE. SPECIES=',
     *  a8,/,' SECOND CALL AT CASCADE#',i3,' WITH INDICES=',3i3)


c -------------------------------------------------------
c END CASCADE TEST
c -------------------------------------------------------

c OPTION:  call cheminit here to set initial chemistry parameters

c ----------------------------------------------------------------
c  END READ.
c ----------------------------------------------------------------

c END CHEMREAD
2000  return
      end
c ----------------------------------------------------------------



       function namechem(titl)
c      integer :: function namechem(titl)
c
c INPUT:  chemical species name, char*8
c OUTPUT: Species index number (integer ::)  associated with the name, 
c           or zero if not found in species list.  
c
c ---------------------------------------------
c History:
c  12/06 Written by Sandy Sillman from boxchemv7.f
c
c -------------------------------------------------------------------
c
c
      use h_rbchemmech
      use h_rbchemlocal

      IMPLICIT NONE

c#     include "rbchemmech.h"
c #     INCLUDE "rbchemvars.h"  
c#     include "rbchemlocal.h"


      character*8 :: titl
      integer :: namechem

       namechem = 0
       if(titl.eq.'        ') return
       do 100 i=1,c_nchem2
        if(titl.eq.c_tchem(i)) then
          namechem = i
          go to 101
        endif
100    continue
101    continue

c END NAMECHEM
2000    return
        end
c ----------------------------------------------------------------

c --------------------------------------------------------------
c END chemread.f
c



c ================================================================
