      module h_rbchemlocarr
      use h_rbchemmech, only: c_kvec, c_cdim
      implicit none
      public 
c=============================================================================
c
c $Id: rbchemlocarr.h,v 1.4 2002/05/02 15:50:22 akinorii Exp $
c
c CODE DEVELOPER
c   Sandy Sillman, U of M. (IMPACT version, 2007)
c
c FILE
c   rbchemlocarr.h
c
c DESCRIPTION:  variable declaration for allocated arrays
c   that are locally shared among subroutines in the rb (quad) chem solver.
c   These are allocated in main quad solver program and passed to subroutines.
c
c  They are declared and allocated in the main chem. solver routine
c    (quadchem.F in IMPACT)
c  This file is included in subsequent subroutines
c
c  NO COMMON BLOCKS.  

c ==========================================================================
c 
c ichemlocarr.h   from  chemlocal.EXT    April, 2007
c
c   INCLUDE and COMMON file 
c     for RADICAL BALANCE-BACK EULER solver for chemistry (quadchem)
c      (chemmain.f cheminit.f chemrates chemsolve.f, linslv.f, jval2.f)
c    IMPACT version.
c
c  Note:  This file uses KVEC.  
c    Its include statement must come AFTER chemmech.EXT
c
c  This file contains LOCAL ARRAYS - used internally in chem. solver.
c   Arrays were separated out from chemlocal.h
c   So that they may be ALLOCATED.  
C   Allocate in ichemsolve.f;  use this include file in other subroutines.
c
c  This file includes declarations and common blocks 
c   for SOLVER VARIABLES 
c   which are shared among the solver subroutines in common blocks
c   but are not saved or passed to the main program.  

c  It also contains declarations for LOCAL VARIABLES
c   which are used in solver subroutines but not retained 
c   or passed to other subroutines  
c
c  Other INCLUDE files: 
c   chemvars.EXT =  input-output variables 
c    that are passed to and from the solver each time it is called.
c   chemmech.EXT =  variables for the chem. mechanism
c     that are set once and do not change as the mechanism is called.
c
c Program written by Sanford Sillman
c History:
c  12/06. Program written by Sandy Sillman based on commq7.EXT
c  7/07  IMPACT version (.h), arrays separated into ichemlocarr.h
c -------------------------------------------------------------
c
c LOCAL SPECIES CONCENTRATIONS 
c     (see also input-output variables in chemvars.EXT)
c
c  Note:  Allocate
c
c xc:         average species concentration over time step, molec/cm3
c                 (internally converted to mol/lit-atm for aqueous)
c                 (note: in expo decay solution, 
c              this is distinguished from final species concentration)
c
c xcfinr:   Ratio of FINAL (end of time step) to AVERAGE concentration
c             (=1 for back-Euler. Used for expo decay solution.
c
c xclastq : OPTION, xc after last aquasolve,  used in next aquasolve
c 
       real*8 :: xc( c_kvec,c_cdim) ! concentration molec/cm3
       real*8 :: xcfinr( c_kvec,c_cdim) ! ratio FINAL/AVG cncn
       real*8 :: xclastq( c_kvec,c_cdim) ! conc molec/cm3

c Cut common for Allocated and passed-through species
c      common/CHEMCONC/ xc,xcfinr,  xclastq

c CHEMICAL REACTION RATES
c   Note:  Allocate in chemmain1. or chemsolve1
c    (ratek rateh rateq rateqq in chemmain1 if rates called here)
c     (see also rate parameters in chemmech.EXT
c       and optional output in chemvars.EXT)

c  ratek     Rate constants (s-1 or cm3 s-1 or equiv. aqueous)
c    (c_rk     Parameters for rate constant calculation in chemmech)
c    (c_rr     Calculated rate in solver: ratek*C1*C2 in chemvars)
c  rateh     Henry's law coefficients Moles/liter-atm
c    (c_rkh    Parameters for Henry's law coefficients in chemmech)
c  rateq     Aqueous equilibrium coefficients Moles/liter
c    (c_rkq    Parameters for aqueous eq. coefficients in chemmech)
c  rateqq    Special equilibrium coefficients Moles/liter
c    (c_rkqq   Parameters for Special eq. coefficients in chemmech)
c  rhdif     Henry's law coeff. modified by droplet diffusion
c  egasaq    Gas-to-aqueous transfer coefficient, s-1
c              (Varies for reaction based on accom, molwt)
c              (This is either input or calculated internally)
c   (c_gasaq   Input gas-aq transfer coefficient in chemvars)
c
croy      real*8 :: ratek(c_kvec,c_rdim)  ! Rate constants
croy      real*8 :: rateh(c_kvec,c_rdim)  ! Henry's law constants
croy      real*8 :: rateq(c_kvec,c_rdim)  ! Aqueous eq. constants
croy      real*8 :: rateqq(c_kvec,c_rdim) ! Special eq. constants
croy      real*8 :: rhdif(c_kvec,c_rdim) ! H. mod by diffusion
croy      real*8 :: egasaq(c_kvec,c_rdim) ! Gas-to-aq transfer s-1

c Cut common for Allocated and passed-through species
c     common/CHEMRATEK/ ratek, rateh, rateq, rateqq, rhdif, egasaq
c
c
c LOCAL CHEMICAL SOLVER VARIABLES:  
c PRODUCTION AND LOSS RATES AND INTERIM CONCENTRATIONS
c SINGLE SPECIES AND LINKED PAIR SOLUTIONS
c  (OPTION - iteration counter is in common for written output)
c
c   Note:  Allocate?  But except for history, geomavg, 
c      these may be local in chemsolve
c
c cpm(c_kvec,is1,is2)    Cross production is1->is2 for multi-group
c                    conversion from species 1 to species 2, molec/cm3
c                         (is1, is2 are indices w/in multi group)
c cpro(c_kvec,is1,is2) Cross-production for pair species:
c              conversion from species 1 to species 2, molec/cm3/tstep
c
c geomavg(kk, ic)   Geometric average adjustment factor 
c                      for difficult convergence.
c                    Solution is geom. avg of current and prior iter,
c                      using geomavg factor, adjusted based on history.
c                      (1= current iter; 0.5=pure geometric avg)
c history(kk, ic, iter) Solution for concentration of species ic
c                         in each iteration, molec/cm3
c                         (used in geometic weighting adjustment)
c
c rlm(c_kvec,c_cdim)     Summed loss for multi group, molec/cm3/timestep
c                         (including inter-group conversion)
c rlmulti(c_kvec       ) Combined loss rate for multi group of species
c                         (Omitting inter-group conversion)
c rloss(c_kvec,c_cdim)  Solver loss rate + final (nonstst), molec/cm3
c rloss1(c_kvec,c_cdim)  Interim rloss without cpro pairsol. adjustment
c rlpair(c_kvec,c_cdim)  Net loss rate for pair group`(mol/cm3/timestep)
c                          weighted by pairfac to conserve pair mass
c
c rpm(c_kvec,c_cdim)     Summed production for all of multi group
c                         (including inter-group conversion)
c rpmulti(c_kvec       ) Combined production rate for multi group`
c                         (Omitting inter-group conversion)
c rppair(c_kvec,c_cdim)  Net production rate for pair group`
c                          weighted by pairfac to conserve pair mass
c rpro(c_kvec,c_cdim)   Solver production rate+initial concentration, 
c                                           molec/cm3/timestep
c rpro1(c_kvec,c_cdim)  Interim rpro w/out cpro pair production
c
c rrp(c_kvec,c_cdim)   Continuous production sum thru cascade, molec/cm3
c rrl(c_kvec,c_cdim)   Continuous loss sum thru cascade,  molec/cm3
c 
c xrm(c_kvec,c_cdim)     Initial concentration for each member of  
c                                      multi-species group, molec/cm3
c                           (used in chemsolve and noxsolve)
c xrp(c_kvec, c_cdim)   Solver initial concentration for single species
c    or for each species in group for paired solution, molec/cm3
c xrppair(c_kvec, c_cdim)  Solver initial concentration for pair group
c    sum, weighted by pairfac to conserve pair mass.  molec/cm3
c       (used in chemsolve only, not nec. in common)
c xrr(c_kvec, c_cdim)   Solver interim solution      for single species
c    or for each species in group for paired solution, molec/cm3
c xrrm(c_kvec,c_cdim)    Initial solution      for each member of  
c               multi-species group (before normalization), molec/cm3
c xrrpair(c_kvec, c_cdim)  Solver solution for pair group sum
c              (weigted by pairfac to conserve mass). molec/cm3
c               used in normalization of indiv. species solutions
c       (used in chemsolve only, not nec. in common)
c
c 
      real*8 :: cpm(c_kvec, c_cdim, c_cdim) !Cross-prod for multi
      real*8 :: cpro(c_kvec,c_cdim, c_cdim)   ! Cross-production 
c 
      real*8 :: history(c_kvec,c_cdim,400)  ! Cncn for each iter
      real*8 :: geomavg(c_kvec,c_cdim)   ! Geom. avg factor
c
      real*8 :: rlm(c_kvec,c_cdim)     ! Loss for multi-spec 
      real*8 :: rlmulti(c_kvec       ) ! Loss group sum for multi
      real*8 :: rloss(c_kvec,c_cdim)   ! Solver loss rate
      real*8 :: rloss1(c_kvec,c_cdim)  ! Interim rloss w/out cpro
      real*8 :: rlpair(c_kvec,c_cdim)  ! Loss rate for pair grp
      real*8 :: rpm(c_kvec,c_cdim)     ! Production rate for multi
      real*8 :: rpmulti(c_kvec       ) !Product'n multi group sum
      real*8 :: rppair(c_kvec,c_cdim)  ! Production for pair grp
      real*8 :: rpro(c_kvec,c_cdim)    ! Solver production rate
      real*8 :: rpro1(c_kvec,c_cdim)   ! Interim rpro w/out cpro 
      real*8 :: rrp(c_kvec,c_cdim)     ! Interim production rate 
      real*8 :: rrl(c_kvec,c_cdim)     ! Interim loss rate 
c 
      real*8 :: xrm(c_kvec,c_cdim)     !  Initial conc. for multi
      real*8 :: xrp(c_kvec,c_cdim)     !Solver prior concentration
      real*8 :: xrppair(c_kvec)        ! Solver prior pair group cn.
      real*8 :: xrr(c_kvec,c_cdim)     ! Solver concentration
      real*8 :: xrrm(c_kvec,c_cdim)    !Initial solution for multi
      real*8 :: xrrpair(c_kvec)        ! Solution for pair group sum

c OPTION - iteration counter is in common for output, may cut from common.
c          -> also in  chemvars as c_iter (to record output)
c     integer :: iter                            ! Iteration counter

c LCHEMRATES,  LREAC  

c Cut common for Allocated and passed-through species
c     common/CHEMSOLVARS/ xrp, xrr, rpro, rloss, rpro1, rloss1 
c    *      ,rrp, rrl, cpro, 
c    *      ,xrm, xrrm, rpm, rlm,cpm, rpmulti, rlmulti
c    *      ,rppair, rlpair, xrppair, xrrpair
c    *      ,history, geomavg


c VARIABLES ASSOCIATED WITH ODD HYDROGEN RADICALS AND ODD NITROGEN
c
c ----------------------------------------
c END OF chemlocarr.h   
c ----------------------------------------
c ================================================================
       end module h_rbchemlocarr
