c=============================================================================
c
c $Id: maxran.F,v 1.1 2003/05/16 22:22:45 akinorii Exp $
c
c FILE
c   maxran.F
c
c ROUTINES
c   maxran
c
c=============================================================================


      Subroutine maxran
     &     (COLMAX,nz,cldfra,ncol,cldinx,colfra,ncolx,sumcv)
c **************************************************************************
c Use Quick Sort
      
c *This subroutine is used to compute the number of possible cases in Maximum-
c *Randomly overlapping scheme for one column; also returns cloud index (=1
c *cloudy and =0 clear) on each level for every case from bottom to top.

c COLMAX is maximum number of returning cases with largest possibilities among
c all possible cases.      

c Author: Yan Feng, University of Michigan
c Date: 09.2002      
c **************************************************************************
      use h_imp_dims
      implicit none

c#     include "imp_dims.h"

c input      
      integer :: COLMAX
      integer :: nz
      real*8 :: cldfra(k1:k2)
      integer :: il,ij
c output
      integer :: ncol   !total counted cases(<=15)
      integer :: ncolx  !total cases
      real*8 :: cldinx(k1:k2,1:COLMAX)
      real*8 :: colfra(1:COLMAX)
      real*8 :: sumcv !total counted fractions

c workspace
      real*8, parameter :: er=1.0d-6
      real*8 :: cf(1:nz-1)         !bottom to top
      real*8 ::
     &     cldx(1:nz-1,1:COLMAX),     !bottom to top
     &     pos(1:COLMAX)

      real*8, parameter ::
     &     maxcv = 1.0d0, mincv = 1.0d-10     !total maximum cloud cover
                                             !minimum cloud cover   
      integer :: icol,icol0,
     &     ig,ic,ik,iz,
     &     ic0,nig,ncol0,knc0,ic1
      real*8 :: gcinx(1:nz-1, 1:nz, 1:Nint(nz/2.+1)),
     &     gnz(1:Nint(nz/2.+1)),
     &     gnc(1:Nint(nz/2.+1)),
     &     gknc(1: nz-1),
     &     gcfra(1:nz, 1:Nint(nz/2.+1)),
     &     gcfra0(1:nz-1, 1:nz),
     &     gcinx0(1:nz-1, 1:nz),
     &     cf0
      integer :: indx(1:100*COLMAX)
      real*8 :: pos0(1:100*COLMAX),cldx0(1:nz-1, 1:100*COLMAX)
      logical :: Is_Gt,Is_Lt,Is_Eq
      integer :: iz2

c* Initialization      
c      write(6,*) 'maxran called by', loc_proc, 'il= ',il,' ij= ',ij
      do iz=1,nz-1
         cf(iz) = cldfra(k1+iz-1)
c         if(loc_proc.eq.12) then
c            write(6,*) 'cf(',iz,')= ',cf(iz)
c         endif
      enddo
c      if(loc_proc.eq.12) then
c         write(98,*) 'il= ', il,' ij= ',ij
c         do iz=1,nz-1
c            write(98,*) 'cf(',iz,')= ',cf(iz)
c         enddo
c      endif

      icol=0
      ncol=0
      do icol=1,COLMAX
        do iz=1,nz-1
          cldx(iz,icol)=0.0d0
        enddo
        pos(icol)=0.0d0
      enddo      

      do ig=1,nz/2.+1
        do ic=1,nz
          do ik=1,nz-1
            gcinx(ik,ic,ig)=0.d0
            gknc(ik)=0.d0
            gcfra0(ik,ic)=0.d0
            gcinx0(ik,ic)=0.d0
          enddo
          gcfra(ic,ig)=0.d0
        enddo
        gnz(ig)=0.d0
        gnc(ig)=0.d0
      enddo

c* The bottom layer
      nig=1
      ig=1
      ik=1
      iz=1
      if(Is_Gt(cf(iz),0.d0,er).and.Is_Lt(cf(iz),1.d0,er)) then
        gknc(ik)=2.d0
        gcinx0(ik,1)=1.d0
        gcinx0(ik,2)=0.d0
        gcfra0(ik,1)=cf(iz)
        gcfra0(ik,2)=1.d0-cf(iz)
      elseif(Is_Eq(cf(iz),1.d0,er)) then
        gknc(ik)=1.d0
        gcinx0(ik,1)=1.d0
        gcfra0(ik,1)=cf(iz)
      else
        gknc(ik)=1.d0
        gcinx0(ik,1)=0.d0
        gcfra0(ik,1)=1.d0-cf(iz)        
      endif
      gnz(ig)=1.d0

c* Bottom-up        
      do iz=2,nz-1
        if(Is_Gt(cf(iz),0.d0,er)) then
          if(Is_Eq(cf(iz-1),0.d0,er)) then
c-----------wrap up the last group          
            gnc(ig)=gknc(gnz(ig))
            do ic=1,gnc(ig)
              gcfra(ic,ig)=gcfra0(gnz(ig),ic)
              gcinx(gnz(ig),ic,ig)=gcinx0(gnz(ig),ic)
            enddo
c            do ik=1, gnz(ig)
c              write(6,'(6e12.4)') (gcinx0(ik,ic),ic=1,gknc(ik))
c              write(6,*) 
c            enddo
            do ik=gnz(ig)-1,1,-1
              ic0=0
              do ic=1,gknc(ik)
 10             ic0=ic0+1
                   
                if(Is_Gt(gcfra0(ik,ic),gcfra0(gnz(ig),ic0),er)) then
                  gcinx(ik,ic0,ig)=gcinx0(ik,ic)
                  gcfra0(ik,ic)=gcfra0(ik,ic)-gcfra0(gnz(ig),ic0)
                  goto 10
           elseif(Is_Eq(gcfra0(ik,ic),gcfra0(gnz(ig),ic0),er)) then
                  gcinx(ik,ic0,ig)=gcinx0(ik,ic)                  
                else
                  write(6,113) 
     &  gcfra0(ik,ic),gcfra0(gnz(ig),ic0), loc_proc
                  stop
                endif           !gcfra0(ik,ic) v.s. gcfra0(ik+1,ic)
              enddo             !ic
            enddo               !ik

c-----------start a new group            
            ig=ig+1
            nig=nig+1
            do ic=1,nz
              do ik=1,nz-1
                gknc(ik)=0.d0
                gcfra0(ik,ic)=0.d0
                gcinx0(ik,ic)=0.d0
              enddo
            enddo
            ik=1
            gnz(ig)=1.d0
            if(Is_Eq(cf(iz),1.d0,er)) then
              gknc(ik)=1.d0
              gcinx0(ik,1)=1.d0
              gcfra0(ik,1)=cf(iz)
            else
              gknc(ik)=2.d0
              gcinx0(ik,1)=1.d0
              gcfra0(ik,1)=cf(iz)
              gcfra0(ik,2)=1.d0-cf(iz)
            endif
          elseif(Is_Gt(cf(iz-1),0.d0,er)) then
            ik=ik+1
            gnz(ig)=gnz(ig)+1.d0
            cf0=cf(iz)
            do ic=1,gknc(ik-1)
              if(Is_Eq(cf0,0.d0,er)) then
                gcfra0(ik,ic)=gcfra0(ik-1,ic)
              elseif(Is_Eq(cf0,-999.d0,er)) then
                gcfra0(ik,ic+1)=gcfra0(ik-1,ic)
              elseif(Is_Gt(cf0,0.d0,er)) then
                if(Is_Eq(cf0,gcfra0(ik-1,ic),er)) then
                  gknc(ik)=gknc(ik-1)
                  gcinx0(ik,ic)=1.d0
                  gcfra0(ik,ic)=gcfra0(ik-1,ic)!cf0
                  cf0=0.d0
                elseif(Is_Lt(cf0,gcfra0(ik-1,ic),er)) then
                  gcinx0(ik,ic)=1.d0
                  gcfra0(ik,ic)=cf0
                  gcfra0(ik,ic+1)=gcfra0(ik-1,ic)-cf0
                  cf0=-999.d0
                  gknc(ik)=gknc(ik-1)+1.d0
                else
                  gcinx0(ik,ic)=1.d0
                  gcfra0(ik,ic)=gcfra0(ik-1,ic)
                  cf0=cf0-gcfra0(ik-1,ic)
                endif           !cf0 v.s. gcfra(ik-1,ic,ig)
              else
                write(6,111) cf0,loc_proc
                stop
              endif             !cf0 v.s. 0.
            enddo               !do ic
          endif                 !cf(iz-1)
        elseif(Is_Eq(cf(iz),0.d0,er)) then
          ik=ik+1
          gnz(ig)=gnz(ig)+1
          gknc(ik)=gknc(ik-1)
          do ic=1,gknc(ik)
            gcfra0(ik,ic)=gcfra0(ik-1,ic)
          enddo          
        else
          write(6,112) cf(iz),loc_proc
          stop
        endif                   !cf(iz)
      enddo                     !do iz

c top layer, wrap up the last group   
      gnc(ig)=gknc(gnz(ig))
      do ic=1,gnc(ig)
        gcfra(ic,ig)=gcfra0(gnz(ig),ic)
        gcinx(gnz(ig),ic,ig)=gcinx0(gnz(ig),ic)
      enddo
      do ik=gnz(ig)-1,1,-1
        ic0=0
        do ic=1,gknc(ik)
 20       ic0=ic0+1

          if(Is_Gt(gcfra0(ik,ic),gcfra0(gnz(ig),ic0),er)) then
            gcinx(ik,ic0,ig)=gcinx0(ik,ic)
            gcfra0(ik,ic)=gcfra0(ik,ic)-gcfra0(gnz(ig),ic0)
            goto 20
         elseif(Is_Eq(gcfra0(ik,ic),gcfra0(gnz(ig),ic0),er)) then
            gcinx(ik,ic0,ig)=gcinx0(ik,ic)
          else
            write(6,113) gcfra0(ik,ic),gcfra0(gnz(ig),ic0),loc_proc
            stop
          endif                 !gcfra0(ik,ic) v.s. gcfra0(ik+1,ic)
        enddo                   !ic        
      enddo                     !ik

c randomly groups the maxium overlapping clouds.
      ncol=1
      do ig=1,nig
        ncol=ncol*gnc(ig)
      enddo
      if(gcfra(ic,1)*gcfra(ic,2).gt.100.*COLMAX) then
        write(6,114)
        stop
      endif
c      write(6,*) 'COLMAX: ', COLMAX
c group 1      
      do ic=1,gnc(1)        
        pos0(ic)=gcfra(ic,1)        
      enddo
      if(gnc(1).gt.COLMAX) then
        call indexx(Nint(gnc(1)),pos0,indx)
        ncol0=COLMAX
        sumcv=0.d0
        do ic=1,ncol0
          pos(ic)=gcfra(indx(ic),1)          
          do ik=1,gnz(1)
            cldx(ik,ic)=gcinx(ik,indx(ic),1)
          enddo
          sumcv=sumcv+pos(ic)
          if(sumcv.ge.maxcv) goto 30          
        enddo
        ic=ic-1
 30     ncol0=ic
      else
        ncol0=gnc(1)
        ic1=1
        do ic=1,gnc(1)
          if(gcfra(ic,1).lt.mincv) goto 35
          pos(ic1)=gcfra(ic,1)
          do ik=1,gnz(1)
            cldx(ik,ic1)=gcinx(ik,ic,1)
          enddo
          ic1=ic1+1
 35     enddo
        ncol0=ic1-1
      endif
c rest of group      
      knc0=0
      do ig=2,nig
        knc0=knc0+gnz(ig-1)
        icol=1
        if(gnc(ig)*ncol0.gt.100.*COLMAX) then
          write(6,114) loc_proc
          stop
        endif          
        do icol0=1,ncol0
          do ic=1,gnc(ig)
            pos0(icol)=pos(icol0)*gcfra(ic,ig)
            do ik=1,knc0+gnz(ig)
              if(ik.le.knc0) then
                cldx0(ik,icol)=cldx(ik,icol0)
              else
                cldx0(ik,icol)=gcinx(ik-knc0,ic,ig)
              endif
            enddo
            icol=icol+1              
          enddo
        enddo
        ncol0=ncol0*gnc(ig)
c        write(6,*) ig,ncol0,knc0+gnz(ig)
c        write(6,*) ig,(pos0(icol0),icol0=1,ncol0)
c        do ic=1,ncol0
c          write(6,'(20f5.0)') (cldx0(ik,ic),ik=1,knc0+gnz(ig))
c          write(6,*)
c        enddo        
        if(ncol0.gt.COLMAX) then
          call indexx(ncol0,pos0,indx)
c          write(6,*) 'indx = ', (indx(ic),ic=1,ncol0)
          ncol0=COLMAX
          sumcv=0.
          do ic=1,ncol0
            pos(ic)=pos0(indx(ic))
            do ik=1,knc0+gnz(ig)
              cldx(ik,ic)=cldx0(ik,indx(ic))
            enddo
            sumcv=sumcv+pos(ic)
            if(sumcv.ge.maxcv) goto 40            
          enddo
          ic=ic-1
 40       ncol0=ic
        else
          ic1=1
          sumcv=0.d0
          do ic=1,ncol0
            if(pos0(ic).lt.mincv) goto 45            
            pos(ic1)=pos0(ic)
            do ik=1,knc0+gnz(ig)
              cldx(ik,ic1)=cldx0(ik,ic)
            enddo
            sumcv=sumcv+pos(ic1)            
            ic1=ic1+1
 45       enddo
          ncol0=ic1-1
        endif
c        write(6,*) ig,ncol0,sumcv
c        write(6,*) ig,(pos(icol0),icol0=1,ncol0)
c        do ic=1,ncol0
c          write(6,'(20e12.4)') (cldx(ik,ic),ik=1,knc0+gnz(ig))
c          write(6,*)
c        enddo
      enddo

c      open(unit=98,file='maxran.output',status='unknown')
c      write(98,*) 'Groups: ', nig

c      write(98,*) 'Total: ncol=', ncol,' Counted: ncol0=',ncol0
c      write(98,*) 'Total counted cloud area is: ', sumcv
c      write(98,*) 'Minimum cloud cover counted is: ', mincv
      
c      write(6,*) ncol,' cases'
c      do icol=1,ncol
c        write(6,'(e12.4)') pos(icol)
c      enddo

c      write(98,*) ncol0,' cases with largest possibilities (bottom-up)'

      ncolx=ncol
      ncol=ncol0

      do icol=1,ncol
        colfra(icol) = pos(icol)
        do ik=1,nz-1
           cldinx(k1+ik-1,icol) = cldx(ik,icol)
        enddo
      enddo

c      close(98)

 111  format('Intermediate cf is not correct: ', e12.4,
     &     'by ',I3)
 112  format('Cloud fraction is < 0.: ',e12.4,'by ',I3)
 113  format('Cloud fraction of two ajacent layers: ', 2e12.4,'by ',I3)
 114  format('Too many cases. Need to extend size of pos0(100.*COLMAX)',
     &     'by ',I3)
      return
      end

      function Is_Eq(a,b,error)
      implicit none
      logical :: Is_Eq
      real*8 :: a,b
      real*8 :: error
      if(abs(a-b).le.error) then
         Is_Eq = .true.
      else
         Is_Eq = .false.
      endif
      return
      end

      function Is_Gt(a,b,error)
      implicit none
      logical :: Is_Gt
      real*8 :: a,b
      real*8 :: error
      if((a-b).gt.error) then
         Is_Gt = .true.
      else
         Is_Gt = .false.
      endif
      return
      end

      function Is_Lt(a,b,error)
      implicit none
      logical :: Is_Lt
      real*8 :: a,b
      real*8 :: error
      if((b-a).gt.error) then
         Is_Lt = .true.
      else
         Is_Lt = .false.
      endif
      return
      end
      
          
      
      
