 
c=============================================================================
c
c $Id: sad_update.F,v 1.6 2002/10/31 23:54:31 jrt Exp $
c
c CODE DEVELOPERS
c   Condense code originally written by:
c     David B. Considine
c     Code 916
c     NASA Goddard Space Flight Center
c     Greenbelt, MD  20771
c     dbc@welkin.gsfc.nasa.gov
c     301-286-4299
c   LLNL modifications:
c     John Tannahill, LLNL
c     jrt@llnl.gov
c
c FILE
c   sad_update.F
c
c ROUTINES
c   Update_Sad1
c   Update_Sad2
c   Update_Sad3
c   Write_Cond_Args
c
c=============================================================================
 

c-----------------------------------------------------------------------------
c
c ROUTINE
c   Update_Sad3
c
c DESCRIPTION
c   This routine sets the tropospheric sulfuric aerosol surface area
c   densities.
c
c ARGUMENTS
c   const    : species concentration, known at zone centers (mixing ratio)
c   hno3cond : condensed phase hno3 array (mixing ratio)
c   hno3gas  : gas       phase hno3 array (mixing ratio)
c   sadimp   : surface area densities (cm^2/cm^3)
c
c-----------------------------------------------------------------------------

      subroutine Update_Sad3
     &  (pres3c, temp3, const, sadimp)

! Guangxing Lin delete lbssad 
      use h_imp_dims, only: i1,i2, ju1, j2, k1, k2, num_species,
     & num_sad,lbssad_timpyr,ilo,ihi,julo,jhi, pr_diag

      use h_setkin_par
    
      use h_imp_control, only: imgas_num, ih2o_num
      use cam_logfile,   only: iulog

      implicit none

c#     include "imp_dims.h"
c#     include "imp_control.h"
c#     include "setkin_par.h"

c     ----------------------
c     Argument declarations.
c     ----------------------
 
      real*8  :: pres3c(ilo:ihi, julo:jhi, k1:k2)
      real*8  :: temp3 (ilo:ihi, julo:jhi, k1:k2)
      real*8  :: const (i1:i2,   ju1:j2,   k1:k2, num_species)
!      real*8  :: lbssad(i1:i2,   ju1:j2,   k1:k2, lbssad_timpyr)
      real*8  :: sadimp(i1:i2,   ju1:j2,   k1:k2, num_sad)


c     ----------------------
c     Variable declarations.
c     ----------------------

      integer :: idumday, idumyear
      integer :: month,il,ij,ik

      real*8  :: rh(i1:i2, ju1:j2, k1:k2)
      real*8  :: air_density(i1:i2, ju1:j2, k1:k2)
      real*8  :: num_so41(i1:i2, ju1:j2, k1:k2)
      real*8  :: num_so42(i1:i2, ju1:j2, k1:k2)
      real*8  :: num_so43(i1:i2, ju1:j2, k1:k2)
      real*8  :: mas_so41(i1:i2, ju1:j2, k1:k2)
      real*8  :: mas_so42(i1:i2, ju1:j2, k1:k2)
      real*8  :: mas_so43(i1:i2, ju1:j2, k1:k2)
      real*8  :: rv_so41(i1:i2, ju1:j2, k1:k2)
      real*8  :: rv_so42(i1:i2, ju1:j2, k1:k2)
      real*8  :: rv_so43(i1:i2, ju1:j2, k1:k2)
      real*8  :: rm_so41(i1:i2, ju1:j2, k1:k2)
      real*8  :: rm_so42(i1:i2, ju1:j2, k1:k2)
      real*8  :: rm_so43(i1:i2, ju1:j2, k1:k2)
      real*8  :: lbs_sad(i1:i2, ju1:j2, k1:k2)

c     -----------------------
c     Parameter declarations.
c     -----------------------

      real*8, parameter :: pi = 3.1415926d0
      real*8, parameter :: rhosul = 1.769d3   !kg/m^3
      real*8, parameter :: sigmod01 = 1.59d0 !sig of so4n1
      real*8, parameter :: sigmod02 = 1.514d0 !sig of so4n2 
      real*8, parameter :: sigmod03 = 1.77623d0 !sig of so4n2 
  


c     ----------------
c     Begin execution.
c     ----------------
 
c      if (pr_diag) then
c        Write (6,*) 'Update_Sad3 called by ', loc_proc
c      end if


c      call Split_Time_Flds (nymd, idumyear, month, idumday)


c     -----------------------------------------------------------
c     Calculate relative humidity from Seinfeld (1986) p. 181.
c     The first rh is the temperature dependent parameter a.
c     The second rh is the saturation vapor pressure of water.
c     The third rh is the actual relative humidity as a fraction.
c     Then make sure rh is between 0 and 1.
c     -----------------------------------------------------------
!       imgas_num=253 !JIALEI **  

      rh(:,:,:) = 1.0d0 - (373.15d0 / temp3(i1:i2,ju1:j2,:))
      
      rh(:,:,:) =
     &  1013.25d0 * Exp (13.3185d0 * rh(:,:,:)    -
     &                    1.9760d0 * rh(:,:,:)**2 -
     &                    0.6445d0 * rh(:,:,:)**3 -
     &                    0.1299d0 * rh(:,:,:)**4)
 
      rh(:,:,:) =
     &  const(:,:,:,ih2o_num) *
     &  pres3c(i1:i2,ju1:j2,:) / rh(:,:,:)
 
      rh(:,:,:) = Max (Min (rh(:,:,:), 1.0d0), 0.0d0)
 

c--------------------------gxlin-11-10-2009------------------
c     Calculate the surface area density based on the mass, number concentration
c     and size distribution of sulfate.
c----------------------------------------------------------------------------

c     calculate the air density in units kg/m^3

      air_density(:,:,:) = const(:,:,:,imgas_num) *
     &  1.0d6 * 28.96d0 * 1.0d-3 / 6.022d23
   
c     get SO4 number concentraion and mass concentration

      num_so41(:,:,:) = const(:,:,:,ISO4N1 ) * air_density(:,:,:)     ! #/m^3
      num_so42(:,:,:) = const(:,:,:,ISO4N2 ) * air_density(:,:,:)     ! #/m^3
      num_so43(:,:,:) = const(:,:,:,ISO4N3 ) * air_density(:,:,:)     ! #/m^3
      mas_so41(:,:,:) = const(:,:,:,ISO4M1 ) * air_density(:,:,:)     ! kg/m^3
      mas_so42(:,:,:) = const(:,:,:,ISO4M2 ) * air_density(:,:,:)     ! kg/m^3
      mas_so43(:,:,:) = const(:,:,:,ISO4M3 ) * air_density(:,:,:)     ! kg/m^3

      
c    get volume mean radius (m)     

      where (num_so41.eq.0.0) 
            
         rv_so41=0.0d0
      
      elsewhere

         rv_so41(:,:,:) = (3.d0 * mas_so41(:,:,:) / num_so41(:,:,:) /
     &     (4.d0 * pi * rhosul)) ** (1.d0/3.d0)

      end where
      
      where (num_so42.eq.0.0) 
            
        rv_so42=0.0d0
      
      elsewhere
 
        rv_so42(:,:,:) = (3.d0 * mas_so42(:,:,:) / num_so42(:,:,:) /
     &     (4.d0 * pi * rhosul)) ** (1.d0/3.d0)
     
       end where 

      where (num_so43.eq.0.0) 
            
      rv_so43=0.0d0
      
      elsewhere

      rv_so43(:,:,:) = (3.d0 * mas_so43(:,:,:) / num_so43(:,:,:) /
     &     (4.d0 * pi * rhosul)) ** (1.d0/3.d0)

      end where

c    get mode radius (m)

      rm_so41(:,:,:) = rv_so41(:,:,:) * exp(-3.d0 *
     $   log(sigmod01) ** 2.d0 / 2.d0 )  
      rm_so42(:,:,:) = rv_so42(:,:,:) * exp(-3.d0 *
     $  log(sigmod02) ** 2.d0 / 2.d0 )  
      rm_so43(:,:,:) = rv_so43(:,:,:) * exp(-3.d0 *
     $  log(sigmod03) ** 2.d0 / 2.d0 )  
c 
c    change units (m-->cm)

      rm_so41(:,:,:) = rm_so41(:,:,:) * 1.d2
      rm_so42(:,:,:) = rm_so42(:,:,:) * 1.d2
      rm_so43(:,:,:) = rm_so43(:,:,:) * 1.d2

c    get SO4 area density (cm^2/cm^3)

      lbs_sad(:,:,:) = 4.d0 * pi * rm_so41(:,:,:)
     &   ** 2.d0 * num_so41(:,:,:) * 1.d-6
     &   +4.d0 * pi * rm_so42(:,:,:) ** 2.d0 * num_so42(:,:,:) * 1.d-6           
     &   +4.d0 * pi * rm_so43(:,:,:) ** 2.d0 * num_so43(:,:,:) * 1.d-6           


c     -----------------------------------------------------------------
c     Now calculate the change in surface area density due to humidity.
c     This is a fit to data from Chuang in the form suggested by Grant.
c     -----------------------------------------------------------------
 
      sadimp(:,:,:,1) =
     &  lbs_sad(:,:,:) *
     &  (1.0d0 +
     &   1.25824d0 * (Exp (2.71811d0 * rh(:,:,:)**11.3214) - 1.0d0) +
     &   24.94300d0 * (1.0d0 - Exp (-0.0329662 * rh(:,:,:))))**2
      !sadimp(i1:i2, ju1:j2, k1:k2,1) = 1.0d-6 !debug test only, Guangxing 

c DEBUG WRITES for Ohio, Antarctica
         il= 56
         ij= 33
         ik=1
        if ( il.eq.-999.and. il.ge.i1.and.il.le.i2
     &      .and.ij.ge.ju1.and.ij.le.j2) then
         write(6,*) 'TEST SULFATE inUpdate_Sad3: '
         write (6,*) ' il, ij, ik =', il, ij, ik
         write (6,*) 'ilo, ihi, julo, jhi =', ilo, ihi, julo, jhi
         write (6,*) ' temp, rh =', temp3(il,ij,ik), rh(il,ij,ik)
c     write(6,*) ' h2o mixing ratio =',
c     &       ih2o_num,  const(il,ij,ik, ih2o_num)
         write(6,*) 'lbssad = ', lbs_sad(il,ij,ik)
         write(6,*) 'sadimp = ', sadimp(il,ij,ik,1)
         write(6,*) 'airdensity = ', air_density(il,ij,ik)
         write(6,*) 'num_so41 = ',  const(il,ij,ik,ISO4N1 )
         write(6,*) 'num_so42 = ', const(il,ij,ik,ISO4N2)
         write(6,*) 'mas_so41 = ',  const(il,ij,ik,ISO4M1 )
         write(6,*) 'mas_so42 = ', const(il,ij,ik,ISO4M2)
         write(6,*) 'rv_so41 ,rm_so41= ', rv_so41(il,ij,ik),
     &      rm_so41(il,ij,ik)
         write(6,*) 'rv_so42 ,rm_so42= ', rv_so42(il,ij,ik), 
     &       rm_so42(il,ij,ik)
         write(6,*) 'iso4n1 iso4m1, = ', ISO4N1, ISO4M1
         write(6,*) 'iso4n2  iso4m2, = ', ISO4N2,ISO4M2 
       endif

         il= 66
         ij=  2
         ik=1
        if (il.eq.-999.and.il.ge.i1.and.il.le.i2
     &       .and.ij.ge.ju1.and.ij.le.j2) then
         write(6,*) 'TEST SULFATE inUpdate_Sad3: '
         write (6,*) ' il, ij, ik =', il, ij, ik
         write (6,*) 'ilo, ihi, julo, jhi =', ilo, ihi, julo, jhi
         write (6,*) ' temp, rh =', temp3(il,ij,ik), rh(il,ij,ik)
c         write(6,*) ' h2o mixing ratio =',
c     &       ih2o_num,  const(il,ij,ik, ih2o_num)
         write(6,*) 'lbssad = ', lbs_sad(il,ij,ik)
         write(6,*) 'sadimp = ', sadimp(il,ij,ik,1)
         write(6,*) 'num_so41 = ',  const(il,ij,ik,ISO4N1 )
         write(6,*) 'num_so42 = ', const(il,ij,ik,ISO4N2) 
         write(6,*) 'iso4n1 = ', ISO4N1
         write(6,*) 'iso4n2 = ', ISO4N2 
        endif
      

      return
 
      end


