      subroutine umsoa(dt,nbox,temp,press,ispbcm, relhum,cldfrac,
     $                     orggas,so4gas,srcgas,
     $           orgaer,orgnon,aernon_org,wtmorgbox,so4non,so4org,zq)
c
c---->do condensation, nucleation, and coagulation for sulfate aerosol,
c     calculate condensation of so4 on non-sulfate aerosol
c
c     dt                time step                             [s]
c     nbox              number of parcels
c     temp(nbox)        temperature                           [K]
c     press(nbox)       air pressure                          [Pa]
c     ispbcm(nbox)      the flag for boundary layer           [0 or 1]
c     relhum(nbox)      relative humidity                     [0-1]
c     orggas(nbox)      ELVOC gas concentration                 [#molec/m3]
c     srcgas(nbox)      ELVOC source rate (gas phase)           [#so4 molec/m3/s]
c     remove srcmas(nbox)      so4 source rate (aqueous phase)       [#so4 molec/m3/s]
c     remove srcpar            h2so4 source rate (particulate)       [#so4 molec/m3/s]
c
c     orgaer(nmomso4,nso4,nbox) pure SOA (with dimension same as SO4)     [#/m3]
c     orgnon(nnon,nbox) SOA attached to other aerosol   [#so4 molec/m3] SOA on 16 non-SOA aerosol+ 3 pure SOA
c     aernon_org(nmomnon,nnon,nbox) 16 non-SOA aersol + 3 sulfate number conc     [kg/m3]
c
c---->number of modes and moments set in umaerinit and stored in modpar_umaer
c
c     norg              number of pure SOA modes  =3
c     nmomorg           number of pure SOA moments =2 
c     nnon              number of non-sulfate aerosol modes =25
c     nmomnon           number of non-sulfate aerosol moments =1 
c
c     moments:   1 - mass
c                2 - number 
c                3 - surface
c
c
c---->load modules
c
      use precision
      use modpar_umaer
      use modbox_umaer
#ifdef DIAG
      use modiag_umaer
#endif
       use cam_logfile,   only: iulog  !Guangxing Lin
c
c---->define accuracy and input/output arrays
c
      implicit real(kdef) (a-h,o-z), integer (i-n)
      dimension temp(nbox),press(nbox),ispbcm(nbox), relhum(nbox),
     $          cldfrac(nbox),orggas(nbox),so4gas(nbox),srcgas(nbox), !,srcmas(nbox),
     $          orgaer(nmomorg,norg,nbox) !JIALEI update to nucleation version
      dimension orgnon(nnon_org,nbox), !JIALEI change from so4non
     $          aernon_org(nmomnon,nnon_org,nbox),wtmorgbox(nbox)
      dimension so4non(nnon,nbox),so4org(norg,nbox),zq(nbox) !JIALEI 11/2017
c
c---->local arrays and parameter
c     xaer              temporary array for sulfate aerosol
c                       (all modes and moments)
c
      dimension xaer(nvec,ntot_org+norg)!nnon_org+nmomorg*norg+2)
      dimension so4non_org(nvec,nnon_org) !JIALEI

      dimension iterbox(nvec),locbox(nvec)
      dimension dtbox(nvec),time(nvec) 
      dimension tempbox(nvec),pressbox(nvec),ispbcmbox(nvec), 
     $          relhumbox(nvec),srcgasbox(nvec)!,srcmasbox(nvec),
!     $          srcparbox(nvec) !JIALEI remove aqueous and particle source
      dimension so4radv(nvec,nso4),wtmorg(nvec),zqbox(nvec) !JIALEI
#ifdef DIAG
      dimension iterstat(15)
      parameter (r1h=0.5d0)
#endif
      parameter (r4=4.d0,r6=6.d0,r10=10.d0,r3q=0.75d0,r1td=1.d0/3.d0)
      parameter (a1=1.333d0,a2=0.71d0,b1=0.572127d0)
      parameter (c1=1.249d0,c2=0.42d0,c3=-0.87d0)
      parameter (rhmin=1.d-2,rhmax=0.99d0)

!      write(iulog,*) "JIALEI-0",maxval(orgaer(1,2,:))

#ifdef DIAG
      iterstat(:)=0
#endif
      maxstat=0
c
c---->loop over all parcels, do calculation for chunk of nvec parcels
c
      nv=nvec
      ibox=0
      idone=0
      time(1:nvec)=dt
      do while (idone==0)
c
c---->fill in new data as long as not all boxes are considered
c

       if (ibox<nbox) then
c
c---->fill local arrays
c
        do ivec=1,nvec
         if (time(ivec)>=dt) then
c
c---->update parcel counter
c
          ibox=ibox+1
          time(ivec)=r0
          iterbox(ivec)=0
c
c---->remember parcel index and preset time step
c
          locbox(ivec)=ibox
          dtbox(ivec)=-r1
c
c---->store sulfate stuff
c
          wtmorg(ivec)=wtmorgbox(ibox) !JIALEI          
          xaer(ivec,1:nnon_org)=orgnon(1:nnon_org,ibox)
          do iorg=1,norg
           do imom=1,nmomorg
            iaer=ntot_org-(iorg-1)*nmomorg-nmomorg+imom-1-1
            xaer(ivec,iaer)=orgaer(imom,iorg,ibox)
           enddo
          enddo
          xaer(ivec,ntot_org-1)=orggas(ibox)
          xaer(ivec,ntot_org)=so4gas(ibox)

c----------JIALEI so4 on relative aerosols
          so4non_org(ivec,1:4)=0
          so4non_org(ivec,5:8)=so4non(2:5,ibox)
          so4non_org(ivec,9:16)=so4non(14:21,ibox)
          xaer(ivec,ntot_org+1:ntot_org+3)=so4org(1:3,ibox)
c------------------------------------------------
#ifdef DIAG
          call diagfill(ivec,ibox)
#endif

c
c---->store non-so4 aerosol number concentrations       [#part/m3]
c

!           xnonum_org(ivec,1)=2.2
          xnonum_org(ivec,1:3)=aernon_org(1,nnon_org-2:nnon_org,ibox) !Sulfate number conc
          xnonum_org(ivec,4:8)=aernon_org(1,4:8,ibox)/pmsnon(1:5) !nOC fOC/BC bOC/BC
          xnonum_org(ivec,9:16)=aernon_org(1,9:16,ibox)/pmsnon(14:21) !Dust SSlT
          xnonum_org(ivec,17:nnon_org)=aernon_org(1,nnon_org-2:nnon_org,ibox) !Sulfate number conc again

c
c---->itore meteorological data and sources
c
          tempbox(ivec)=temp(ibox)
          pressbox(ivec)=press(ibox)
          ispbcmbox(ivec) = ispbcm(ibox)
          relhumbox(ivec)=max(rhmin,relhum(ibox))
          srcgasbox(ivec)=srcgas(ibox) !JIALEI
          zqbox(ivec)=zq(ibox) !JIALEI
!          srcmasbox(ivec)=srcmas(ibox)
!          srcparbox(ivec)=srcpar(ibox)
c 
c---->the following quantities don't change during time integration
c     and are stored in common block /parcel/
c---->vapor pressures:
c     h2osat        h2o saturation pressure             [#molec/m3]
c     h2ogas        h2o gas concentration               [#molec/m3]
c     so4sat        h2so4 saturation pressure           [#molec/m3]
c     orgsat        ELVOC saturation pressure represented by pinic acid  [#molec/m3]
          h2osat(ivec)=h2osat_f(temp(ibox))
          h2ogas(ivec)=relhumbox(ivec)*h2osat(ivec)
!          so4sat(ivec)=so4sat_f(temp(ibox))!JIALEI
          orgsat(ivec)=orgsat_f(temp(ibox))  !JIALEI
c
c---->aerosol composition and Kelvin effect:
c     so4mfrac      so4 mass fraction in aerosol        [1]
c     so4frac       so4 mole fraction in aerosol        [1]
c     so4dens       aerosol density                     [kg/m3]
c     so4kelvin     factor for Kelvin effect            [m]
c

          so4mfrac(ivec)=so4mfrac_f(temp(ibox),h2osat(ivec),
     $                                         h2ogas(ivec))
          so4frac(ivec)=so4mfrac(ivec)*h2omol
     $             /(so4mfrac(ivec)*h2omol+(r1-so4mfrac(ivec))*so4mol)
          so4dens(ivec)=so4dens_f(temp(ibox),so4mfrac(ivec))

          so4kelvin(ivec)=
     $             r2*so4sig_f(temp(ibox),so4frac(ivec),so4mfrac(ivec))
     $                *(so4frac(ivec)*so4mol+(r1-so4frac(ivec))*h2omol)
     $                /(rgas*temp(ibox)*so4dens(ivec))

c---------------JIALEI calculate wet radius of sulfate
      do iorg=1,nso4
       wetmas=max(r2,aernon_org(1,iorg,ibox)
     $       /max(1.0E-30,aernon_org(1,iorg+16,ibox)))
     $       *so4min/so4mfrac(ivec)
       so4radv(ivec,iorg)=(r3q*wetmas/(pi*so4dens(ivec)))**r1td
      enddo
c-------------------------------------------

c
c---->rwetnon       humid volume mean radius            [m]
C
c     if nrh_flg=1 assume it is sea salt,
c     humidity growth for sea salt according to Gerber (1985)
c
          rhnew=max(rhmin,min(rhmax,relhum(ibox)))
          factor=r3q*so4min/(pi*so4mfrac(ivec)*so4dens(ivec)) 
!          do inon=1,nnon
!           if (nrh_flg(inon)==1) then
c           rwetnon(ivec,inon)=(radvnon(inon)**3
c    $                   +(crh1*radvnon(inon)**crh2)
c    $                  /((crh3*radvnon(inon)**crh4)
c    $                   -log10(rhnew))
c    $                   +factor*xaer(ivec,inon)
c    $                 /max(r1,xnonum(ivec,inon)))**r1td


            rwetnon_org(ivec,13:16)=min((radvnon(18:21)**3  !Seasalt
     $                   +(crh1(18:21)*radvnon(18:21)**crh2(18:21))
     $                  /((crh3(18:21)*radvnon(18:21)**crh4(18:21))
     $                   -log10(rhnew))
     $                   +factor*so4non_org(ivec,13:16)
     $  /max(r1,xnonum_org(ivec,13:16)))**r1td,r10*radvnon(18:21))



            rwetnon_org(ivec,9:12)=min((radvnon(14:17)**3 !Dust
     $                +factor*so4non_org(ivec,9:12)
     $ /max(r1,xnonum_org(ivec,9:12)))**r1td,r10*radvnon(14:17))



            rwetnon_org(ivec,4:8)=min((radvnon(1:5)**3 !BC/OC
     $                +factor*so4non_org(ivec,4:8)
     $ /max(r1,xnonum_org(ivec,4:8)))**r1td,r10*radvnon(1:5))

!           endif
!          enddo

c$$$          rwetnon(ivec,1:nnon-2)=(radvnon(1:nnon-2)**3
c$$$     $                  +factor*xaer(ivec,1:nnon-2)
c$$$     $                /max(r1,xnonum(ivec,1:nnon-2)))**r1td
c$$$          rwetnon(ivec,nnon-1:nnon)=(radvnon(nnon-1:nnon)**3
c$$$     $                   +(crh1*radvnon(nnon-1:nnon)**crh2)
c$$$     $                  /((crh3*radvnon(nnon-1:nnon)**crh4)
c$$$     $                   -log(rhnew))
c$$$     $                   +factor*xaer(ivec,nnon-1:nnon)
c$$$     $                 /max(r1,xnonum(ivec,nnon-1:nnon)))**r1td
          rwetnon_org(ivec,1:3)=so4radv(ivec,1:3)
          rwetnon_org(ivec,17:nnon_org)=so4radv(ivec,1:3)

!          rwetnon_org(ivec,1:nnon_org)=min(rwetnon_org(ivec,1:nnon_org),
!     $                              r10*radvnon(1:nnon))
c
c---->Kelvin effect for non-sulfate aerosol
c     assuming that non-sulfate aerosol is completely coated with so4
c----------------JIALEI for nucleation version 10/2017---Kelvin effect for non-SOA aerosol


          xnonkelvin_org(ivec,4:16)=exp(so4kelvin(ivec)
     $                                 /rwetnon_org(ivec,4:16))

          xnonkelvin_org(ivec,1:3)=exp(so4kelvin(ivec)/rwetnon_org(ivec,1:3)) !JIALEI check
          xnonkelvin_org(ivec,17:nnon_org)=exp(so4kelvin(ivec)/rwetnon_org(ivec,1:3))
c------------------------------------------------------------


c
c---->thermodynamic quantities:
c     pathair       mean free path of air               [m]
c     pathso4       mean free path of h2so4 in air      [m]
c     diffso4       diffusion coeff. for h2so4 in air   [m2/s]
c     viscos        dynamic viscosity of air            [Ns/m2]
c
          pathair(ivec)=pathair_f(temp(ibox),press(ibox))
          pathso4(ivec)=b1*pathair(ivec)
          diffso4(ivec)=r4*pi*diffso4_org_f(temp(ibox),press(ibox))
          viscos (ivec)=viscos_f(temp(ibox))
#ifdef KULMALA
c
c---->quantities for nucleation:
c     so4crit       h2so4 conc with nuc' rate of 1/cm3  [#molec/m3]
c
!          so4crit(ivec)=so4crit_f(temp(ibox),relhum(ibox))
#else
!          so4crit(ivec)=r0
#endif
c
c---->quantities for condensation:
c     so4sol        h2so4 sat pressure over solution    [#molec/m3]
c     condnon_org       condensation coef's for non-SOA aer [m3/#molec]
c                   incl. correction for collision geometry
c                   following Fuchs and Sutugin (1971)
c----------------------update by JIALEI 10/2017

!          so4sol(ivec)=so4sol_f(temp(ibox),so4sat(ivec),so4frac(ivec))
          orgfrac=1.0  !JIALEI check
          orgsol(ivec)=so4sol_f(temp(ibox),orgsat(ivec),orgfrac)  !JIALEI assume pure SOA !JIALEI check
!          so4crit(ivec)=max(r2*orgsol(ivec),so4crit(ivec)) !JIALEI


          do inon=1,3
           xknu=pathso4(ivec)/rwetnon_org(ivec,inon)
           condnon_org(ivec,inon)=diffso4(ivec)*so4alpha(ivec,inon)
     $                       /(r1+((a1*xknu+a2)/(r1+xknu)
     $                       +a1*(r1-etaso4(inon))/etaso4(inon))*xknu)
     $               *rwetnon_org(ivec,inon)
          enddo

          do inon=4,8
           xknu=pathso4(ivec)/rwetnon_org(ivec,inon)
           condnon_org(ivec,inon)=diffso4(ivec)*alphanon(inon-3)
     $                       /(r1+((a1*xknu+a2)/(r1+xknu)
     $               +a1*(r1-etanon(inon-3))/etanon(inon-3))*xknu)
     $               *rwetnon_org(ivec,inon)
          enddo

          do inon=9,16
           xknu=pathso4(ivec)/rwetnon_org(ivec,inon)
           condnon_org(ivec,inon)=diffso4(ivec)*alphanon(inon+5)
     $                       /(r1+((a1*xknu+a2)/(r1+xknu)
     $               +a1*(r1-etanon(inon+5))/etanon(inon+5))*xknu)
     $               *rwetnon_org(ivec,inon)
          enddo

          do inon=17,nnon_org
           condnon_org(ivec,inon)=condnon_org(ivec,inon-16)
          enddo

c----------------------------------------------------------------
c
c---->quantities for coagulation update by JIALEI 10/2017
c
          do inon=1,nnon_org
           xknu=pathair(ivec)/rwetnon_org(ivec,inon)
           diffnon_org(ivec,inon)=r4*boltz*temp(ivec)
     $                  /(r6*pi*viscos(ivec)*rwetnon_org(ivec,inon))
     $                  *(r1+xknu*(c1+c2*exp(c3/xknu)))
          enddo

c
c---->quantities for merging:
c     pmsmerg_org       limit particle mass for merging     [#molec]
c
          do iorg=1,norg
           pmsmerg_org(ivec,iorg)=rhonon(4)*1*pmsfac(iorg)   !assume the same density with bOC and mass fraction of SOA is 1 JIALEI
          enddo
          pmsmerg_org(ivec,norg+1)=r2
         endif

c
c---->exit if all boxes considered
c
         if (ibox>=nbox) then
          exit
         endif
        enddo
       endif
c
c----->shrink local array is all boxes are considered
c
       if (ibox>=nbox) then
        nvnew=nv
        do iv=1,nv
         if (iv>nvnew) exit
         if (time(iv)>=dt) then
          do jv=nvnew,iv+1,-1
           if (time(jv)<dt) then
c
c---->shift parcel data
c
            iterbox(iv)=iterbox(jv)
            locbox(iv)=locbox(jv)
            dtbox(iv)=dtbox(jv)
            time(iv)=time(jv)
            time(jv)=dt

            xaer(iv,1:ntot_org+3)=xaer(jv,1:ntot_org+3) !JIALEI
!            so4non_org(iv,1:nnon_org)=so4non_org(jv,1:nnon_org) !JIALEI
            xnonum_org(iv,1:nnon_org)=xnonum_org(jv,1:nnon_org) !JIALEI
            wtmorg(iv)=wtmorg(jv) !JIALEI

            tempbox(iv)=tempbox(jv) 
            pressbox(iv)=pressbox(jv)
            ispbcmbox(iv)=ispbcmbox(jv)
            relhumbox(iv)=relhumbox(jv)
            srcgasbox(iv)=srcgasbox(jv)
            zqbox(iv)=zqbox(jv)  !JIALEI
!            srcmasbox(iv)=srcmasbox(jv)
!            srcparbox(iv)=srcparbox(jv)

            h2osat(iv)=h2osat(jv)
            h2ogas(iv)=h2ogas(jv)
!            so4sat(iv)=so4sat(jv)
            orgsat(iv)=orgsat(iv) !JIALEI
             
            so4mfrac(iv)=so4mfrac(jv)
            so4frac(iv)=so4frac(jv)
            so4dens(iv)=so4dens(jv)

!            so4kelvin(iv)=so4kelvin(jv)  !JIALEI

            rwetnon_org(iv,1:nnon_org)=rwetnon_org(jv,1:nnon_org) !JIALEI
            xnonkelvin_org(iv,1:nnon_org)=xnonkelvin_org(jv,1:nnon_org) !JIALEI
            diffnon_org(iv,1:nnon_org)=diffnon_org(jv,1:nnon_org)  !JIALEI

            pathair(iv)=pathair(jv)
            pathso4(iv)=pathso4(jv)
            diffso4(iv)=diffso4(jv)
            viscos (iv)=viscos (jv)

!            so4crit(iv)=so4crit(jv)!JIALEI
!            so4sol(iv)=so4sol(jv) !JIALEI
            orgsol(iv)=orgsol(jv) !JIALEI
            condnon_org(iv,1:nnon_org)=condnon_org(jv,1:nnon_org)  !JIALEI

            pmsmerg_org(iv,1:norg+1)=pmsmerg_org(jv,1:norg+1)
#ifdef DIAG
            call diagshift(iv,jv)
#endif
            exit
           endif
          enddo
          nvnew=jv-1
         endif
        enddo
        nv=nvnew
       endif
c---->do one iteration of for time integration using Runge-Kutta
c
       call aertime_org(dt,nv,iterbox,dtbox,time,
     $              tempbox,pressbox, ispbcmbox, relhumbox,
     $              srcgasbox,!srcmasbox,srcparbox, !JIALEI remove 
     $              xaer,wtmorg,zqbox)!!,so4non_org)
c
c---->if iter>itermax set parcel's time to dt
c
       if (maxval(iterbox(1:nv))>itermax/2) then
        print*,'warning! iter>itermax!'
        write(iulog,*)'warning! iter>itermax!'
!        write(iulog,*)'gxlin-srcgasbox', srcgasbox
!        write(iulog,*)'gxlin-srcmasbox', srcmasbox JIALEI
!        write(iulog,*)'gxlin-srcparbox', srcparbox JIALEI
!        write(iulog,*)'gxlin-xaer', xaer 
        do iv=1,nv
         if (iterbox(iv)>itermax) exit
        enddo
        write(*,200) locbox(iv),dt,time(iv),dtbox(iv)
        time(iv)=dt
       endif
       idone=1
c
c---->check overall success
c
       if (ibox>=nbox) then
        if (minval(time(1:nv))>=dt) then
           idone=1
!           if (maxval(iterbox(1:nv)).gt.300)then
           write(iulog,*) "JIALEI-iter-soa=",maxval(iterbox(1:nv))
!           end if
        end if
       endif
c
c---->check success for nv boxes
c
       do iv=1,nv
        if (time(iv)>=dt) then
c
c---->restore new concentrations update by JIALEI
c
         ib=locbox(iv)
         orgnon(1:nnon_org,ib)=xaer(iv,1:nnon_org)
         do iorg=1,norg
          do imom=1,nmomorg
           iaer=ntot_org-(iorg-1)*nmomorg-nmomorg+imom-1-1
           orgaer(imom,iorg,ib)=xaer(iv,iaer)
          enddo
         enddo
         orggas(ib)=xaer(iv,ntot_org-1)
         so4gas(ib)=xaer(iv,ntot_org)
         so4org(1:3,ib)=xaer(iv,ntot_org+1:ntot_org+3) !JIALEI
        end if
       enddo
!       write(iulog,*) "JIALEI-5",maxval(xaer(:,ntot-2))

c
c---->end over parcel loop
c
      enddo

#ifdef DIAG
c
c---->write statistics for iterations
c
      write(*,100) dt
      write(*,110) maxstat
      do i=1,15
       istat=r1h*(i+1)**3-mod(i,2)
       write(*,120) iterstat(i),istat
      enddo
#endif
!      write(iulog,*) "JIALEI-6",maxval(orgaer(1,2,:))
      return
 100  format(/,'timestep: ',f10.0)
 110  format('maximum number of iteration: ',i7)
 120  format(i8,' less equal ',i5,' iterations')
 200  format(' parcel: ',i6,/,
     $       ' requested total time: ',f10.0,'s',/,
     $       ' reached total time:   ',f10.0,'s',/,
     $       ' last time step:       ',1pe10.2,'s',/)
      
      continue
      end
c-----------------------------------------------------------------------
      subroutine aertime_org(dt,nv,iterbox,dtbox,time,
     $                   temp,press,ispbcm, relhum,srcgas,!srcmas,srcpar,
     $                   xaer,wtmorg,zq)!,so4non_org)
c 
c---->time integration for nv parcels
c     Runge Kutta 4th order with adaptive time stepping:
c     W.H. Press et al., Numerical Recipes, Cambridge University Press (1988),
c     chapter 15.
c
c     xaer     array of nv parcels with initial values:
c              xaer(1:nv,1:nnon)            so4 on non-sulfate aerosol
c              xaer(1:nv,nnon+1:nnon+naer)  so4 aerosol (nso4*nmomso4)
c              xaer(1:nv,nnon+naer+1)       so4 gas
c----->update for nucleation version 10/2017 JIALEI
c---->load modules
c
      use precision
      use modpar_umaer
      use modbox_umaer
      use cam_logfile,   only: iulog  !Guangxing Lin

c
c---->define accuracy and input/output arrays
c
      implicit real(kdef) (a-h,o-z), integer (i-n)
      dimension iterbox(nv)
      dimension dtbox(nv),time(nv),
     $          temp(nv),press(nv),ispbcm(nv), relhum(nv),
     $          srcgas(nv),zq(nv) !JIALEI !,srcmas(nv),srcpar(nv)
      dimension xaer(nvec,ntot_org+norg),wtmorg(nvec)!!,so4non_org(nvec,nnon_org) !JIALEI add wtmorg, so4non_org
      dimension so4sav(nvec),orgsav(nvec,6) !JIALEI store so4gas to calculated so4 include in new SOA particles when nucleate
c
c---->define local arrays and parameter
c
      dimension xtmp(nvec,ntot_org),fxaer(nv,ntot_org),
     $          xsav(nvec,ntot_org),fxsav(nv,ntot_org),
     $          dthalf(nv),dt6inv(nv),errmax(nv),
     $          orgtot(nv),orgcor(nv),so4org(nvec,norg) !JIALEI

      parameter (r4=4.d0,r1000=1000.d0,r1h=0.5d0,r1q=0.25d0)
      parameter (r6inv=r1/6.d0)
      parameter (predu=-0.25d0,pgrow=-0.2d0,fcor=r1/15.d0,
     $           safety=0.90d0,errcon=(r4/safety)**(-5)) !JIALEI change safety from 0.9 to 0.45
      parameter (relacc=5.0d-2) !JIALEI change from 5.0d-2 to 2.5d-2
      parameter (so4max=1.d12)
c
c---->declare interfaces
c
      interface


      subroutine derivative_org(nv,temp,press,ispbcm, relhum,srcgas, !JIALEI
     $                      xaer,fxaer,dt,iflg,wtmorg,so4org,zq)
      use precision
      use modpar_umaer
      use modbox_umaer
      implicit real(kdef) (a-h,o-z), integer (i-n)
      dimension temp(nv),press(nv),ispbcm(nv), relhum(nv),
     $          srcgas(nv),wtmorg(nvec),so4org(nvec,norg) !,srcmas(nv),srcpar(nv)
      dimension xaer(nvec,ntot_org),fxaer(nv,ntot_org),dt(nv),zq(nv)
      target xaer
      end subroutine derivative_org

      subroutine rk4step_org(nv,temp,press,ispbcm,relhum,srcgas,
     $                   xaer,fxaer,dtbox,xout,wtmorg,so4org,zq)
      use precision
      use modpar_umaer
      use modbox_umaer
      implicit real(kdef) (a-h,o-z), integer (i-n)
      dimension temp(nv),press(nv),ispbcm(nv), relhum(nv),
     $          srcgas(nv),wtmorg(nvec),so4org(nvec,norg)!,srcmas(nv),srcpar(nv)
      dimension xaer(nvec,ntot_org),fxaer(nv,ntot_org),dtbox(nv),
     $          xout(nvec,ntot_org),zq(nv)
      target xaer,xtmp
      end subroutine rk4step_org

      subroutine merge_org(ivec,xaer,wtmorg)!!,so4non_org)
      use precision
      use modpar_umaer
      use modbox_umaer
      implicit real(kdef) (a-h,o-z), integer (i-n)
      dimension xaer(nvec,ntot_org+norg),wtmorg(nvec)!!,so4non_org(nvec,nnon_org)
      target xaer
      end subroutine merge_org
      end interface


c
c---->calculate initial time derivates (fxaer)
c
      so4sav=xaer(:,ntot_org) !JIALEI save original so4gas
      so4org=xaer(:,ntot_org+1:ntot_org+norg) !JIALEI

#ifdef DIAG
      call diagpoint(nv,1,1)
#endif
      dt6inv(1:nv)=max(r0,r6inv*dtbox(1:nv))
      call derivative_org(nv,temp,press,ispbcm,relhum,srcgas,!srcmas,srcpar,
     $                xaer(:,1:ntot_org),fxaer,dt6inv,1,wtmorg,so4org,zq)
c
c---->estimate initial time step if time=0
c

      do iv=1,nv
       if (time(iv)==r0 .and. iterbox(iv)==0) then
        dtbox(iv)=min(dtmax,dt)
        do itot=1,ntot_org-2
         if (xaer(iv,itot)*fxaer(iv,itot)<r0) then
         dtbox(iv)=min(dtbox(iv),max(dtmin,
     $             -safety*xaer(iv,itot)/fxaer(iv,itot)))
         end if
        enddo
        if (xaer(iv,ntot_org-1)*fxaer(iv,ntot_org-1)<r0) then
         dtbox(iv)=min(dtbox(iv),max(dtmin,
     $             -safety*min(so4max,xaer(iv,ntot_org-1))/fxaer(iv,ntot_org-1)))
        end if

       endif
      enddo


c
c---->save old values and derivatives
c
      xsav(1:nv,1:ntot_org)=xaer(1:nv,1:ntot_org)
      fxsav(1:nv,1:ntot_org)=fxaer(1:nv,1:ntot_org)
c
c---->take first half Runge-Kutta step
c
      dthalf(1:nv)=r1h*dtbox(1:nv)

      call rk4step_org(nv,temp,press,ispbcm,relhum,srcgas,!srcmas,srcpar,
     $             xaer(:,1:ntot_org),fxaer,dthalf,xtmp,wtmorg,so4org,zq)

c
c---->ensure positive definiteness and mass conservation
c

      do iv=1,nv
       orgtot(iv)=xtmp(iv,ntot_org-1)+sum(xtmp(iv,1:nnon_org))
       orgcor(iv)=max(r0,xtmp(iv,ntot_org-1))+sum(max(r0,xtmp(iv,1:nnon_org)))
      enddo
      do iorg=1,norg
       inum=nnon_org+iorg*nmomorg
       imas=inum-1
       do iv=1,nv
        orgtot(iv)=orgtot(iv)+xtmp(iv,imas)
        orgcor(iv)=orgcor(iv)+max(r0,xtmp(iv,imas))
        if (xtmp(iv,inum)<=r0) then
           xtmp(iv,inum)=xtmp(iv,imas)
     $                                      /pmsmerg_org(iv,iorg+1)
        end if
        if (xtmp(iv,imas)<=r0) then
          xtmp(iv,inum)=r0
        end if
       enddo
      enddo
      do iv=1,nv
       xtmp(iv,1:ntot_org)=max(r0,xtmp(iv,1:ntot_org))
      enddo
      do iv=1,nv
       if (orgcor(iv)>orgtot(iv)) then
        factor=orgtot(iv)/orgcor(iv)
        xtmp(iv,1:ntot_org)=factor*xtmp(iv,1:ntot_org)
       endif
      enddo
c
c---->take second half Runge-Kutta step
c

#ifdef DIAG
      call diagstore(nv,1)
      call diagpoint(nv,0,1)
#endif

      call derivative_org(nv,temp,press,ispbcm,relhum,srcgas,!srcmas,srcpar,
     $                xtmp,fxaer,dt6inv,0,wtmorg,so4org,zq)


      call rk4step_org(nv,temp,press,ispbcm,relhum,srcgas,!srcmas,srcpar,
     $             xtmp,fxaer,dthalf,xaer(:,1:ntot_org),wtmorg,so4org,zq)

c
c---->take one large Runge-Kutta step    
c
#ifdef DIAG
      call diagrestore(nv,1)
      call diagpoint(nv,2,1)
#endif
      call rk4step_org(nv,temp,press,ispbcm,relhum,srcgas,!srcmas,srcpar,
     $             xsav,fxsav,dtbox,xtmp,wtmorg,so4org,zq)

c
c---->calculate error, new time step and concentrations,
c     advance time counter
c
      errmax(1:nv)=r0
      do itot=1,ntot_org-2
       if(itot.lt.17.or.itot.gt.19.and.itot.ne.4)then  !JIALEI remove SOAOm1-3 and SOAnOC
       do iv=1,nv
        dxaer=xaer(iv,itot)-xtmp(iv,itot)
        xscale=abs(xsav(iv,itot))+dtbox(iv)*abs(fxsav(iv,itot))
        xscale=max(xscalemin_org(itot),xscale)
        errmax(iv)=max(errmax(iv),abs(dxaer/xscale))
       enddo
       endif
      enddo
      do iv=1,nv
       dxaer=xaer(iv,ntot_org-1)-xtmp(iv,ntot_org-1)
       xscale=abs(xsav(iv,ntot_org-1))
     $       +max(dtbox(iv)*abs(fxsav(iv,ntot_org-1)),r1000*srcgas(iv))
       xscale=max(xscalemin_org(ntot_org-1),xscale)
       errmax(iv)=max(errmax(iv),abs(dxaer/xscale))
      enddo

      do iv=1,nv
       iterbox(iv)=iterbox(iv)+1
       errmax(iv)=errmax(iv)/relacc
       if (errmax(iv)>r1) then
        dtbox(iv)=max(r1q*dtbox(iv),
     $                safety*dtbox(iv)*(errmax(iv)**predu))
        xaer(iv,1:ntot_org)=xsav(iv,1:ntot_org)
        xtmp(iv,1:ntot_org)=r0
       else
        dtold=dtbox(iv)
        time(iv)=time(iv)+dtbox(iv)
        dtend=max(r0,dt-time(iv)+epsilon)
        if (errmax(iv)>errcon) then
         dtbox(iv)=min(dtend,safety*dtbox(iv)*(errmax(iv)**pgrow))
        else
         dtbox(iv)=min(dtend,r4*dtbox(iv))
        endif
c
c---->use xtmp to achieve 5th order accuracy
c     and ensure positive definiteness and mass conservation
c
        orgtot(iv)=xaer(iv,ntot_org-1)+sum(xaer(iv,1:nnon_org))
        do itot=1,norg
         inum=nnon_org+itot*nmomorg
         imas=inum-1
         orgtot(iv)=orgtot(iv)+xaer(iv,imas)
        enddo
        xtmp(iv,1:ntot_org)=xaer(iv,1:ntot_org)
     $          +fcor*(xaer(iv,1:ntot_org)-xtmp(iv,1:ntot_org))

        if (minval((xaer(iv,1:ntot_org)-xsav(iv,1:ntot_org))
     $            *(xtmp(iv,1:ntot_org)-xsav(iv,1:ntot_org)))>=r0) then
         xaer(iv,1:ntot_org)=xtmp(iv,1:ntot_org)
#ifdef DIAG
         call diagupdate(time(iv),dtold,iv,1)
        else
         call diagupdate(time(iv),dtold,iv,0)
#endif
        endif


        orgcor(iv)=max(r0,xaer(iv,ntot_org-1))+sum(max(r0,xaer(iv,1:nnon_org)))
        do itot=1,norg
         inum=nnon_org+itot*nmomorg
         imas=inum-1
         orgcor(iv)=orgcor(iv)+max(r0,xaer(iv,imas))
         if (xaer(iv,inum)<=r0) xaer(iv,inum)=xaer(iv,imas)
     $                                       /pmsmerg_org(iv,itot+1)
         if (xaer(iv,imas)<=r0) xaer(iv,inum)=r0
        enddo
        xaer(iv,1:ntot_org)=max(r0,xaer(iv,1:ntot_org))
        if (orgcor(iv)>orgtot(iv)) then
         factor=orgtot(iv)/orgcor(iv)
          xaer(iv,1:ntot_org)=factor*xaer(iv,1:ntot_org)
        endif

!!       xaer(iv,ntot_org+1)=xaer(iv,ntot_org+1)+(so4sav(iv)-xaer(iv,ntot_org)) !JIALEI calculate SO4 in new SOA particle
c
c---->shift particles to larger bins if overlap
c
        call merge_org(iv,xaer,wtmorg)!!,so4non_org)


       endif
      enddo
      return
      end
c-----------------------------------------------------------------------
      subroutine rk4step_org(nv,temp,press,ispbcm,relhum,srcgas,
     $                   xaer,fxaer,dtbox,xout,wtmorg,so4org,zq)
c
c---->do explicit time step using 4th order Runge-Kutta
c
c
c---->load modules
c
      use precision
      use modpar_umaer
      use modbox_umaer
      use cam_logfile,   only: iulog  !Guangxing Lin

c
c---->define accuracy and input/output arrays
c
      implicit real(kdef) (a-h,o-z), integer (i-n)
      dimension temp(nv),press(nv),ispbcm(nv),relhum(nv),
     $          srcgas(nv),wtmorg(nvec)!,srcmas(nv),srcpar(nv)
      dimension xaer(nvec,ntot_org),fxaer(nv,ntot_org),dtbox(nv),
     $          xout(nvec,ntot_org),so4org(nvec,norg),zq(nv) !JIALEI
      target xaer,xout
c
c---->define local arrays and parameter
c
      dimension dthalf(nv),dt6inv(nv),
     $          fxtmp(nv,ntot_org),fxmid(nv,ntot_org)

      parameter (r1h=0.5d0,r6inv=r1/6.d0)
c
c---->declare interfaces
c
      interface
      subroutine derivative_org(nv,temp,press,ispbcm,relhum,srcgas,!JIALEI
     $                      xaer,fxaer,dt,iflg,wtmorg,so4org,zq)
      use precision
      use modpar_umaer
      use modbox_umaer
      implicit real(kdef) (a-h,o-z), integer (i-n)
      dimension temp(nv),press(nv),ispbcm(nv),relhum(nv),
     $          srcgas(nv),wtmorg(nvec),so4org(nvec,norg)!,srcmas(nv),srcpar(nv)
      dimension xaer(nvec,ntot_org),fxaer(nv,ntot_org),dt(nv),zq(nv)
      target xaer
      end subroutine derivative_org
      end interface
c
c---->start Runge-Kutta
c
       dthalf(1:nv)=r1h*dtbox(1:nv)
      dt6inv(1:nv)=r6inv*dtbox(1:nv)
c
c---->first step
c
      do itot=1,ntot_org
       do iv=1,nv
        xout(iv,itot)=max(r0,xaer(iv,itot)+dthalf(iv)*fxaer(iv,itot))

       enddo

      enddo
 


      do iorg=1,norg
       inum=nnon_org+iorg*nmomorg
       imas=inum-1
       do iv=1,nv

        xout(iv,inum)=min(xout(iv,inum),xout(iv,imas)
     $                              /pmsmerg_org(iv,iorg+1))
        if (xout(iv,inum)<=r0) xout(iv,inum)=xout(iv,imas)
     $                              /pmsmerg_org(iv,iorg+1)
       enddo
      enddo

c
c---->second step
c
#ifdef DIAG
      call diagpoint(nv,0,3)
#endif
      call derivative_org(nv,temp,press,ispbcm,relhum,srcgas,!srcmas,srcpar,
     $                xout,fxtmp,dt6inv,0,wtmorg,so4org,zq)

      do itot=1,ntot_org
       do iv=1,nv
        xout(iv,itot)=max(r0,xaer(iv,itot)+dthalf(iv)*fxtmp(iv,itot))
       enddo
      enddo
      do iorg=1,norg
       inum=nnon_org+iorg*nmomorg
       imas=inum-1
       do iv=1,nv
        xout(iv,inum)=min(xout(iv,inum),xout(iv,imas)
     $                              /pmsmerg_org(iv,iorg+1))
        if (xout(iv,inum)<=r0) xout(iv,inum)=xout(iv,imas)
     $                              /pmsmerg_org(iv,iorg+1)
       enddo
      enddo

c
c---->third step
c
#ifdef DIAG
      call diagpoint(nv,0,2)
#endif
      call derivative_org(nv,temp,press,ispbcm,relhum,srcgas,!srcmas,srcpar,
     $                xout,fxmid,dt6inv,0,wtmorg,so4org,zq)

      do itot=1,ntot_org
       do iv=1,nv
        xout(iv,itot)=max(r0,xaer(iv,itot)+dtbox(iv)*fxmid(iv,itot))
        fxmid(iv,itot)=fxmid(iv,itot)+fxtmp(iv,itot)
       enddo
      enddo
      do iorg=1,norg
       inum=nnon_org+iorg*nmomorg
       imas=inum-1
       do iv=1,nv
        xout(iv,inum)=min(xout(iv,inum),xout(iv,imas)
     $                              /pmsmerg_org(iv,iorg+1))
        if (xout(iv,inum)<=r0) xout(iv,inum)=xout(iv,imas)
     $                              /pmsmerg_org(iv,iorg+1)
       enddo
      enddo

c
c---->fourth step
c
#ifdef DIAG
      call diagadd(nv,2,3)
      call diagpoint(nv,0,3)
#endif
      call derivative_org(nv,temp,press,ispbcm,relhum,srcgas,!srcmas,srcpar,
     $                xout,fxtmp,dt6inv,0,wtmorg,so4org,zq)

      do itot=1,ntot_org
       do iv=1,nv
        xout(iv,itot)=xaer(iv,itot)
     $               +dt6inv(iv)*(fxaer(iv,itot)+r2*fxmid(iv,itot)
     $                                             +fxtmp(iv,itot))
       enddo
      enddo

#ifdef DIAG
      call diagchange(nv,dt6inv)
#endif
      return
      continue
      end
c-----------------------------------------------------------------------
      subroutine derivative_org(nv,temp,press,ispbcm,relhum,srcgas,
     $                      xaer,fxaer,dt,iflg,wtmorg,so4org,zq)
c
c---->calculate time derivates (fxaer)
c
c
c---->load modules
c
      use precision
      use modpar_umaer
      use modbox_umaer
      use cam_logfile,   only: iulog  !Guangxing Lin

c
c---->define accuracy and input/output arrays
c
      implicit real(kdef) (a-h,o-z), integer (i-n)
      dimension xaer(nvec,ntot_org),fxaer(nv,ntot_org),dt(nv)
      dimension temp(nv),press(nv),ispbcm(nv),relhum(nv),
     $          srcgas(nv),wtmorg(nvec),so4org(nvec,norg)!,srcmas(nv),srcpar(nv)
      dimension fxaer2(nv, ntot_org),zq(nv) !JIALEI
      target xaer
c
c---->local arrays and parameter
c
!      dimension so4radv(nv,nso4),
!     $          so4knu(nv,nso4),so4vel(nv,nso4)

      dimension orgradv(nv,norg), !JIALEI
     $          orgknu(nv,norg),orgvel(nv,norg) !JIALEI

      parameter (r8=8.d0,r3q=0.75d0,r1td=1.d0/3.d0)
c
c---->define pointer
c
      dimension orgnon(:,:),orgaer(:,:),orggas(:),so4gas(:)
      pointer orgnon,orgaer,orggas,so4gas
c
c---->set pointer
c
      orgnon=>xaer(1:nvec,1:nnon_org)
      orgaer=>xaer(1:nvec,nnon_org+1:nnon_org+naer)
      orggas=>xaer(1:nvec,ntot_org-1)
      so4gas=>xaer(1:nvec,ntot_org) 
c
c---->initialize derivatives
c
      fxaer(1:nv,1:ntot_org)=r0
      fxaer2(1:nv, 1:ntot_org) = r0
c
c---->calculate wet aerosol size (Tang and Munkelwitz (1994)),
c               Knudsen number,
c               (square of) characteristic velocity

c----------->update by JIALEI--------
      do iorg=1,norg
       inum=iorg*nmomorg
       imas=inum-1
       do iv=1,nv
      factor=r3q*so4min/(pi*so4mfrac(iv)*so4dens(iv))
!        wetmas=max(r2,so4aer(iv,imas)
!     $        /max(epsilon/pmsmerg_org(iv,iorg+1),so4aer(iv,inum)))
!     $        *so4min/so4mfrac(iv)
        wetmas=max(r2,(orgaer(iv,imas)+orgnon(iv,20-iorg)) !JIALEI check
     $        /max(epsilon/pmsmerg_org(iv,iorg+1),orgaer(iv,inum)))
     $        *(wtmorg(iv)/avno/1000)/1 !JIALEI remove so4mfrac(iv) and assume pure SOA
        orgvel (iv,iorg)=r8*boltz*temp(iv)/(pi*wetmas)
        radvorg=(r3q*wetmas/(pi*rhonon(4)))**r1td
c------------------------JIALEI-------set limit of radv-----------------
        orgradv(iv,iorg)=min(10*radvorg,(radvorg**3
     $                +factor*so4org(iv,4-iorg)
     $              /max(r1,orgaer(iv,inum)))**r1td)
       if(iorg.eq.3)orgradv(iv,iorg)=min(5.0d-9,orgradv(iv,iorg))
       if(iorg.eq.2)orgradv(iv,iorg)=min(5.0d-8,max(5.0d-9,orgradv(iv,iorg)))
       if(iorg.eq.1)orgradv(iv,iorg)=min(6.3d-7,max(5.0d-8,orgradv(iv,iorg)))
c------------------------------------------------------------
        orgknu (iv,iorg)=pathair(iv)/orgradv(iv,iorg)
       enddo
      enddo

c-----------------------------------------


c
c---->calculate fraction in accumulation mode
c
      if (iflg==1) call accumode_org(nv,orgaer,orgradv,orgnon)
c
c---->nucleation
c
      fxaer2=fxaer
!      call nucleation_org(nv,temp,relhum,orggas,fxaer,wtmorg,so4gas,zq) !JIALEI


!!      call nucleation_boundary_org(nv,temp,relhum,orggas,orgaer, orgradv, 
!!     &                         orgknu, fxaer2,wtmorg) !JIALEI

!!      do iv=1, nv
!!        if(ispbcm(iv).eq.1) then
!!         fxaer(iv, :) = fxaer2(iv, :)
!!        else 
!!          fxaer(iv, :) = fxaer(iv, :) 
!!        endif
!!      enddo

c
c---->condensation on so4 aerosol
c
      call condensorg(nv,orggas,orgaer,orgradv,orgknu,dt,fxaer) !JIALEI

c
c---->condensation on non-so4 aerosol
c
      call condensnon_org(nv,orggas,orgnon,dt,fxaer) !JIALEI
c
c---->coagulation
c
      call coagulation_org(nv,temp,orggas,orgaer,orgnon,
     $                 orgradv,orgknu,orgvel,fxaer)
c

      return
      continue
      end
c-----------------------------------------------------------------------
      subroutine nucleation_org(nv,temp,relhum,orgaer,orggas,so4gas,wtmorg,zq,dt,so4org,apin,isop,ncol,lchnk)
c
c---->calculate binary nucleation rate 
c     if -DKULMALA       according to Kulmala et al. (1998) 
c     elif -DKULNEW      according to Vehkamaeki et al. (2002)
c     elif -DFITZGERALD  according to Fitzgerald et al. (1998) 
c     else               according to classsical theory Zhao, Turco (1995)
c
c     nucleation rate   rnucnum [#particles/m3]
c       and mass rate   rnucmas [#molecules so4/m3]
c

c
c---->load modules
c
      use precision
      use modpar_umaer
      use modbox_umaer
      use cam_logfile,   only: iulog  !Guangxing Lin
      use cam_history,     only: outfld !JIALEI 05/2018

c
c---->define accuracy and input/output arrays
c
      implicit real(kdef) (a-h,o-z), integer (i-n)
      dimension temp(nv),relhum(nv),zq(nv),so4org(norg,nv)
      dimension orggas(nv),so4gas(nv),orgaer(nmomorg,norg,nv),wtmorg(nv)
      dimension apin(nv),isop(nv)
      real*8 :: nrr1(nv),nrr2(nv),nrr3(nv)
      integer :: ncol,lchnk

!      target xaer

c
c---->define limit values for nucleation rate and mass of nucleus
c
      parameter (rnucmax=1.d12)
      parameter (rmasmin=2.d0,rmasmax=300.d0)

!#elif KULNEW
c
c---->calculate number and mass of nucleated particles per time
c     following Vehkamaki (2002)
c
c
c---->define local parameter
c
      parameter (so4gmin=1.0,so4gmax=1.d18) !JIALEI change so4min=1.d7 to 1.0
      parameter (tmin=180.d0)
      parameter(rnuc  =1.7e-9)
      parameter(rden = 1.5e3)

!      dimension orggas(:),so4gas(:)
!      pointer orggas,so4gas

!      orggas=>xaer(1:nvec,ntot_org-1)
!      so4gas=>xaer(1:nvec,ntot_org)


c
c---->indices for so4gas and smallest particles
c
      inum=norg*nmomorg
      ixnum=nnon_org+inum !JIALEI check
      ixmas=ixnum-1
      igorg=ntot_org-1
      igso4=ntot_org
c
c---->start loop over parcels
c
      do iv=1,nv
        if(apin(iv).gt.(0.5*isop(iv)))then
c
c---->calculate h2so4 mole/mass fraction in critical nucleus
c
!       if (orggas(iv)>so4gmin) then
        tempnew=max(tmin,temp(iv))
        orgnew=min(so4gmax,orggas(iv))
        so4new=min(so4gmax,so4gas(iv))
        zqnew=zq(iv)
        orgnuc=1.0  !JIALEI checkso4fracnuc_f(tempnew,relhum(iv),orgnew) !JIALEI assume pure SOA

!       if (so4gas(iv)>so4gmin) then

c-----------Organic + Sulfuric nucleation----------------------
        rnucnum=3.27d-21*so4new*so4new*orgnew*exp(-(tempnew-278)/10)*1.0d-12 !JIALEI new nucleation rate

       if(rnucnum*1.0d-6.lt.0.1)then
          rnucnum=2.5*rnucnum
       else if(rnucnum*1.0d-6.ge.0.1.and.rnucnum*1.0d-6.lt.1.0)then
          rnucnum=1.79*rnucnum
       else if(rnucnum*1.0d-6.ge.1.0.and.rnucnum*1.0d-6.lt.10.0)then
          rnucnum=1.39*rnucnum
       end if

         rmas=(wtmorg(iv)/3+98.0*2/3)*1.0e-3/6.022e23   
         rnucmas = 4.0*3.1415927/3. * (rnuc)**3 * rden / rmas
         rnucmas=rnucnum*min(rmasmax,max(rmasmin,rnucmas))
         rnucmas_org=rnucmas*1/3!wtmorg(iv)/(wtmorg(iv)+98.0*2)
         rnucmas_so4=rnucmas-rnucmas_org
         rnucnum0=rnucnum

!        orgaer(1,1,iv)=orgaer(1,1,iv)+rnucmas_org*dt ! mass   (small part)  
!        orgaer(2,1,iv)=orgaer(2,1,iv)+rnucnum*dt ! number (small part)
!        orggas(iv)=max(1.0e-30,orggas(iv)-rnucmas_org*dt) ! orggas
!        so4gas(iv)=max(1.0e-30,so4gas(iv)-rnucmas_so4*dt) ! so4gas

!       orgnew=orgnew-rnucmas
!       end if

!       if (orggas(iv)>so4gmin) then
!!       tempnew=max(278.0,tempnew)
c-------------Pure organic nucleation-----------------------------
       orgnew1=orgnew/1.0d6/1.0d7 ! change unit from m-3 to 10^7 cm-3
       rnucnum1=0.04001*(orgnew1)**(1.848+0.1863/orgnew1)*1.0d6*exp(-(tempnew-278)/10)
       rnucnum2=min(2.5,2*150*zqnew*0.001366*(orgnew1)**(1.566+0.1863/orgnew1))*1.0e6*exp(-(tempnew-278)/10)
!       if(rnucnum1.gt.0.or.rnucnum2.gt.0) write(iulog,*) orgnew,rnucnum,rnucnum1,rnucnum2      
       rnucnum=rnucnum1+rnucnum2

!       if(rnucnum*1.0d-6.lt.0.1)then
!          rnucnum=2.5*rnucnum
!       else if(rnucnum*1.0d-6.ge.0.1.and.rnucnum*1.0d-6.lt.1.0)then
!          rnucnum=1.79*rnucnum
!       else if(rnucnum*1.0d-6.ge.1.0.and.rnucnum*1.0d-6.lt.10.0)then
!          rnucnum=1.39*rnucnum
!       end if
!         if(zqnew.le.12.or.tempnew.gt.230)rnucnum=1.0E-30
         rmas=wtmorg(iv)*1.0e-3/6.022e23
         rnucmas = 4.0*3.1415927/3. * (rnuc)**3 * rden / rmas
         rnucmas=rnucnum*min(rmasmax,max(rmasmin,rnucmas))

!        if(.false.)then
        if((orggas(iv)-(rnucmas+rnucmas_org)*dt).lt.0)then
           rate_org=orggas(iv)/(rnucmas+rnucmas_org)/dt
           rnucmas=rnucmas*rate_org
           rnucmas_org=rnucmas_org*rate_org
           rnucmas_so4=rnucmas_so4*rate_org
           rnucnum0=rnucnum0*rate_org
           rnucnum1=rnucnum1*rate_org
           rnucnum2=rnucnum2*rate_org
        end if
!        end if    

         nrr1(iv)=rnucnum0
         nrr2(iv)=rnucnum1
         nrr3(iv)=rnucnum2

        orgaer(1,1,iv)=orgaer(1,1,iv)+(rnucmas+rnucmas_org)*dt ! mass   (smallpart)
        orgaer(2,1,iv)=orgaer(2,1,iv)+(rnucnum0+rnucnum1+rnucnum2)*dt ! number (small part)
        orggas(iv)=max(1.0e-30,orggas(iv)-(rnucmas+rnucmas_org)*dt) ! orggas
        so4gas(iv)=max(1.0e-30,so4gas(iv)-rnucmas_so4*dt)
        so4org(1,iv)=so4org(1,iv)+rnucmas_so4*dt
!       endif 
!       endif
       else
         nrr1(iv)=0
         nrr2(iv)=0
         nrr3(iv)=0
       end if
      enddo
      
      call outfld("um_VAR3D_01",nrr1(:),ncol,lchnk) !05/2018 JIALEI
      call outfld("um_VAR3D_02",nrr2(:),ncol,lchnk) !05/2018 JIALEI
      call outfld("um_VAR3D_03",nrr3(:),ncol,lchnk) !05/2018 JIALEI

      return
      end
c-----------------------------------------------------------------------
      subroutine condensorg(nv,orggas,orgaer,orgradv,orgknu,dt,fxaer)
c
c---->calculte condensation of h2so4 
c     on pre-existing sulfat aerosol including Kelvin effect,
c     assuming a lognormal particle distribution with prescribed sigma,
c     correction for collision geometry follows Fuchs, Sutugin (1971),
c     see also Jacobson (1999), Fundamentals of Atmospheric Modeling, p. 457.
c     the accommodation coefficient eta:
c          if -DLOWACCOEF 0.04 (van Dingenen and Raes, 1991) 
c          else           1.00
c
c
c---->load modules
c
      use precision
      use modpar_umaer
      use modbox_umaer
      use cam_logfile,   only: iulog  !Guangxing Lin
c
c---->define accuracy, input/output arrays and local parameter
c
      implicit real(kdef) (a-h,o-z), integer (i-n)
      dimension orggas(nvec),orgaer(nvec,naer)
      dimension orgradv(nv,norg),orgknu(nv,norg)
      dimension fxaer(nv,ntot_org),dt(nv)

      parameter (r4=4.d0)
      parameter (a1=1.333d0,a2=0.71d0)
      parameter (b1=0.572127d0)
      parameter (eta_aer = 1.0)
#ifdef DIAG
      fxcon(1:nv)=r0
#endif

c
c---->index for so4gas
c
      igorg=ntot_org-1  !JIALEI check
c
c---->start loop over bins
c
      do iorg=1,norg
       inum=iorg*nmomorg
       imas=inum-1
       ixnum=nnon_org+inum
       ixmas=ixnum-1
c
c---->start loop over parcels
c     Euler backward used to damp overshooting of an explicit time scheme
c
       do iv=1,nv

        soaknu=b1*orgknu(iv,iorg)
c$$$        alpha=exp(-log(so4sig(iv,iorg))**2)
        beta=alphanon(4)/(r1+((a1*soaknu+a2)
     $      /(r1+soaknu)+a1*(r1-eta_aer)/eta_aer)*soaknu)
        if(orgradv(iv,iorg).le.1.0E-30)then  !JIALEI check
            orgsupsat=0
        else
        orgsupsat=max(0.0,(orggas(iv)
     $           -orgsol(iv)*exp(so4kelvin(iv)/orgradv(iv,iorg)))) !JIALEI 
        end if
        coeforg=beta*diffso4(iv)*orgradv(iv,iorg)*orgaer(iv,inum) !JIALEI keep diffusion factor of H2SO4 for org
        orgcond=coeforg/(r1+dt(iv)*coeforg)*orgsupsat

        orgpart=max(r1,orgaer(iv,imas)/max(epsilon,orgaer(iv,inum)))


c
c---->add to derivatives
c     reduce number concentration in case of evaporation
c     
        fxaer(iv,ixmas)=fxaer(iv,ixmas)+orgcond               ! mass   (bin iorg)
        fxaer(iv,ixnum)=fxaer(iv,ixnum)+min(r0,orgcond)       ! number (bin iorg)
     $                        /(orgpart*max(r1,orgpart-r4))
        fxaer(iv,igorg)=fxaer(iv,igorg)-orgcond               ! so4gas
#ifdef DIAG
        fxcon(iv)=fxcon(iv)+orgcond
#endif
       enddo
      enddo

      return
      end
c-----------------------------------------------------------------------
      subroutine condensnon_org(nv,orggas,orgnon,dt,fxaer)
c
c---->calculte condensation of h2so4 
c     on non-sulfat aerosol including Kelvin effect,
c     assuming a lognormal particle distribution with prescribed sigma,
c     correction for collision geometry follows Fuchs, Sutugin (1971),
c     see also Jacobson (1999), Fundamentals of Atmospheric Modeling, p. 457.
c     the accommodation coefficient eta:
c          if -DLOWACCOEF 0.04 (van Dingenen and Raes, 1991) 
c          else           1.00
c
c
c---->load modules
c
      use precision
      use modpar_umaer
      use modbox_umaer
      use cam_logfile,   only: iulog  !Guangxing Lin

c
c---->define accuracy, input/output arrays and local parameter
c
      implicit real(kdef) (a-h,o-z), integer (i-n)
      dimension orggas(nvec),orgnon(nvec,nnon_org)
      dimension fxaer(nv,ntot_org),dt(nv)
      parameter (r1h=0.5d0)
      parameter (ralim=10.d0)
#ifdef DIAG
        fxconn(1:nv)=r0
#endif
c
c---->index for so4gas
c
      igorg=ntot_org-1
c
c---->start loop over non-sulfate aerosol
c
      do inon=1,nnon_org-3
       do iv=1,nv
        orgsupsat=orggas(iv)-orgsol(iv)*xnonkelvin_org(iv,inon)
        if (orgsupsat>r0) then
         eta=min(r1,ralim*orgsupsat/max(r1,orggas(iv)))
         coefnon=condnon_org(iv,inon)*min(max(r1h*orgnon(iv,inon),
     $                                    eta*xnonum_org(iv,inon)),
     $                                        xnonum_org(iv,inon))
        else
         coefnon=condnon_org(iv,inon)*min(r1h*orgnon(iv,inon),
     $                                    xnonum_org(iv,inon))
        endif
        cond=max(0.0,coefnon/(r1+dt(iv)*coefnon)*orgsupsat)
c
c---->add to derivatives
c
        fxaer(iv,inon )=fxaer(iv,inon )+cond                    ! non-so4
        fxaer(iv,igorg)=fxaer(iv,igorg)-cond                    ! so4gas
#ifdef DIAG
        fxconn(iv)=fxconn(iv)+cond
#endif
       enddo
      enddo

      return
      end
c-----------------------------------------------------------------------
      subroutine coagulation_org(nv,temp,orggas,orgaer,orgnon,
     $                       orgradv,orgknu,orgvel,fxaer)
c
c---->calculate coagulation of sulfate aerosol
c     following Seinfeld (1997) table 12.1
c     using Fuchs form of Brownian coagulation coefficient
c
c
c---->load modules
c
      use precision
      use modpar_umaer
      use modbox_umaer
      use cam_logfile,   only: iulog  !Guangxing Lin

c
c---->define accuracy and input/output arrays
c
      implicit real(kdef) (a-h,o-z), integer (i-n)
      dimension temp(nv)
      dimension orggas(nvec),orgaer(nvec,naer),orgnon(nvec,nnon_org)
      dimension orgradv(nv,norg),orgknu(nv,norg),orgvel(nv,norg)
      dimension fxaer(nv,ntot_org)
c
c---->define local arrays and parameter
c
      dimension diff(nv,norg),glam(nv,norg)
      dimension orgcoag(nv,norg,norg)

      parameter (r3=3.d0,r4=4.d0,r6=6.d0,r8=8.d0,r1h=0.5d0,r3h=1.5d0)
      parameter (a1=1.249d0,a2=0.42d0,a3=-0.87d0)
#ifdef DIAG
      fxcoag(1:nv)=r0
#endif
c
c---->precalculate diffusion coefficient for particle,
c     characteristic length scale (glam)
c
      do iorg=1,norg
       do iv=1,nv
        diff(iv,iorg)=r4*boltz*temp(iv)
     $            /(r6*pi*viscos(iv)*orgradv(iv,iorg))
     $            *(r1+orgknu(iv,iorg)*(a1+a2*exp(a3/orgknu(iv,iorg))))
        xlam=diff(iv,iorg)/(pi*sqrt(orgvel(iv,iorg)))
        glam(iv,iorg)=r2/(r3*orgradv(iv,iorg)*xlam)
     $                *((orgradv(iv,iorg)+xlam)**3
     $                 -(orgradv(iv,iorg)**2+xlam**2)**r3h)
     $               -r2*orgradv(iv,iorg)
      
        glam(iv,iorg)=glam(iv,iorg)*glam(iv,iorg)
       enddo
      enddo
c
c---->coagulation coefficients including kinetic effects
c
      do jorg=1,norg
       do iorg=jorg,norg
        do iv=1,nv
         radsum=orgradv(iv,iorg)+orgradv(iv,jorg)
         diffsum=diff(iv,iorg)+diff(iv,jorg)
         orgcoag(iv,iorg,jorg)=pi*diffsum*radsum
     $        /(radsum/(radsum+sqrt(glam(iv,iorg)+glam(iv,jorg)))
     $         +diffsum/(sqrt(orgvel(iv,iorg)+orgvel(iv,jorg))*radsum))

        enddo
       enddo
      enddo

c
c---->autocoagulation
c
      do iorg=1,norg
       inum=iorg*nmomorg
       imas=inum-1
       ixnum=nnon_org+inum
       ixmas=ixnum-1
       do iv=1,nv
        dnumcoag=r1h*orgcoag(iv,iorg,iorg)*orgaer(iv,inum)
     $                                    *orgaer(iv,inum)
c
c---->add to derivatives
c
        fxaer(iv,ixnum)=fxaer(iv,ixnum)-dnumcoag         ! number (bin iorg)

       
        enddo
      enddo
c
c---->(non-auto) coagulation
c
      do jorg=1,norg
       jnum=jorg*nmomorg
       jmas=jnum-1
       jxnum=nnon_org+jnum
       jxmas=jxnum-1
       do iorg=jorg+1,norg
        inum=iorg*nmomorg
        imas=inum-1
        ixnum=nnon_org+iorg*nmomorg
        ixmas=ixnum-1
        do iv=1,nv
         dmascoag=orgcoag(iv,iorg,jorg)*orgaer(iv,imas)*orgaer(iv,jnum) !JIALEI check
         dnumcoag=orgcoag(iv,iorg,jorg)*orgaer(iv,inum)*orgaer(iv,jnum)
     
         dmascoag_non=dmascoag/max(epsilon,orgaer(iv,imas))*orgnon(iv,iorg+16) !JIALEI add for SOAOM mass change
c
c---->add to derivatives
c
         fxaer(iv,jxmas)=fxaer(iv,jxmas)+dmascoag        ! mas    (bin jorg)
         fxaer(iv,ixmas)=fxaer(iv,ixmas)-dmascoag        ! mass   (bin iorg)
         fxaer(iv,ixnum)=fxaer(iv,ixnum)-dnumcoag        ! number (bin iorg)
c---------------JIALEI add for SOAOM mass change
         fxaer(iv,20-iorg)=fxaer(iv,20-iorg)-dmascoag_non
         fxaer(iv,20-jorg)=fxaer(iv,20-jorg)+dmascoag_non

         
         
        enddo
       enddo
      enddo

  
c
c---->calculate coagulation coefficients
c     for coagulation between SOA aerosol and non-SOA aerosol
c
      igorg=ntot_org-1


      do inon=1,nnon_org-3
       if(inon.ne.4)then
       if(inon.ge.1.and.inon.le.3)then
       xvelnon=r8*boltz/(pi*pmssrc(inon))
       end if
       if(inon.ge.4.and.inon.le.8)then
       xvelnon=r8*boltz/(pi*pmsnon(inon-3))
       end if
       if(inon.ge.9.and.inon.le.nnon_org-3)then
       xvelnon=r8*boltz/(pi*pmsnon(inon+5))
       end if

       do iorg=1,norg
        inum=iorg*nmomorg
        imas=inum-1
        ixnum=nnon_org+inum
        ixmas=ixnum-1

        do iv=1,nv
         if(inon.ge.1.and.inon.le.3)then
         factor=radvso4/rwetnon_org(iv,inon)/100
         end if
         if(inon.ge.4.and.inon.le.8)then        
         factor=radvnon(inon-3)/rwetnon_org(iv,inon)
         end if
         if(inon.ge.9.and.inon.le.nnon_org-3)then
         factor=radvnon(inon+5)/rwetnon_org(iv,inon)
         end if

         if(iv.eq.1.and.iorg.eq.1)then
!           write(iulog,*) inon,factor
         end if

         velnon=xvelnon*temp(iv)
         xlamnon=diffnon_org(iv,inon)/(pi*factor*sqrt(velnon))
         glamnon=r2/(r3*rwetnon_org(iv,inon)*xlam)
     $               *((rwetnon_org(iv,inon)+xlam)**3
     $                -(rwetnon_org(iv,inon)**2+xlam**2)**r3h)
     $              -r2*rwetnon_org(iv,inon)
         glamnon=glamnon*glamnon




         radsum=orgradv(iv,iorg)+rwetnon_org(iv,inon)
         diffsum=diff(iv,iorg)+diffnon_org(iv,inon)
         orgnoncoag=pi*diffsum*radsum
     $        /(radsum/(radsum+sqrt(glam(iv,iorg)+glamnon))
     $         +diffsum/(radsum*(orgvel(iv,iorg)+velnon)))

c
c---->coagulation of SOA aerosol and non-sulfate aerosol
c
c$$$         xnum=min(max(r1,r1h*so4non(iv,inon)),xnonum(iv,inon))
         xnum=xnonum_org(iv,inon)
         dmascoag=orgnoncoag*orgaer(iv,imas)*xnum
         dnumcoag=orgnoncoag*orgaer(iv,inum)*xnum
         
!         dmascoag_non=dmascoag/max(epsilon,orgaer(iv,imas))*orgnon(iv,20-iorg) !JIALEI add for SOAOM
         dmascoag_non=dnumcoag/max(epsilon,orgaer(iv,inum))*orgnon(iv,20-iorg) !JIALEI add for SOAOM

         
c
c---->add to derivatives
c     assume immediate evaporation if supsat<0
c
         orgsupsat=orggas(iv)-orgsol(iv)*xnonkelvin_org(iv,inon)
 


!         if (orgsupsat>r0) then !JIALEI check assume no evaporation
          fxaer(iv,inon) =fxaer(iv,inon) +dmascoag+dmascoag_non     ! mas    (bin inon)
!         else
!          fxaer(iv,igorg)=fxaer(iv,igorg)+dmascoag !+dmascoag_non
!         endif
         fxaer(iv,20-iorg)=fxaer(iv,20-iorg)-dmascoag_non  !JIALEI for SOAOM change
         fxaer(iv,ixmas)=fxaer(iv,ixmas)-dmascoag        ! mass   (bin iorg)
         fxaer(iv,ixnum)=fxaer(iv,ixnum)-dnumcoag        ! number (bin iorg)
    
     
        enddo
       enddo
       endif
      enddo


1919  continue
      return
      end
c-----------------------------------------------------------------------

      subroutine settling_org(nv,orgaer,orgradv,fxaer,orgnon) !JIALEI add orgnon
c
c---->gravitational settling
c     Stokes law with Cunningham slip correction factor
c     see Seinfeld (1997) eq. 8.34 and eq. 8.42
c
c
c---->load modules
c
      use precision
      use modpar_umaer
      use modbox_umaer
c
c---->define accuracy, input/output arrays
c
      implicit real(kdef) (a-h,o-z), integer (i-n)
      dimension orgaer(nvec,naer),orgradv(nv,norg),orgnon(nvec,nnon_org)
      dimension fxaer(nv,ntot_org)
c
c---->define local parameter
c
      parameter (r1h=0.5d0,r5h=2.5d0)
      parameter (a1=2.d0/9.d0)
      parameter (b1=1.257d0,b2=0.4d0,b3=1.1d0)
      parameter (r3=3.d0)
      parameter (dzinv=r1/65.d0)
#ifdef DIAG        
      fxgrvn(1:nv)=r0
      fxgrvm(1:nv)=r0
#endif
c
c---->start loop over bins
c
      do iorg=1,norg
       inum=iorg*nmomorg
       imas=inum-1
       ixnum=nnon_org+inum
       ixmas=ixnum-1
c
c---->start loop over parcels
c
       do iv=1,nv
        xlnsg=log(so4sig(iv,iorg))
        xlnsg2=xlnsg*xlnsg
c
c---->calculate terminal fall velocities for mass and number
c
        radm=orgradv(iv,iorg)*exp(r5h*xlnsg2)
        orgknum=pathair(iv)/radm
        wmas=a1*grav/viscos(iv)*radm*radm*rhonon(4)
     $      *(r1+orgknum*(b1+b2*exp(-b3/orgknum)))

        radn=orgradv(iv,iorg)*exp(-r1h*xlnsg2)
        orgknun=pathair(iv)/radn
        wnum=a1*grav/viscos(iv)*radn*radn*rhonon(4)
     $      *(r1+orgknun*(b1+b2*exp(-b3/orgknun)))
c
c---->add to derivatives
c
        fxaer(iv,ixmas)=fxaer(iv,ixmas)-orgaer(iv,imas)      ! mass  (bin iorg)
     $                                 *wmas*dzinv
        fxaer(iv,ixnum)=fxaer(iv,ixnum)-orgaer(iv,inum)      ! nuber (bin iorg)
     $                                 *wnum*dzinv
        fxaer(iv,iorg+16)=fxaer(iv,iorg+16)-orgnon(iv,iorg+16) !mass of SOAOM
     $                                 *wmas*dzinv

       enddo
      enddo

      return
      end
c-----------------------------------------------------------------------
      subroutine merge_org(ivec,xaer,wtmorg)!!,so4non_org)
c
c---->merging
c     move particles greater radmerg in diameter to next mode
c
c
c---->load modules
c
      use precision
      use modpar_umaer
      use modbox_umaer
      use cam_logfile,   only: iulog  !Guangxing Lin

c
c---->define accuracy and input/output arrays
c
      implicit real(kdef) (a-h,o-z), integer (i-n)
      dimension xaer(nvec,ntot_org+norg),wtmorg(nvec)!!,so4non_org(nvec,nnon_org)
      target xaer
c
c---->define local arrays and parameter
c
      dimension orgradv(norg)

      parameter (r8=8.d0,r1h=0.5d0,r3h=1.5d0,
     $           r3q=0.75d0,r1td=1.d0/3.d0)
c
c---->define pointer
c
      dimension orgaer(:)
      pointer orgaer
      dimension orgnon(:)
      pointer orgnon
c
c---->set pointer
c


      orgaer=>xaer(ivec,nnon_org+1:nnon_org+naer)
      orgnon=>xaer(ivec,1:nnon_org)


c
c---->calculate wet aerosol size (Tang and Munkelwitz (1994)),
c
      factor=r3q*so4min/(pi*so4mfrac(ivec)*so4dens(ivec))
      do iorg=1,norg
       inum=iorg*nmomorg
       imas=inum-1
       wetmas=max(r2,(orgaer(imas)+orgnon(20-iorg)) !JIALEI check
     $       /max(epsilon/pmsmerg_org(ivec,iorg+1),orgaer(inum)))
     $       *(wtmorg(ivec)/avno/1000)/1 !JIALEI assume pure SOA
!     $       *(200/avno/1000)/1 !JIALEI assume pure SOA
       radvorg=(r3q*wetmas/(pi*rhonon(4)))**r1td
       orgradv(iorg)=min(10*radvorg,(radvorg**3
     $                +factor*xaer(ivec,4-iorg+ntot_org)
     $              /max(r1,orgaer(inum)))**r1td)
!      write(iulog,*) ivec,iorg,r10,orgradv(iorg),radvorg,factor,xaer(ivec,4-iorg+ntot_org),orgaer(inum)
      enddo
c
c---->start loop over aerosol bins
c
      do iorg=2,norg
       xlnsg=log(so4sig(ivec,iorg))
       drgvdrv=exp(r3h*xlnsg*xlnsg)
       drgdrv=r1/drgvdrv
       radg=orgradv(iorg)*drgdrv

       if (radg>radgmerg(1,iorg)) then
        inum=iorg*nmomorg
        imas=inum-1
        inmerg=inum-nmomorg
        immerg=inmerg-1
c
c---->calculate change in mass and number due to merging
c
        radgv=orgradv(iorg)*drgvdrv
        kmergn=1
        kmergv=1
        do k=1,nmerg
         if (radgmerg(k,iorg)<radg)  kmergn=k
         if (radgmerg(k,iorg)<radgv) kmergv=k
        enddo

        k=kmergv
        kp1=k+1
        dmmerg=(fracmerg(k  )*(radgmerg(kp1,iorg)-radgv)
     $         -fracmerg(kp1)*(radgmerg(k  ,iorg)-radgv))
     $        /(radgmerg(kp1,iorg)-radgmerg(k,iorg))


        dmmerg=min(r1,dmmerg)*orgaer(imas)
        k=kmergn
        kp1=k+1
        dnmerg=(fracmerg(k  )*(radgmerg(kp1,iorg)-radg)
     $         -fracmerg(kp1)*(radgmerg(k  ,iorg)-radg))
     $        /(radgmerg(kp1,iorg)-radgmerg(k,iorg))
        dnmerg=min(r1,dnmerg)*orgaer(inum)
 
        dmmerg_non=dmmerg/max(epsilon,orgaer(imas))*orgnon(20-iorg) !JIALEI add for SOAOM
        dmmerg_so4=dmmerg/max(epsilon,orgaer(imas))*xaer(ivec,4-iorg+ntot_org) !JIALEI add for SO4OM

c
c---->add changes (simple Euler backward)
c
        orgaer(imas)  =orgaer(imas)  -dmmerg
        orgaer(immerg)=orgaer(immerg)+dmmerg
      
        orgnon(20-iorg)=orgnon(20-iorg) -dmmerg_non !JIALEI
        orgnon(21-iorg)=orgnon(21-iorg) +dmmerg_non !JIALEI for SOAOM !JIALEI check

        xaer(ivec,4-iorg+ntot_org)=xaer(ivec,4-iorg+ntot_org)-dmmerg_so4 !JIALEI
        xaer(ivec,5-iorg+ntot_org)=xaer(ivec,5-iorg+ntot_org)+dmmerg_so4 !JIALEI for SO4OM 

                                               
        if (orgaer(imas)==r0) dnmerg=orgaer(inum)
        orgaer(inum)  =orgaer(inum)  -dnmerg
        orgaer(inmerg)=orgaer(inmerg)+dnmerg
       endif
      enddo

      return
      end
c-----------------------------------------------------------------------
      subroutine accumode_org(nv,orgaer,orgradv,orgnon)
c
c---->calculate fraction of particles in accumulation mode
c
c
c---->load modules
c
      use precision
      use modpar_umaer
      use modbox_umaer
c
c---->define accuracy and input/output arrays
c
      implicit real(kdef) (a-h,o-z), integer (i-n)
      dimension orgaer(nvec,naer),orgradv(nv,norg)
      dimension orgnon(nvec,nnon_org)
c
c---->define local arrays and parameter
c
      parameter (r3h=1.5d0,r5=5.d0)
c
c---->add for non-sulfate aerosol that can act as CCN
#ifndef NONON
      parameter (r3q=0.75d0,r1td=1.d0/3.d0)
      parameter (fso4crit=10.d0)
      dimension xorgnon(nv,nnon_org),forgnon(nv,nnon_org)
      dimension rso4min(nv)
#endif
c
c---->sulfate aerosol
c
      do iorg=1,norg
       inum=iorg*nmomorg
       do iv=1,nv
        radg=orgradv(iv,iorg)*exp(-r3h*log(so4sig(iv,iorg))**2)
        if (r5*so4sig(iv,iorg)*radg>radaccu(1)) then
         if (radg<racculim) then
          do iaccu=1,naccu
           if (radgaccu(iaccu,iorg,1)>radg) exit
          enddo
          k=iaccu
          km1=k-1
          fracsrc_org(iv,iorg)=orgaer(iv,inum)
     $         *(fracaccu(km1,iorg)*(radgaccu(k  ,iorg,1)-radg)
     $          -fracaccu(k  ,iorg)*(radgaccu(km1,iorg,1)-radg))
     $         /(radgaccu(k  ,iorg,1)-radgaccu(km1,iorg,1))
         else
          do iaccu=1,naccu
           if (radgaccu(iaccu,iorg,2)<=radg) exit
          enddo
          k=iaccu
          km1=k-1
          fracsrc_org(iv,iorg)=orgaer(iv,inum)
     $         *(fracaccu(km1,iorg)*(radgaccu(k  ,iorg,2)-radg)
     $          -fracaccu(k  ,iorg)*(radgaccu(km1,iorg,2)-radg))
     $         /(radgaccu(k  ,iorg,2)-radgaccu(km1,iorg,2))
         endif
        else
         fracsrc_org(iv,iorg)=r0
        endif
       enddo
      enddo
      

      return
      end
c-----------------------------------------------------------------------
      function orgsat_f(temp)  !JIALEI 10/2017
c---->number of ELVOC molecules in saturation represented bu pinic acid
c
c---->load modules
c
      use precision
      use modpar_umaer
c
c---->define accuracy and local parameter
c
      implicit real(kdef) (a-h,o-z), integer (i-n)
c
c---->Bilde et al. (2001), EST(35)3344-3349
c
      parameter (a1=5691.7d0,
     $           a2=14.73d0)
      orgsat_f=10**(-a1/temp+a2)/(boltz*temp)
      return
      end
c----------------------------------------------------------------------
      function rnucnum_org_f(temp,relhum,orggas,orgfrac)
c
c---->calculate nucleation rate: rnucnum [#particles/m3]
c
c
c---->load modules
c
      use precision
      use modpar_umaer
c
c---->define accuracy
c
      implicit real(kdef) (a-h,o-z), integer (i-n)
c
c---->Vehkamaeki et al. (2002)
c
      parameter (a1=0.14309d0,
     $           a2=2.21956d0,
     $           a3=2.73911d-2,
     $           a4=7.22811d-5,
     $           a5=5.91822d0)
      parameter (b1=0.117489d0,
     $           b2=0.462532d0,
     $           b3=1.18059d-2,
     $           b4=4.04196d-5,
     $           b5=15.7963d0)
      parameter (c1=0.215554d0,
     $           c2=8.10269d-2,
     $           c3=1.43581d-3,
     $           c4=4.7758d-6,
     $           c5=2.91297d0)
      parameter (d1=3.58856d0,
     $           d2=4.9508d-2,
     $           d3=2.1382d-4,
     $           d4=3.10801d-7,
     $           d5=2.93333d-2)
      parameter (e1=1.14598d0,
     $           e2=6.00796d-1,
     $           e3=8.64245d-3,
     $           e4=2.28947d-5,
     $           e5=8.44985)
      parameter (f1=2.15855d0,
     $           f2=8.08121d-2,
     $           f3=4.07382d-4,
     $           f4=4.01947d-7,
     $           f5=7.21326d-1)
      parameter (g1=1.6241d0,
     $           g2=1.60106d-2,
     $           g3=3.77124d-5,
     $           g4=3.21794d-8,
     $           g5=1.13255d-2)
      parameter (h1=9.71682d0,
     $           h2=1.15048d-1,
     $           h3=1.57098d-4,
     $           h4=4.00914d-7,
     $           h5=0.71186d0)
      parameter (p1=1.05611d0,
     $           p2=9.03378d-3,
     $           p3=1.98417d-5,
     $           p4=2.46048d-8,
     $           p5=5.79087d-2)
      parameter (q1=0.148712d0,
     $           q2=2.83508d-3,
     $           q3=9.24619d-6,
     $           q4=5.00427d-9,
     $           q5=1.27081d-2)
      parameter (convfac=1.d-6,convinv=1.d6)
      parameter (expmax=46.d0)

      sfracinv=r1/orgfrac
      temp2=temp*temp
      temp3=temp*temp2

      a=+a1+a2*temp-a3*temp2+a4*temp3+a5*sfracinv
      b=+b1+b2*temp-b3*temp2+b4*temp3+b5*sfracinv
      c=-c1-c2*temp+c3*temp2-c4*temp3-c5*sfracinv
      d=-d1+d2*temp-d3*temp2+d4*temp3-d5*sfracinv
      e=+e1-e2*temp+e3*temp2-e4*temp3-e5*sfracinv
      f=+f1+f2*temp-f3*temp2-f4*temp3+f5*sfracinv
      g=+g1-g2*temp+g3*temp2+g4*temp3-g5*sfracinv
      h=+h1-h2*temp+h3*temp2+h4*temp3+h5*sfracinv
      p=-p1+p2*temp-p3*temp2+p4*temp3-p5*sfracinv
      q=-q1+q2*temp-q3*temp2+q4*temp3-q5*sfracinv


      rhln=log(relhum)
      rhln2=rhln*rhln
      rhln3=rhln*rhln2
      orgln=max(r0,log(orggas*convfac))
      orgln2=orgln*orgln
      orgln3=orgln*orgln2

      expon=min(expmax,a+b*rhln+c*rhln2+d*rhln3
     $                +e*orgln+f*rhln*orgln+g*rhln2*orgln
     $                +h*orgln2+p*rhln*orgln2+q*orgln3)
      rnucnum_org_f=convinv*exp(expon)

      return
      end

c-----------------------------------------------------------------------
      function rnucmas_org_f(temp,relhum,orggas,orgfrac)
c
c---->calculate total number of molecules in critical cluster [#molec/part]
c     Vehkamaeki et al. (2002)
c------>update by JIALEI 10/2017 for nucleation version
c---->load modules
c
      use precision
      use modpar_umaer
      use cam_logfile,   only: iulog  !Guangxing Lin
c
c---->define accuracy and local parameter
c
      implicit real(kdef) (a-h,o-z), integer (i-n)

      parameter (a1=2.95413d-3,
     $           a2=9.76834d-2,
     $           a3=1.02485d-3,
     $           a4=2.18646d-6,
     $           a5=1.01717d-1)
      parameter (b1=2.05064d-3,
     $           b2=7.58504d-3,
     $           b3=1.92654d-4,
     $           b4=6.70430d-7,
     $           b5=2.55774d-1)
      parameter (c1=3.22308d-3,
     $           c2=8.52637d-4,
     $           c3=1.54757d-5,
     $           c4=5.66661d-8,
     $           c5=3.38444d-2)
      parameter (d1=4.74323d-2,
     $           d2=6.25104d-4,
     $           d3=2.65066d-6,
     $           d4=3.67471d-9,
     $           d5=2.67251d-4)
      parameter (e1=1.25211d-2,
     $           e2=5.80655d-3,
     $           e3=1.01674d-4,
     $           e4=2.88195d-7,
     $           e5=9.42243d-2)
      parameter (f1=3.85460d-2,
     $           f2=6.72316d-4,
     $           f3=2.60288d-6,
     $           f4=1.19416d-8,
     $           f5=8.51515d-3)
      parameter (g1=1.83749d-2,
     $           g2=1.72072d-4,
     $           g3=3.71766d-7,
     $           g4=5.14875d-10,
     $           g5=2.68660d-4)
      parameter (h1=6.19974d-2,
     $           h2=9.06958d-4,
     $           h3=9.11728d-7,
     $           h4=5.36796d-9,
     $           h5=7.74234d-3)
      parameter (p1=1.21827d-2,
     $           p2=1.06650d-4,
     $           p3=2.53460d-7,
     $           p4=3.63519d-10,
     $           p5=6.10065d-4)
      parameter (q1=3.20184d-4,
     $           q2=1.74762d-5,
     $           q3=6.06504d-8,
     $           q4=1.42177d-11,
     $           q5=1.35751d-4)


      parameter (so4gmin=1.0,so4gmax=1.d17)  !JIALEI change min=1.d10 to 1.d4
      parameter (convfac=1.d-6)

      sfracinv=r1/orgfrac
      temp2=temp*temp
      temp3=temp*temp2

      a=-a1-a2*temp+a3*temp2-a4*temp3-a5*sfracinv
      b=-b1-b2*temp+b3*temp2-b4*temp3-b5*sfracinv
      c=+c1+c2*temp-c3*temp2+c4*temp3+c5*sfracinv
      d=+d1-d2*temp+d3*temp2-d4*temp3-d5*sfracinv
      e=-e1+e2*temp-e3*temp2+e4*temp3+e5*sfracinv
      f=-f1-f2*temp+f3*temp2+f4*temp3-f5*sfracinv
      g=-g1+g2*temp-g3*temp2-g4*temp3+g5*sfracinv
      h=-h1+h2*temp-h3*temp2-h4*temp3-h5*sfracinv
      p=+p1-p2*temp+p3*temp2-p4*temp3+p5*sfracinv
      q=+q1-q2*temp+q3*temp2-q4*temp3+q5*sfracinv


      rhln=log(relhum)
      rhln2=rhln*rhln
      rhln3=rhln*rhln2
      orgln=log(min(so4gmax,max(so4gmin,orggas))*convfac)
      orgln2=orgln*orgln
      orgln3=orgln*orgln2

      rnucmas_org_f=exp(a+b*rhln+c*rhln2+d*rhln3
     $             +e*orgln+f*rhln*orgln+g*rhln2*orgln
     $             +h*orgln2+p*rhln*orgln2+q*orgln3)

      return
      end
c-----------------------------------------------------------------------
      function diffso4_org_f(temp,press)
c
c---->calculate diffusion coefficient for h2so4 gas molec in air
c     following Chapman and Cowling (1970), Davis (1983)
c     eq. 8.13 Seinfeld (1997) / eq. 17.17 Jacobson (1998)
c
c
c---->load modules
c
      use precision
      use modpar_umaer
c
c---->define accuracy and local parameter
c
      implicit real(kdef) (a-h,o-z), integer (i-n)
      parameter (r3=3.d0,r8=8.d0)
      parameter (dso4=5.1209323d-10)

      diffso4_org_f=r3*boltz*temp/(r8*dso4*dso4*press)
     $         *sqrt(rgas*temp*(drymol+so4mol)/(r2*pi*drymol*so4mol))

      return
      end
c-----------------------------------------------------------------------

      subroutine nucleation_boundary_org(nv,temp,relhum,orggas,orgaer, 
     &                  orgradv, orgknu, fxaer, wtmorg)
c---------------------------------------------------------------------
c     Purpose: the activation nucleation parameterization (Kulmala et al., 2006, ACP)
c              the nucleation rate is propotional to H2SO4.
c     by Minghuai Wang (minghuai@umich.edu), 2007-09
c
c----------------------------------------------------------------------
c     nucleation rate   rnucnum [#particles/m3]
c       and mass rate   rnucmas [#molecules so4/m3]

c
c---->load modules
c
      use precision
      use modpar_umaer
      use modbox_umaer
c
c---->define accuracy and input/output arrays
c
      implicit real(kdef) (a-h,o-z), integer (i-n)
      dimension temp(nv),relhum(nv)
      dimension orggas(nvec),fxaer(nv,ntot_org)
      dimension orgaer(nvec, naer)
      dimension orgradv(nv, norg), orgknu(nv, norg),wtmorg(nvec) !JIALEI add wtmorg
c
c---->define the rate constant for activation nucleation
c     (Kulmala et al., 2006, ACP; Sihto et al., 2006, ACP)
c     (Kerminen and Kulmala, 2002, J. Aerosol Sci.)
c
      parameter (coe_a = 1.0e-6 )   ! /s
      parameter (coe_k = 0.5e-18 )  ! m^3/s
      parameter (gama = 0.23 )
      parameter (eta_aer = 1.0)  ! mass accomodation coefficient set to be unity
      parameter (a1 = 1.333, a2 = 0.71)
      parameter (r8 = 8.0)

c
c---->define density, radius, mass for cluster, and H2SO4 molecules
c
c     radius (meter)
      parameter(rcrit =1.0e-9)
      parameter(rnuc  =3.0e-9)

c     density (kg/m^3)      
      parameter(rden = 1.5e3) !JIALEI change from 1.7e3 to 1.5e3 refer to bOC
c     molecule weight for H2SO4 (g)
!      parameter (wso4 = 98 ) !JIALEI check
c     mass for sinlge H2SO4 molecule (kg)
!      parameter(rmas = wso4*1.0e-3/6.022e23) !JIALEI check

c
c---->define limit values for nucleation rate and mass of nucleus
c
      parameter (rnucmax=1.d12)
      parameter (rmasmin=2.d0,rmasmax=300.d0)
c
c---->calculate number and mass of nucleated particles per time
c
c---->define local parameter
c
      parameter (so4gmin=1.d0,so4gmax=1.d18) !JIALEI change 1.d7 to 1.d4
      parameter (tmin=195.d0)

c
c---->indices for so4gas and smallest particles
c
      inum=norg*nmomorg
      ixnum=nnon_org+inum
      ixmas=ixnum-1
      igorg=ntot_org-1
c
c---->start loop over parcels
c
      do iv=1,nv
c
c---->calculate h2so4 mole/mass fraction in critical nucleus
c
       if (orggas(iv)>so4gmin) then
        tempnew=max(tmin,temp(iv))
        orgnew=min(so4gmax,orggas(iv))
cc        so4nuc=so4fracnuc_f(tempnew,relhum(iv),so4new)
c$$$        so4mnuc=so4nuc*so4mol
c$$$     $        /(so4nuc*so4mol+(r1-so4nuc)*h2omol)
c$$$        so4mnuc=min(r1,max(r0,so4mnuc))
c
c---->calculate nucleation rate for 1 nm clusters: rnucnum [#particles/m3]
c------->activation nucleation

        rnucnum = coe_a * orgnew
c-------> kinetic nucleation
c        rnucnum = coe_k * so4new * so4new

c---->calculate nucleation rate for 3 nm clusters
c     Kerminen and Kulmala, 2002, Journal of Aerosol Science
c
c----> first, calcualte the condensation sink: CS
        cs = 0.0
        do iorg = 1, norg
         inum2 = iorg * nmomorg
         xknu=orgknu(iv,iorg)
         beta=alphanon(4)/(r1+((a1*xknu+a2)
     $       /(r1+xknu)+a1*(r1-eta_aer)/eta_aer)*xknu)
         cs = cs + beta * orgradv(iv, iorg) * orgaer(iv, inum2)
        end do

       do inon = 1, 3
         xknu = pathair(iv)/rwetnon_org(iv, inon)
         beta = so4alpha(iv,inon) /(r1+((a1*xknu+a2)/(r1+xknu)
     $                       +a1*(r1-eta_aer)/eta_aer)*xknu)
         cs = cs + beta * rwetnon_org(iv,inon) * xnonum_org(iv, inon)
        end do
        
       do inon = 4, 8
         xknu = pathair(iv)/rwetnon_org(iv, inon)
         beta = alphanon(inon-3) /(r1+((a1*xknu+a2)/(r1+xknu)
     $                       +a1*(r1-eta_aer)/eta_aer)*xknu)
         cs = cs + beta * rwetnon_org(iv,inon) * xnonum_org(iv, inon)
        end do

       do inon = 9,16
         xknu = pathair(iv)/rwetnon_org(iv, inon)
         beta = alphanon(inon+5) /(r1+((a1*xknu+a2)/(r1+xknu)
     $                       +a1*(r1-eta_aer)/eta_aer)*xknu)
         cs = cs + beta * rwetnon_org(iv,inon) * xnonum_org(iv, inon)
        end do


c---->second, calculate the growth rate: GR
        xvel = sqrt(r8*boltz*temp(iv)/(pi*rmas))
        gr = 3.0*1.0e-9 * xvel * wtmorg(iv) * (orgnew * 1.0e-6) / rden
c        gr = gr * 2.5 * 4

        eta = gama * cs / gr
        fscale = exp ( eta * ( 1/(rnuc*1.0e9) - 1/(rcrit*1.0e9)))
c        write(0, *) ' path, diffso4/(4*pi)', pathso4(iv), diffso4(iv)/(4*3.141659),
c     &    diffso4(iv)/(4*3.141659)/pathso4(iv)
c        write(0, 1005) ' the factor eta', fscale, eta, cs, gr, so4new/1.0e6, rnucnum/1.0e6
1005     format(a15, 6e15.5)

        rnucnum = rnucnum * fscale
c
c---->calculate the number of so4 molecules in the cluster
c
c        rnucmas=so4nuc*rnucmas_f(tempnew,relhum(iv),so4new,so4nuc)
c        rnucmas=rnucnum*min(rmasmax,max(rmasmin,rnucmas))

        rmas=wtmorg(iv)*1.0e-3/6.022e23
        rnucmas = 4.0*3.1415927/3. * (rnuc)**3 * rden / rmas
        rnucmas=rnucnum*min(rmasmax,max(rmasmin,rnucmas))
c        write (0, *) 'rnucmas', rnucmas, rnucnum/1.0e6

c         rnucmas = 1.0
c         rnucmas = rnucnum * rnucmas

c
c---->add to derivatives
c
        fxaer(iv,ixmas)=fxaer(iv,ixmas)+rnucmas ! mass   (small part)
        fxaer(iv,ixnum)=fxaer(iv,ixnum)+rnucnum ! number (small part)
        fxaer(iv,igorg)=fxaer(iv,igorg)-rnucmas ! so4gas
#ifdef DIAG
        fxnucn(iv)=rnucnum
        fxnucm(iv)=rnucmas
       else
        fxnucn(iv)=r0
        fxnucm(iv)=r0
#endif
       endif
      enddo

c      stop

      return
      end
c--------------------------------------------------------------------
      function orgsol_f(temp,so4sat,so4frac)
c
c---->calculate number concentration over the solution for h2so4
c     Taleb et al. (1996), JGR (101) pp 25967-25977
c     based on Zeleznik (1991) data
c
c
c---->load modules
c
      use precision
      use modpar_umaer
      use cam_logfile,   only: iulog  !Guangxing Lin

c
c---->define accuracy and local parameter
c
      implicit real(kdef) (a-h,o-z), integer (i-n)
      parameter (r10=10.0d0)
      parameter (a1=5672.d0,
     $           a2=4074000.d0,
     $           a3=4.421d8,
     $           b12=1.8975332d0)
      parameter (tempmin=190.d0,tempmax=298.d0)

      tempnew=max(tempmin,min(tempmax,temp))
      h2ofrac=r1-so4frac
      a12=a1-a2/tempnew+a3/(tempnew*tempnew)
      so4gam=r10**(a12*h2ofrac*h2ofrac
     $              /(tempnew*(h2ofrac+b12*so4frac)**2))
      orgsol_f=so4gam*so4frac*so4sat

      return
      end

